@@ -27,10 +27,8 @@
 #include "libknot/yparser/ypformat.h"
 #include "libknot/yparser/yptrafo.h"
 #include "contrib/files.h"
-#include "contrib/mempattern.h"
 #include "contrib/sockaddr.h"
 #include "contrib/string.h"
-#include "contrib/ucw/mempool.h"
 
 // The active configuration.
 conf_t *s_conf;
@@ -139,12 +137,16 @@ static void init_cache(
 	conf->cache.ctl_timeout = conf_int(&val) * 1000;
 
 	conf->cache.srv_nsid = conf_get(conf, C_SRV, C_NSID);
+
+	val = conf_get(conf, C_SRV, C_ECS);
+	conf->cache.use_ecs = conf_bool(&val);
 }
 
 int conf_new(
 	conf_t **conf,
 	const yp_item_t *schema,
 	const char *db_dir,
+	size_t max_conf_size,
 	conf_flag_t flags)
 {
 	if (conf == NULL) {
@@ -163,16 +165,8 @@ int conf_new(
 		goto new_error;
 	}
 
-	// Initialize a config mempool.
-	out->mm = malloc(sizeof(knot_mm_t));
-	if (out->mm == NULL) {
-		ret = KNOT_ENOMEM;
-		goto new_error;
-	}
-	mm_ctx_mempool(out->mm, MM_DEFAULT_BLKSIZE);
-
 	// Initialize query modules list.
-	out->query_modules = mm_alloc(out->mm, sizeof(list_t));
+	out->query_modules = malloc(sizeof(list_t));
 	if (out->query_modules == NULL) {
 		ret = KNOT_ENOMEM;
 		goto new_error;
@@ -182,7 +176,7 @@ int conf_new(
 	// Set the DB api.
 	out->api = knot_db_lmdb_api();
 	struct knot_db_lmdb_opts lmdb_opts = KNOT_DB_LMDB_OPTS_INITIALIZER;
-	lmdb_opts.mapsize = (size_t)CONF_MAPSIZE * 1024 * 1024;
+	lmdb_opts.mapsize = max_conf_size;
 	lmdb_opts.maxreaders = CONF_MAX_DB_READERS;
 	lmdb_opts.flags.env = KNOT_DB_LMDB_NOTLS;
 
@@ -198,7 +192,7 @@ int conf_new(
 			goto new_error;
 		}
 
-		ret = out->api->init(&out->db, out->mm, &lmdb_opts);
+		ret = out->api->init(&out->db, NULL, &lmdb_opts);
 
 		// Remove the database to ensure it is temporary.
 		if (!remove_path(lmdb_opts.path)) {
@@ -214,7 +208,7 @@ int conf_new(
 			lmdb_opts.flags.env |= KNOT_DB_LMDB_RDONLY;
 		}
 
-		ret = out->api->init(&out->db, out->mm, &lmdb_opts);
+		ret = out->api->init(&out->db, NULL, &lmdb_opts);
 	}
 	if (ret != KNOT_EOK) {
 		goto new_error;
@@ -292,11 +286,10 @@ int conf_clone(
 
 	// Set shared items.
 	out->api = s_conf->api;
-	out->mm = s_conf->mm;
 	out->db = s_conf->db;
 
 	// Initialize query modules list.
-	out->query_modules = mm_alloc(out->mm, sizeof(list_t));
+	out->query_modules = malloc(sizeof(list_t));
 	if (out->query_modules == NULL) {
 		yp_schema_free(out->schema);
 		free(out);
@@ -307,7 +300,7 @@ int conf_clone(
 	// Open common read-only transaction.
 	ret = conf_refresh_txn(out);
 	if (ret != KNOT_EOK) {
-		mm_free(out->mm, out->query_modules);
+		free(out->query_modules);
 		yp_schema_free(out->schema);
 		free(out);
 		return ret;
@@ -346,6 +339,7 @@ conf_t *conf_update(
 			conf->io.zones = s_conf->io.zones;
 		}
 		if ((flags & CONF_UPD_FMODULES) && s_conf != NULL) {
+			free(conf->query_modules);
 			conf->query_modules = s_conf->query_modules;
 			conf->query_plan = s_conf->query_plan;
 		}
@@ -395,22 +389,17 @@ void conf_free(
 	}
 	if (conf->io.zones != NULL) {
 		trie_free(conf->io.zones);
-		mm_free(conf->mm, conf->io.zones);
 	}
 
 	conf_mod_load_purge(conf, false);
 	conf_deactivate_modules(conf->query_modules, &conf->query_plan);
-	mm_free(conf->mm, conf->query_modules);
+	free(conf->query_modules);
 	conf_mod_unload_shared(conf);
 
 	if (!conf->is_clone) {
 		if (conf->api != NULL) {
 			conf->api->deinit(conf->db);
 		}
-		if (conf->mm != NULL) {
-			mp_delete(conf->mm->ctx);
-			free(conf->mm);
-		}
 	}
 
 	free(conf);
