@@ -13,15 +13,6 @@
     You should have received a copy of the GNU General Public License
     along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
-/*!
- * \file
- *
- * Server configuration core.
- *
- * \addtogroup config
- *
- * @{
- */
 
 #pragma once
 
@@ -49,7 +40,7 @@
 	} while (0)
 
 #define CONF_LOG_ZONE(severity, zone, msg, ...) do { \
-	log_fmt_zone(severity, LOG_SOURCE_ZONE, zone, "config, " msg, ##__VA_ARGS__); \
+	log_fmt_zone(severity, LOG_SOURCE_ZONE, zone, NULL, "config, " msg, ##__VA_ARGS__); \
 	} while (0)
 
 /*! Configuration getter output. */
@@ -90,8 +81,6 @@ typedef struct {
 	const struct knot_db_api *api;
 	/*! Configuration schema. */
 	yp_item_t *schema;
-	/*! Memory context. */
-	knot_mm_t *mm;
 	/*! Configuration database. */
 	knot_db_t *db;
 
@@ -125,6 +114,7 @@ typedef struct {
 		int32_t srv_max_tcp_clients;
 		int32_t ctl_timeout;
 		conf_val_t srv_nsid;
+		bool use_ecs;
 	} cache;
 
 	/*! List of dynamically loaded modules. */
@@ -187,10 +177,11 @@ void conf_refresh_hostname(
 /*!
  * Creates new or opens old configuration database.
  *
- * \param[out] conf   Configuration.
- * \param[in] schema  Configuration schema.
- * \param[in] db_dir  Database path or NULL.
- * \param[in] flags   Access flags.
+ * \param[out] conf          Configuration.
+ * \param[in] schema         Configuration schema.
+ * \param[in] db_dir         Database path or NULL.
+ * \param[in] max_conf_size  Maximum configuration DB size in bytes (LMDB mapsize).
+ * \param[in] flags          Access flags.
  *
  * \return Error code, KNOT_EOK if success.
  */
@@ -198,6 +189,7 @@ int conf_new(
 	conf_t **conf,
 	const yp_item_t *schema,
 	const char *db_dir,
+	size_t max_conf_size,
 	conf_flag_t flags
 );
 
@@ -284,5 +276,3 @@ int conf_export(
 	const char *file_name,
 	yp_style_t style
 );
-
-/*! @} */
