@@ -1,4 +1,4 @@
-/*  Copyright (C) 2017 CZ.NIC, z.s.p.o. <knot-dns@labs.nic.cz>
+/*  Copyright (C) 2018 CZ.NIC, z.s.p.o. <knot-dns@labs.nic.cz>
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -21,14 +21,13 @@
 
 #include "knot/conf/schema.h"
 #include "knot/conf/confio.h"
-#include "knot/conf/migration.h"
 #include "knot/conf/tools.h"
 #include "knot/common/log.h"
 #include "knot/journal/journal.h"
 #include "knot/updates/acl.h"
 #include "libknot/rrtype/opt.h"
-#include "dnssec/lib/dnssec/tsig.h"
-#include "dnssec/lib/dnssec/key.h"
+#include "libdnssec/tsig.h"
+#include "libdnssec/key.h"
 
 #define HOURS(x)	((x) * 3600)
 #define DAYS(x)		((x) * HOURS(24))
@@ -73,10 +72,10 @@ static const knot_lookup_t dnssec_key_algs[] = {
 };
 
 const knot_lookup_t child_record[] = {
-	{ CHILD_RECORDS_NONE,     "none"     },
-	{ CHILD_RECORDS_EMPTY,    "delete-dnssec"    },
+	{ CHILD_RECORDS_NONE,     "none" },
+	{ CHILD_RECORDS_EMPTY,    "delete-dnssec" },
 	{ CHILD_RECORDS_ROLLOVER, "rollover" },
-	{ CHILD_RECORDS_ALWAYS,   "always"   },
+	{ CHILD_RECORDS_ALWAYS,   "always" },
 	{ 0, NULL }
 };
 
@@ -88,8 +87,9 @@ const knot_lookup_t acl_actions[] = {
 };
 
 static const knot_lookup_t serial_policies[] = {
-	{ SERIAL_POLICY_INCREMENT, "increment" },
-	{ SERIAL_POLICY_UNIXTIME,  "unixtime" },
+	{ SERIAL_POLICY_INCREMENT,  "increment" },
+	{ SERIAL_POLICY_UNIXTIME,   "unixtime" },
+	{ SERIAL_POLICY_DATESERIAL, "dateserial" },
 	{ 0, NULL }
 };
 
@@ -103,6 +103,7 @@ static const knot_lookup_t journal_content[] = {
 static const knot_lookup_t zonefile_load[] = {
 	{ ZONEFILE_LOAD_NONE,  "none" },
 	{ ZONEFILE_LOAD_DIFF,  "difference" },
+	{ ZONEFILE_LOAD_DIFSE, "difference-no-serial" },
 	{ ZONEFILE_LOAD_WHOLE, "whole" },
 	{ 0, NULL }
 };
@@ -156,11 +157,7 @@ static const yp_item_t desc_server[] = {
 	                                                KNOT_EDNS_MAX_UDP_PAYLOAD, YP_SSIZE } },
 	{ C_LISTEN,               YP_TADDR, YP_VADDR = { 53 }, YP_FMULTI },
 	{ C_COMMENT,              YP_TSTR,  YP_VNONE },
-	/* Obsolete items. */
-	{ C_RATE_LIMIT,           YP_TINT,  YP_VINT = { 0, INT32_MAX, 0 } },
-	{ C_RATE_LIMIT_SLIP,      YP_TINT,  YP_VINT = { 0, 100, 1 } },
-	{ C_RATE_LIMIT_TBL_SIZE,  YP_TINT,  YP_VINT = { 1, INT32_MAX, 393241 } },
-	{ C_RATE_LIMIT_WHITELIST, YP_TNET,  YP_VNONE, YP_FMULTI },
+	{ C_ECS,                  YP_TBOOL, YP_VNONE },
 	{ NULL }
 };
 
@@ -298,10 +295,6 @@ static const yp_item_t desc_policy[] = {
 	{ C_MODULE,              YP_TDATA, YP_VDATA = { 0, NULL, mod_id_to_bin, mod_id_to_txt }, \
 	                                   YP_FMULTI | FLAGS, { check_modref } }, \
 	{ C_COMMENT,             YP_TSTR,  YP_VNONE }, \
-	/* Obsolete, old journal items. */ \
-	{ C_JOURNAL,             YP_TSTR,  YP_VNONE, FLAGS }, \
-	{ C_MAX_JOURNAL_SIZE,    YP_TINT,  YP_VINT = { 0, SSIZE_MAX, SSIZE_MAX, YP_SSIZE }, FLAGS }, \
-	{ C_IXFR_DIFF,           YP_TBOOL, YP_VNONE }, \
 
 static const yp_item_t desc_template[] = {
 	{ C_ID, YP_TSTR, YP_VNONE, CONF_IO_FREF },
@@ -346,8 +339,5 @@ const yp_item_t conf_schema[] = {
 	{ C_TPL,      YP_TGRP, YP_VGRP = { desc_template }, YP_FMULTI, { check_template } },
 	{ C_ZONE,     YP_TGRP, YP_VGRP = { desc_zone }, YP_FMULTI | CONF_IO_FZONE, { check_zone } },
 	{ C_INCL,     YP_TSTR, YP_VNONE, CONF_IO_FDIFF_ZONES | CONF_IO_FRLD_ALL, { include_file } },
-	/* Renamed modules. */
-	{ C_MOD_ONLINE_SIGN,  YP_TGRP, YP_VGRP = { schema_mod_online_sign }, YP_FMULTI, { check_mod_online_sign } },
-	{ C_MOD_SYNTH_RECORD, YP_TGRP, YP_VGRP = { schema_mod_synth_record }, YP_FMULTI, { check_mod_synth_record } },
 	{ NULL }
 };
