@@ -1,4 +1,4 @@
-/*  Copyright (C) 2017 CZ.NIC, z.s.p.o. <knot-dns@labs.nic.cz>
+/*  Copyright (C) 2018 CZ.NIC, z.s.p.o. <knot-dns@labs.nic.cz>
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -25,7 +25,7 @@
 #include <sys/stat.h>
 #include <unistd.h>
 
-#include "dnssec/key.h"
+#include "libdnssec/key.h"
 #include "knot/conf/tools.h"
 #include "knot/conf/conf.h"
 #include "knot/conf/module.h"
@@ -259,34 +259,6 @@ int check_module_id(
 int check_server(
 	knotd_conf_check_args_t *args)
 {
-	bool present = false;
-
-	conf_val_t val;
-	val = conf_get_txn(args->extra->conf, args->extra->txn, C_SRV, C_RATE_LIMIT);
-	if (val.code == KNOT_EOK) {
-		present = true;
-	}
-
-	val = conf_get_txn(args->extra->conf, args->extra->txn, C_SRV, C_RATE_LIMIT_SLIP);
-	if (val.code == KNOT_EOK) {
-		present = true;
-	}
-
-	val = conf_get_txn(args->extra->conf, args->extra->txn, C_SRV, C_RATE_LIMIT_TBL_SIZE);
-	if (val.code == KNOT_EOK) {
-		present = true;
-	}
-
-	val = conf_get_txn(args->extra->conf, args->extra->txn, C_SRV, C_RATE_LIMIT_WHITELIST);
-	if (val.code == KNOT_EOK) {
-		present = true;
-	}
-
-	if (present) {
-		CONF_LOG(LOG_NOTICE, "obsolete RRL configuration in the server, "
-		                     "use module mod-rrl instead");
-	}
-
 	return KNOT_EOK;
 }
 
@@ -422,14 +394,6 @@ int check_remote(
 int check_template(
 	knotd_conf_check_args_t *args)
 {
-	conf_val_t ixfr_diff = conf_rawid_get_txn(args->extra->conf, args->extra->txn,
-	                                          C_TPL, C_IXFR_DIFF, args->id, args->id_len);
-	if (ixfr_diff.code == KNOT_EOK) {
-		CONF_LOG(LOG_NOTICE, "IXFR from differences setting is obsolete "
-		         "and will be removed in the next major release, use "
-		         "template.zonefile-load instead");
-	}
-
 	// Stop if the default template.
 	if (is_default_id(args->id, args->id_len)) {
 		return KNOT_EOK;
@@ -459,14 +423,6 @@ int check_template(
 int check_zone(
 	knotd_conf_check_args_t *args)
 {
-	conf_val_t ixfr_diff = conf_rawid_get_txn(args->extra->conf, args->extra->txn,
-	                                          C_ZONE, C_IXFR_DIFF, args->id, args->id_len);
-	if (ixfr_diff.code == KNOT_EOK) {
-		CONF_LOG(LOG_NOTICE, "IXFR from differences setting is obsolete "
-		         "and will be removed in the next major release, use "
-		         "zone.zonefile-load instead");
-	}
-
 	return KNOT_EOK;
 }
 
@@ -521,9 +477,9 @@ int include_file(
 		goto include_error;
 	}
 
-	// Evaluate include pattern.
+	// Evaluate include pattern (empty wildcard match is also valid).
 	ret = glob(path, 0, glob_error, &glob_buf);
-	if (ret != 0) {
+	if (ret != 0 && (ret != GLOB_NOMATCH || strchr(path, '*') == NULL)) {
 		ret = KNOT_EFILE;
 		goto include_error;
 	}
