@@ -1,4 +1,4 @@
-/*  Copyright (C) 2017 CZ.NIC, z.s.p.o. <knot-dns@labs.nic.cz>
+/*  Copyright (C) 2018 CZ.NIC, z.s.p.o. <knot-dns@labs.nic.cz>
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -23,7 +23,7 @@
 #include "knot/conf/confio.h"
 #include "knot/ctl/commands.h"
 #include "knot/dnssec/key-events.h"
-#include "knot/events/handlers.h"
+#include "knot/events/events.h"
 #include "knot/events/log.h"
 #include "knot/nameserver/query_module.h"
 #include "knot/updates/zone-update.h"
@@ -34,7 +34,8 @@
 #include "contrib/macros.h"
 #include "contrib/mempattern.h"
 #include "contrib/string.h"
-#include "zscanner/scanner.h"
+#include "contrib/ucw/lists.h"
+#include "libzscanner/scanner.h"
 #include "contrib/strtonum.h"
 
 #define MATCH_OR_FILTER(args, code) ((args)->data[KNOT_CTL_IDX_FILTER] == NULL || \
@@ -100,11 +101,7 @@ static int get_zone(ctl_args_t *args, zone_t **zone)
 	if (dname == NULL) {
 		return KNOT_EINVAL;
 	}
-
-	int ret = knot_dname_to_lower(dname);
-	if (ret != KNOT_EOK) {
-		return ret;
-	}
+	knot_dname_to_lower(dname);
 
 	*zone = knot_zonedb_find(args->server->zone_db, dname);
 	if (*zone == NULL) {
@@ -185,7 +182,7 @@ static int zone_status(zone_t *zone, ctl_args_t *args)
 		if (zone->contents != NULL) {
 			knot_rdataset_t *soa = node_rdataset(zone->contents->apex,
 			                                     KNOT_RRTYPE_SOA);
-			ret = snprintf(buff, sizeof(buff), "%u", knot_soa_serial(soa));
+			ret = snprintf(buff, sizeof(buff), "%u", knot_soa_serial(soa->rdata));
 		} else {
 			ret = snprintf(buff, sizeof(buff), "none");
 		}
@@ -368,7 +365,9 @@ static int zone_ksk_sbm_confirm(zone_t *zone, ctl_args_t *args)
 
 	ret = knot_dnssec_ksk_sbm_confirm(&ctx);
 	kdnssec_ctx_deinit(&ctx);
-	if (ret == KNOT_EOK) {
+
+	conf_val_t val = conf_zone_get(conf(), C_DNSSEC_SIGNING, zone->name);
+	if (ret == KNOT_EOK && conf_bool(&val)) {
 		// NOT zone_events_schedule_user(), intentionally!
 		zone_events_schedule_now(zone, ZONE_EVENT_DNSSEC);
 	}
@@ -418,6 +417,15 @@ static int zone_txn_begin(zone_t *zone, ctl_args_t *args)
 	return KNOT_EOK;
 }
 
+static void zone_txn_update_clear(zone_t *zone)
+{
+	assert(zone->control_update);
+
+	zone_update_clear(zone->control_update);
+	free(zone->control_update);
+	zone->control_update = NULL;
+}
+
 static int zone_txn_commit(zone_t *zone, ctl_args_t *args)
 {
 	UNUSED(args);
@@ -426,6 +434,15 @@ static int zone_txn_commit(zone_t *zone, ctl_args_t *args)
 		return KNOT_TXN_ENOTEXISTS;
 	}
 
+	// NOOP if empty changeset/contents.
+	if (((zone->control_update->flags & UPDATE_INCREMENTAL) &&
+	     changeset_empty(&zone->control_update->change)) ||
+	    ((zone->control_update->flags & UPDATE_FULL) &&
+	     zone_contents_is_empty(zone->control_update->new_cont))) {
+		zone_txn_update_clear(zone);
+		return KNOT_EOK;
+	}
+
 	// Sign update.
 	conf_val_t val = conf_zone_get(conf(), C_DNSSEC_SIGNING, zone->name);
 	bool dnssec_enable = (zone->control_update->flags & UPDATE_SIGN) && conf_bool(&val);
@@ -433,9 +450,7 @@ static int zone_txn_commit(zone_t *zone, ctl_args_t *args)
 		zone_sign_reschedule_t resch = { 0 };
 		int ret = knot_dnssec_sign_update(zone->control_update, &resch);
 		if (ret != KNOT_EOK) {
-			zone_update_clear(zone->control_update);
-			free(zone->control_update);
-			zone->control_update = NULL;
+			zone_txn_update_clear(zone);
 			return ret;
 		}
 		log_dnssec_next(zone->name, (time_t)resch.next_sign);
@@ -452,9 +467,7 @@ static int zone_txn_commit(zone_t *zone, ctl_args_t *args)
 		return ret;
 	}
 
-	zone_update_clear(zone->control_update);
-	free(zone->control_update);
-	zone->control_update = NULL;
+	zone_txn_update_clear(zone);
 
 	zone_events_schedule_now(zone, ZONE_EVENT_NOTIFY);
 
@@ -469,9 +482,7 @@ static int zone_txn_abort(zone_t *zone, ctl_args_t *args)
 		return KNOT_TXN_ENOTEXISTS;
 	}
 
-	zone_update_clear(zone->control_update);
-	free(zone->control_update);
-	zone->control_update = NULL;
+	zone_txn_update_clear(zone);
 
 	return KNOT_EOK;
 }
@@ -490,16 +501,16 @@ typedef struct {
 
 static send_ctx_t *create_send_ctx(const knot_dname_t *zone_name, ctl_args_t *args)
 {
-	send_ctx_t *ctx = mm_alloc(&args->mm, sizeof(*ctx));
+	send_ctx_t *ctx = mm_calloc(&args->mm, 1, sizeof(*ctx));
 	if (ctx == NULL) {
 		return NULL;
 	}
-	memset(ctx, 0, sizeof(*ctx));
 
 	ctx->args = args;
 
 	// Set the dump style.
 	ctx->style.show_ttl = true;
+	ctx->style.original_ttl = true;
 	ctx->style.human_tmstamp = true;
 
 	// Set the output data buffers.
@@ -532,18 +543,28 @@ static send_ctx_t *create_send_ctx(const knot_dname_t *zone_name, ctl_args_t *ar
 
 static int send_rrset(knot_rrset_t *rrset, send_ctx_t *ctx)
 {
-	int ret = snprintf(ctx->ttl, sizeof(ctx->ttl), "%u", knot_rrset_ttl(rrset));
-	if (ret <= 0 || ret >= sizeof(ctx->ttl)) {
-		return KNOT_ESPACE;
+	if (rrset->type != KNOT_RRTYPE_RRSIG) {
+		int ret = snprintf(ctx->ttl, sizeof(ctx->ttl), "%u", rrset->ttl);
+		if (ret <= 0 || ret >= sizeof(ctx->ttl)) {
+			return KNOT_ESPACE;
+		}
 	}
 
 	if (knot_rrtype_to_string(rrset->type, ctx->type, sizeof(ctx->type)) < 0) {
 		return KNOT_ESPACE;
 	}
 
-	for (size_t i = 0; i < rrset->rrs.rr_count; ++i) {
-		ret = knot_rrset_txt_dump_data(rrset, i, ctx->rdata,
-		                               sizeof(ctx->rdata), &ctx->style);
+	for (size_t i = 0; i < rrset->rrs.count; ++i) {
+		if (rrset->type == KNOT_RRTYPE_RRSIG) {
+			int ret = snprintf(ctx->ttl, sizeof(ctx->ttl), "%u",
+			                   knot_rrsig_original_ttl(knot_rdataset_at(&rrset->rrs, i)));
+			if (ret <= 0 || ret >= sizeof(ctx->ttl)) {
+				return KNOT_ESPACE;
+			}
+		}
+
+		int ret = knot_rrset_txt_dump_data(rrset, i, ctx->rdata,
+		                                   sizeof(ctx->rdata), &ctx->style);
 		if (ret < 0) {
 			return ret;
 		}
@@ -588,7 +609,7 @@ static int get_owner(uint8_t *out, size_t out_len, knot_dname_t *origin,
 	assert(owner != NULL);
 
 	bool fqdn = false;
-	int prefix_len = 0;
+	size_t prefix_len = 0;
 
 	size_t owner_len = strlen(owner);
 	if (owner_len > 0 && (owner_len != 1 || owner[0] != '@')) {
@@ -601,14 +622,10 @@ static int get_owner(uint8_t *out, size_t out_len, knot_dname_t *origin,
 		if (dname == NULL) {
 			return KNOT_EINVAL;
 		}
-
-		int ret = knot_dname_to_lower(dname);
-		if (ret != KNOT_EOK) {
-			return ret;
-		}
+		knot_dname_to_lower(dname);
 
 		prefix_len = knot_dname_size(out);
-		if (prefix_len <= 0) {
+		if (prefix_len == 0) {
 			return KNOT_EINVAL;
 		}
 
@@ -618,8 +635,8 @@ static int get_owner(uint8_t *out, size_t out_len, knot_dname_t *origin,
 
 	// Append the origin.
 	if (!fqdn) {
-		int origin_len = knot_dname_size(origin);
-		if (origin_len <= 0 || origin_len > out_len - prefix_len) {
+		size_t origin_len = knot_dname_size(origin);
+		if (origin_len == 0 || origin_len > out_len - prefix_len) {
 			return KNOT_EINVAL;
 		}
 		memcpy(out + prefix_len, origin, origin_len);
@@ -825,7 +842,7 @@ static int get_ttl(zone_t *zone, ctl_args_t *args, uint32_t *ttl)
 
 	const zone_node_t *node = zone_update_get_node(zone->control_update, owner);
 	if (node == NULL) {
-		return KNOT_ETTL;
+		return KNOT_EINVAL;
 	}
 
 	uint16_t type;
@@ -833,12 +850,7 @@ static int get_ttl(zone_t *zone, ctl_args_t *args, uint32_t *ttl)
 		return KNOT_EINVAL;
 	}
 
-	knot_rdataset_t *rdataset = node_rdataset(node, type);
-	if (rdataset == NULL) {
-		return KNOT_ETTL;
-	}
-
-	*ttl = knot_rdataset_ttl(rdataset);
+	*ttl = node_rrset(node, type).ttl;
 
 	return KNOT_EOK;
 }
@@ -906,14 +918,14 @@ static int create_rrset(knot_rrset_t **rrset, zone_t *zone, ctl_args_t *args,
 
 	// Create output rrset.
 	*rrset = knot_rrset_new(scanner->r_owner, scanner->r_type,
-	                        scanner->r_class, NULL);
+	                        scanner->r_class, scanner->r_ttl, NULL);
 	if (*rrset == NULL) {
 		ret = KNOT_ENOMEM;
 		goto parser_failed;
 	}
 
 	ret = knot_rrset_add_rdata(*rrset, scanner->r_data, scanner->r_data_length,
-	                           scanner->r_ttl, NULL);
+	                           NULL);
 parser_failed:
 	zs_deinit(scanner);
 	mm_free(&args->mm, scanner);
@@ -940,12 +952,7 @@ static int zone_txn_set(zone_t *zone, ctl_args_t *args)
 	}
 
 	ret = zone_update_add(zone->control_update, rrset);
-	knot_rrset_free(&rrset, NULL);
-
-	// Silently update TTL.
-	if (ret == KNOT_ETTL) {
-		ret = KNOT_EOK;
-	}
+	knot_rrset_free(rrset, NULL);
 
 	return ret;
 }
@@ -973,7 +980,7 @@ static int zone_txn_unset(zone_t *zone, ctl_args_t *args)
 		}
 
 		ret = zone_update_remove(zone->control_update, rrset);
-		knot_rrset_free(&rrset, NULL);
+		knot_rrset_free(rrset, NULL);
 		return ret;
 	} else {
 		uint8_t owner[KNOT_DNAME_MAXLEN];
@@ -997,7 +1004,128 @@ static int zone_txn_unset(zone_t *zone, ctl_args_t *args)
 			return zone_update_remove_node(zone->control_update, owner);
 		}
 	}
+}
 
+static bool zone_exists(const knot_dname_t *zone, void *data)
+{
+	assert(zone);
+	assert(data);
+
+	knot_zonedb_t *db = data;
+
+	return knot_zonedb_find(db, zone) != NULL;
+}
+
+static bool zone_names_distinct(const knot_dname_t *zone, void *data)
+{
+	assert(zone);
+	assert(data);
+
+	knot_dname_t *zone_to_purge = data;
+
+	return !knot_dname_is_equal(zone, zone_to_purge);
+}
+
+static int orphans_purge(ctl_args_t *args)
+{
+	assert(args->data[KNOT_CTL_IDX_FILTER] != NULL);
+	bool only_orphan = (strlen(args->data[KNOT_CTL_IDX_FILTER]) == 1);
+
+	if (args->data[KNOT_CTL_IDX_ZONE] == NULL) {
+		// Purge KASP DB.
+		if (only_orphan || MATCH_AND_FILTER(args, CTL_FILTER_PURGE_KASPDB)) {
+			list_t zones;
+			init_list(&zones);
+			if (kasp_db_open(*kaspdb()) == KNOT_EOK &&
+			    kasp_db_list_zones(*kaspdb(), &zones) == KNOT_EOK) {
+				ptrnode_t *zn;
+				WALK_LIST(zn, zones) {
+					knot_dname_t *zone_name = (knot_dname_t *)zn->d;
+					if (!zone_exists(zone_name, args->server->zone_db)) {
+						(void)kasp_db_delete_all(*kaspdb(), zone_name);
+					}
+					knot_dname_free(zone_name, NULL);
+				}
+				ptrlist_free(&zones, NULL);
+			}
+		}
+
+		// Purge zone journals of unconfigured zones.
+		if (only_orphan || MATCH_AND_FILTER(args, CTL_FILTER_PURGE_JOURNAL)) {
+			list_t zones;
+			init_list(&zones);
+			if (journal_db_list_zones(&args->server->journal_db, &zones) == KNOT_EOK) {
+				ptrnode_t *zn;
+				WALK_LIST(zn, zones) {
+					journal_t journal = { 0 };
+					knot_dname_t *zone_name = (knot_dname_t *)zn->d;
+					if (!zone_exists(zone_name, args->server->zone_db) &&
+					    journal_open(&journal, &args->server->journal_db,
+					                 zone_name) == KNOT_EOK) {
+						journal_scrape(&journal);
+						journal_close(&journal);
+					}
+					knot_dname_free(zone_name, NULL);
+				}
+				ptrlist_free(&zones, NULL);
+			}
+		}
+
+		// Purge timers of unconfigured zones.
+		if (only_orphan || MATCH_AND_FILTER(args, CTL_FILTER_PURGE_TIMERS)) {
+			(void)zone_timers_sweep(args->server->timers_db,
+			                        zone_exists, args->server->zone_db);
+		}
+	} else {
+		uint8_t buff[KNOT_DNAME_MAXLEN];
+		while (true) {
+			knot_dname_t *zone_name =
+				knot_dname_from_str(buff, args->data[KNOT_CTL_IDX_ZONE],
+				                    sizeof(buff));
+			if (zone_name == NULL) {
+				log_ctl_zone_str_error(args->data[KNOT_CTL_IDX_ZONE],
+				                       "control, error (%s)",
+				                       knot_strerror(KNOT_EINVAL));
+				send_error(args, knot_strerror(KNOT_EINVAL));
+				return KNOT_EINVAL;
+			}
+			knot_dname_to_lower(zone_name);
+
+			if (!zone_exists(zone_name, args->server->zone_db)) {
+				// Purge KASP DB.
+				if (only_orphan || MATCH_AND_FILTER(args, CTL_FILTER_PURGE_KASPDB)) {
+					if (kasp_db_open(*kaspdb()) == KNOT_EOK) {
+						(void) kasp_db_delete_all(*kaspdb(), zone_name);
+					}
+				}
+
+				// Purge zone journal.
+				if (only_orphan || MATCH_AND_FILTER(args, CTL_FILTER_PURGE_JOURNAL)) {
+					journal_t journal = { 0 };
+					if (journal_open(&journal, &args->server->journal_db,
+					                 zone_name) == KNOT_EOK) {
+						(void)journal_scrape(&journal);
+						journal_close(&journal);
+					}
+				}
+
+				// Purge zone timers.
+				if (only_orphan || MATCH_AND_FILTER(args, CTL_FILTER_PURGE_TIMERS)) {
+					(void)zone_timers_sweep(args->server->timers_db,
+					                        zone_names_distinct, zone_name);
+				}
+			}
+
+			// Get next zone name.
+			int ret = knot_ctl_receive(args->ctl, &args->type, &args->data);
+			if (ret != KNOT_EOK || args->type != KNOT_CTL_TYPE_DATA) {
+				break;
+			}
+			ctl_log_data(&args->data);
+		}
+	}
+
+	return KNOT_EOK;
 }
 
 static int zone_purge(zone_t *zone, ctl_args_t *args)
@@ -1010,11 +1138,13 @@ static int zone_purge(zone_t *zone, ctl_args_t *args)
 	// Purge the zone timers.
 	if (MATCH_OR_FILTER(args, CTL_FILTER_PURGE_TIMERS)) {
 		memset(&zone->timers, 0, sizeof(zone->timers));
+		(void)zone_timers_sweep(args->server->timers_db,
+		                        zone_names_distinct, zone->name);
 	}
 
 	// Expire the zone.
 	if (MATCH_OR_FILTER(args, CTL_FILTER_PURGE_EXPIRE)) {
-		(void)event_expire(conf(), zone);
+		zone_events_schedule_user(zone, ZONE_EVENT_EXPIRE);
 	}
 
 	// Purge the zone file.
@@ -1227,7 +1357,11 @@ static int ctl_zone(ctl_args_t *args, ctl_cmd_t cmd)
 	case CTL_ZONE_UNSET:
 		return zones_apply(args, zone_txn_unset);
 	case CTL_ZONE_PURGE:
-		return zones_apply(args, zone_purge);
+		if (MATCH_AND_FILTER(args, CTL_FILTER_PURGE_ORPHAN)) {
+			return orphans_purge(args);
+		} else {
+			return zones_apply(args, zone_purge);
+		}
 	case CTL_ZONE_STATS:
 		return zones_apply(args, zone_stats);
 	default:
