@@ -36,14 +36,14 @@ static bool key_present(const kdnssec_ctx_t *ctx, bool ksk, bool zsk)
 	return false;
 }
 
-static bool key_id_present(const kdnssec_ctx_t *ctx, const char *keyid, uint16_t flag)
+static bool key_id_present(const kdnssec_ctx_t *ctx, const char *keyid, bool want_ksk)
 {
 	assert(ctx);
 	assert(ctx->zone);
 	for (size_t i = 0; i < ctx->zone->num_keys; i++) {
 		const knot_kasp_key_t *key = &ctx->zone->keys[i];
 		if (strcmp(keyid, key->id) == 0 &&
-		    dnssec_key_get_flags(key->key) == flag) {
+		    key->is_ksk == want_ksk) {
 			return true;
 		}
 	}
@@ -88,34 +88,36 @@ static knot_kasp_key_t *key_get_by_id(kdnssec_ctx_t *ctx, const char *keyid)
 	return NULL;
 }
 
-static int generate_key(kdnssec_ctx_t *ctx, bool ksk, bool zsk, knot_time_t when_active, bool pre_active)
+static int generate_key(kdnssec_ctx_t *ctx, kdnssec_generate_flags_t flags,
+			knot_time_t when_active, bool pre_active)
 {
 	assert(!pre_active || when_active == 0);
 
 	knot_kasp_key_t *key = NULL;
-	int ret = kdnssec_generate_key(ctx, ksk, zsk, &key);
+	int ret = kdnssec_generate_key(ctx, flags, &key);
 	if (ret != KNOT_EOK) {
 		return ret;
 	}
 
 	key->timing.remove = 0;
 	key->timing.retire = 0;
-	key->timing.active = (ksk ? 0 : when_active);
-	key->timing.ready  = (ksk ? when_active : 0);
+	key->timing.active = ((flags & DNSKEY_GENERATE_KSK) ? 0 : when_active);
+	key->timing.ready  = ((flags & DNSKEY_GENERATE_KSK) ? when_active : 0);
 	key->timing.publish    = (pre_active ? 0 : ctx->now);
 	key->timing.pre_active = (pre_active ? ctx->now : 0);
 
 	return KNOT_EOK;
 }
 
-static int share_or_generate_key(kdnssec_ctx_t *ctx, bool ksk, bool zsk, knot_time_t when_active, bool pre_active)
+static int share_or_generate_key(kdnssec_ctx_t *ctx, kdnssec_generate_flags_t flags,
+				 knot_time_t when_active, bool pre_active)
 {
 	assert(!pre_active || when_active == 0);
 
 	knot_dname_t *borrow_zone = NULL;
 	char *borrow_key = NULL;
 
-	if (!ksk) {
+	if (!(flags & DNSKEY_GENERATE_KSK)) {
 		return KNOT_EINVAL;
 	} // for now not designed for rotating shared ZSK
 
@@ -126,16 +128,16 @@ static int share_or_generate_key(kdnssec_ctx_t *ctx, bool ksk, bool zsk, knot_ti
 	}
 
 	// if we already have the policy-last key, we have to generate new one
-	if (ret == KNOT_ENOENT || key_id_present(ctx, borrow_key, DNSKEY_FLAGS_KSK)) {
+	if (ret == KNOT_ENOENT || key_id_present(ctx, borrow_key, true)) {
 		knot_kasp_key_t *key = NULL;
-		ret = kdnssec_generate_key(ctx, ksk, zsk, &key);
+		ret = kdnssec_generate_key(ctx, flags, &key);
 		if (ret != KNOT_EOK) {
 			return ret;
 		}
 		key->timing.remove = 0;
 		key->timing.retire = 0;
-		key->timing.active = (ksk ? 0 : when_active);
-		key->timing.ready  = (ksk ? when_active : 0);
+		key->timing.active = ((flags & DNSKEY_GENERATE_KSK) ? 0 : when_active);
+		key->timing.ready  = ((flags & DNSKEY_GENERATE_KSK) ? when_active : 0);
 		key->timing.publish    = (pre_active ? 0 : ctx->now);
 		key->timing.pre_active = (pre_active ? ctx->now : 0);
 
@@ -175,12 +177,12 @@ static int share_or_generate_key(kdnssec_ctx_t *ctx, bool ksk, bool zsk, knot_ti
 			assert(newkey != NULL);
 			newkey->timing.remove = 0;
 			newkey->timing.retire = 0;
-			newkey->timing.active = (ksk ? 0 : when_active);
-			newkey->timing.ready  = (ksk ? when_active : 0);
+			newkey->timing.active = ((flags & DNSKEY_GENERATE_KSK) ? 0 : when_active);
+			newkey->timing.ready  = ((flags & DNSKEY_GENERATE_KSK) ? when_active : 0);
 			newkey->timing.publish    = (pre_active ? 0 : ctx->now);
 			newkey->timing.pre_active = (pre_active ? ctx->now : 0);
-			newkey->is_ksk = ksk;
-			newkey->is_zsk = zsk;
+			newkey->is_ksk = (flags & DNSKEY_GENERATE_KSK);
+			newkey->is_zsk = (flags & DNSKEY_GENERATE_ZSK);
 		}
 	}
 	free(borrow_zone);
@@ -188,6 +190,8 @@ static int share_or_generate_key(kdnssec_ctx_t *ctx, bool ksk, bool zsk, knot_ti
 	return ret;
 }
 
+#define GEN_KSK_FLAGS (DNSKEY_GENERATE_KSK | (ctx->policy->singe_type_signing ? DNSKEY_GENERATE_ZSK : 0))
+
 static bool running_rollover(const kdnssec_ctx_t *ctx)
 {
 	bool res = false;
@@ -468,7 +472,7 @@ static int exec_new_signatures(kdnssec_ctx_t *ctx, knot_kasp_key_t *newkey)
 		knot_kasp_key_t *key = &ctx->zone->keys[i];
 		key_state_t keystate = get_key_state(key, ctx->now);
 		uint8_t keyalg = dnssec_key_get_algorithm(key->key);
-		if (((newkey->is_ksk && key->is_ksk) || (newkey->is_zsk && key->is_zsk))
+		if (((newkey->is_ksk && key->is_ksk) || (newkey->is_zsk && key->is_zsk && !key->is_ksk))
 		    && keystate == DNSSEC_KEY_STATE_ACTIVE) {
 			if (key->is_ksk || keyalg != dnssec_key_get_algorithm(newkey->key)) {
 				key->timing.retire_active = ctx->now;
@@ -545,16 +549,16 @@ int knot_dnssec_key_rollover(kdnssec_ctx_t *ctx, zone_sign_reschedule_t *resched
 	// generate initial keys if missing
 	if (!key_present(ctx, true, false) && !key_present(ctx, true, true)) {
 		if (ctx->policy->ksk_shared) {
-			ret = share_or_generate_key(ctx, true, ctx->policy->singe_type_signing, ctx->now, false);
+			ret = share_or_generate_key(ctx, GEN_KSK_FLAGS, ctx->now, false);
 		} else {
-			ret = generate_key(ctx, true, ctx->policy->singe_type_signing, ctx->now, false);
+			ret = generate_key(ctx, GEN_KSK_FLAGS, ctx->now, false);
 		}
 		reschedule->plan_ds_query = true;
 		if (ret == KNOT_EOK) {
 			reschedule->keys_changed = true;
 			if (!ctx->policy->singe_type_signing &&
 			    !key_present(ctx, false, true)) {
-				ret = generate_key(ctx, false, true, ctx->now, false);
+				ret = generate_key(ctx, DNSKEY_GENERATE_ZSK, ctx->now, false);
 			}
 		}
 	}
@@ -562,12 +566,12 @@ int knot_dnssec_key_rollover(kdnssec_ctx_t *ctx, zone_sign_reschedule_t *resched
 	if (algorithm_present(ctx, ctx->policy->algorithm) == 0 &&
 	    !running_rollover(ctx) && ret == KNOT_EOK) {
 		if (ctx->policy->ksk_shared) {
-			ret = share_or_generate_key(ctx, true, ctx->policy->singe_type_signing, 0, true);
+			ret = share_or_generate_key(ctx, GEN_KSK_FLAGS, 0, true);
 		} else {
-			ret = generate_key(ctx, true, ctx->policy->singe_type_signing, 0, true);
+			ret = generate_key(ctx, GEN_KSK_FLAGS, 0, true);
 		}
 		if (!ctx->policy->singe_type_signing && ret == KNOT_EOK) {
-			ret = generate_key(ctx, false, true, 0, true);
+			ret = generate_key(ctx, DNSKEY_GENERATE_ZSK, 0, true);
 		}
 		log_zone_info(ctx->zone->dname, "DNSSEC, algorithm rollover started");
 		if (ret == KNOT_EOK) {
@@ -578,12 +582,12 @@ int knot_dnssec_key_rollover(kdnssec_ctx_t *ctx, zone_sign_reschedule_t *resched
 	if (!signing_scheme_present(ctx) &&
 	    !running_rollover(ctx) && ret == KNOT_EOK) {
 		if (ctx->policy->ksk_shared) {
-			ret = share_or_generate_key(ctx, true, ctx->policy->singe_type_signing, 0, false);
+			ret = share_or_generate_key(ctx, GEN_KSK_FLAGS, 0, false);
 		} else {
-			ret = generate_key(ctx, true, ctx->policy->singe_type_signing, 0, false);
+			ret = generate_key(ctx, GEN_KSK_FLAGS, 0, false);
 		}
 		if (!ctx->policy->singe_type_signing && ret == KNOT_EOK) {
-			ret = generate_key(ctx, false, true, 0, false);
+			ret = generate_key(ctx, DNSKEY_GENERATE_ZSK, 0, false);
 		}
 		log_zone_info(ctx->zone->dname, "DNSSEC, signing scheme rollover started");
 		if (ret == KNOT_EOK) {
@@ -602,9 +606,9 @@ int knot_dnssec_key_rollover(kdnssec_ctx_t *ctx, zone_sign_reschedule_t *resched
 		switch (next.type) {
 		case GENERATE:
 			if (next.ksk && ctx->policy->ksk_shared) {
-				ret = share_or_generate_key(ctx, true, ctx->policy->singe_type_signing, 0, false);
+				ret = share_or_generate_key(ctx, GEN_KSK_FLAGS, 0, false);
 			} else {
-				ret = generate_key(ctx, next.ksk, ctx->policy->singe_type_signing || !next.ksk, 0, false);
+				ret = generate_key(ctx, next.ksk ? GEN_KSK_FLAGS : DNSKEY_GENERATE_ZSK, 0, false);
 			}
 			if (ret == KNOT_EOK) {
 				log_zone_info(ctx->zone->dname, "DNSSEC, %cSK rollover started",
