@@ -1,4 +1,4 @@
-/*  Copyright (C) 2017 CZ.NIC, z.s.p.o. <knot-dns@labs.nic.cz>
+/*  Copyright (C) 2018 CZ.NIC, z.s.p.o. <knot-dns@labs.nic.cz>
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -38,7 +38,7 @@ static int create_nsec_rrset(knot_rrset_t *rrset, const zone_node_t *from,
 {
 	assert(from);
 	assert(to);
-	knot_rrset_init(rrset, from->owner, KNOT_RRTYPE_NSEC, KNOT_CLASS_IN);
+	knot_rrset_init(rrset, from->owner, KNOT_RRTYPE_NSEC, KNOT_CLASS_IN, ttl);
 
 	// Create bitmap
 	dnssec_nsec_bitmap_t *rr_types = dnssec_nsec_bitmap_new();
@@ -61,7 +61,7 @@ static int create_nsec_rrset(knot_rrset_t *rrset, const zone_node_t *from,
 	dnssec_nsec_bitmap_write(rr_types, rdata + next_owner_size);
 	dnssec_nsec_bitmap_free(rr_types);
 
-	return knot_rrset_add_rdata(rrset, rdata, rdata_size, ttl, NULL);
+	return knot_rrset_add_rdata(rrset, rdata, rdata_size, NULL);
 }
 
 /*!
@@ -117,13 +117,14 @@ static int connect_nsec_nodes(zone_node_t *a, zone_node_t *b,
 		knot_rrset_t *old_nsec_lc = knot_rrset_copy(&old_nsec, NULL);
 		ret = knot_rrset_rr_to_canonical(old_nsec_lc);
 		if (ret != KNOT_EOK) {
-			knot_rrset_free(&old_nsec_lc, NULL);
+			knot_rrset_free(old_nsec_lc, NULL);
 			return ret;
 		}
 
 		bool equal = knot_rrset_equal(&new_nsec, old_nsec_lc,
 		                              KNOT_RRSET_COMPARE_WHOLE);
-		knot_rrset_free(&old_nsec_lc, NULL);
+		equal = (equal && (old_nsec_lc->ttl == new_nsec.ttl));
+		knot_rrset_free(old_nsec_lc, NULL);
 
 		if (equal) {
 			// current NSEC is valid, do nothing
@@ -344,8 +345,7 @@ cleanup:
 /*!
  * \brief Add entry for removed NSEC to the changeset.
  */
-int knot_nsec_changeset_remove(const zone_node_t *n,
-                               changeset_t *changeset)
+int knot_nsec_changeset_remove(const zone_node_t *n, changeset_t *changeset)
 {
 	if (changeset == NULL) {
 		return KNOT_EINVAL;
@@ -369,7 +369,7 @@ int knot_nsec_changeset_remove(const zone_node_t *n,
 	if (!knot_rrset_empty(&rrsigs)) {
 		knot_rrset_t synth_rrsigs;
 		knot_rrset_init(&synth_rrsigs, n->owner, KNOT_RRTYPE_RRSIG,
-		                KNOT_CLASS_IN);
+		                KNOT_CLASS_IN, rrsigs.ttl);
 		result = knot_synth_rrsig(KNOT_RRTYPE_NSEC, &rrsigs.rrs,
 		                          &synth_rrsigs.rrs, NULL);
 		if (result == KNOT_ENOENT) {
