@@ -1,4 +1,4 @@
-/*  Copyright (C) 2017 CZ.NIC, z.s.p.o. <knot-dns@labs.nic.cz>
+/*  Copyright (C) 2018 CZ.NIC, z.s.p.o. <knot-dns@labs.nic.cz>
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -44,7 +44,7 @@ inline static bool valid_nsec3_node(const zone_node_t *node)
 		return false;
 	}
 
-	if (nsec3->rr_count != 1) {
+	if (nsec3->count != 1) {
 		return false;
 	}
 
@@ -62,7 +62,8 @@ static bool are_nsec3_nodes_equal(const zone_node_t *a, const zone_node_t *b)
 
 	knot_rrset_t a_rrset = node_rrset(a, KNOT_RRTYPE_NSEC3);
 	knot_rrset_t b_rrset = node_rrset(b, KNOT_RRTYPE_NSEC3);
-	return knot_rrset_equal(&a_rrset, &b_rrset, KNOT_RRSET_COMPARE_WHOLE);
+	return knot_rrset_equal(&a_rrset, &b_rrset, KNOT_RRSET_COMPARE_WHOLE) &&
+	       (a_rrset.ttl == b_rrset.ttl);
 }
 
 static bool nsec3_opt_out(const zone_node_t *node, bool opt_out_enabled)
@@ -129,9 +130,8 @@ static int copy_signatures(zone_tree_t *from, zone_tree_t *to)
 
 	for (/* NOP */; !trie_it_finished(it); trie_it_next(it)) {
 		zone_node_t *node_from = (zone_node_t *)*trie_it_val(it);
-		zone_node_t *node_to = NULL;
 
-		zone_tree_get(to, node_from->owner, &node_to);
+		zone_node_t *node_to = zone_tree_get(to, node_from->owner);
 		if (node_to == NULL) {
 			continue;
 		}
@@ -167,7 +167,7 @@ static void free_nsec3_tree(zone_tree_t *nodes)
 		knot_rdataset_t *rrsig = node_rdataset(node, KNOT_RRTYPE_RRSIG);
 		knot_rdataset_clear(nsec3, NULL);
 		knot_rdataset_clear(rrsig, NULL);
-		node_free(&node, NULL);
+		node_free(node, NULL);
 	}
 
 	trie_it_free(it);
@@ -242,7 +242,7 @@ static int nsec3_fill_rdata(uint8_t *rdata, size_t rdata_len,
  * \return Pointer to created RRSet on success, NULL on errors.
  */
 static int create_nsec3_rrset(knot_rrset_t *rrset,
-                              knot_dname_t *owner,
+                              const knot_dname_t *owner,
                               const dnssec_nsec3_params_t *params,
                               const dnssec_nsec_bitmap_t *rr_types,
                               const uint8_t *next_hashed,
@@ -253,7 +253,11 @@ static int create_nsec3_rrset(knot_rrset_t *rrset,
 	assert(params);
 	assert(rr_types);
 
-	knot_rrset_init(rrset, owner, KNOT_RRTYPE_NSEC3, KNOT_CLASS_IN);
+	knot_dname_t *owner_copy = knot_dname_copy(owner, NULL);
+	if (owner_copy == NULL) {
+		return KNOT_ENOMEM;
+	}
+	knot_rrset_init(rrset, owner_copy, KNOT_RRTYPE_NSEC3, KNOT_CLASS_IN, ttl);
 
 	size_t rdata_size = nsec3_rdata_size(params, rr_types);
 	uint8_t rdata[rdata_size];
@@ -261,16 +265,23 @@ static int create_nsec3_rrset(knot_rrset_t *rrset,
 	int ret = nsec3_fill_rdata(rdata, rdata_size, params, rr_types,
 	                           next_hashed);
 	if (ret != KNOT_EOK) {
+		knot_dname_free(owner_copy, NULL);
 		return ret;
 	}
 
-	return knot_rrset_add_rdata(rrset, rdata, rdata_size, ttl, NULL);
+	ret = knot_rrset_add_rdata(rrset, rdata, rdata_size, NULL);
+	if (ret != KNOT_EOK) {
+		knot_dname_free(owner_copy, NULL);
+		return ret;
+	}
+
+	return KNOT_EOK;
 }
 
 /*!
  * \brief Create NSEC3 node.
  */
-static zone_node_t *create_nsec3_node(knot_dname_t *owner,
+static zone_node_t *create_nsec3_node(const knot_dname_t *owner,
                                       const dnssec_nsec3_params_t *nsec3_params,
                                       zone_node_t *apex_node,
                                       const dnssec_nsec_bitmap_t *rr_types,
@@ -292,14 +303,14 @@ static zone_node_t *create_nsec3_node(knot_dname_t *owner,
 	int ret = create_nsec3_rrset(&nsec3_rrset, owner, nsec3_params,
 	                             rr_types, NULL, ttl);
 	if (ret != KNOT_EOK) {
-		node_free(&new_node, NULL);
+		node_free(new_node, NULL);
 		return NULL;
 	}
 
 	ret = node_add_rrset(new_node, &nsec3_rrset, NULL);
 	knot_rrset_clear(&nsec3_rrset, NULL);
 	if (ret != KNOT_EOK) {
-		node_free(&new_node, NULL);
+		node_free(new_node, NULL);
 		return NULL;
 	}
 
@@ -326,9 +337,10 @@ static zone_node_t *create_nsec3_node_for_node(const zone_node_t *node,
 	assert(apex);
 	assert(params);
 
-	knot_dname_t *nsec3_owner;
-	nsec3_owner = knot_create_nsec3_owner(node->owner, apex->owner, params);
-	if (!nsec3_owner) {
+	uint8_t nsec3_owner[KNOT_DNAME_MAXLEN];
+	int ret = knot_create_nsec3_owner(nsec3_owner, sizeof(nsec3_owner),
+	                                  node->owner, apex->owner, params);
+	if (ret != KNOT_EOK) {
 		return NULL;
 	}
 
@@ -345,8 +357,8 @@ static zone_node_t *create_nsec3_node_for_node(const zone_node_t *node,
 		dnssec_nsec_bitmap_add(rr_types, KNOT_RRTYPE_NSEC3PARAM);
 	}
 
-	zone_node_t *nsec3_node;
-	nsec3_node = create_nsec3_node(nsec3_owner, params, apex, rr_types, ttl);
+	zone_node_t *nsec3_node = create_nsec3_node(nsec3_owner, params, apex,
+	                                            rr_types, ttl);
 	dnssec_nsec_bitmap_free(rr_types);
 
 	return nsec3_node;
@@ -358,14 +370,13 @@ static zone_node_t *create_nsec3_node_for_node(const zone_node_t *node,
 static int connect_nsec3_base(knot_rdataset_t *a_rrs, const knot_dname_t *b_name)
 {
 	assert(a_rrs);
-	uint8_t algorithm = knot_nsec3_algorithm(a_rrs, 0);
+	uint8_t algorithm = knot_nsec3_alg(a_rrs->rdata);
 	if (algorithm == 0) {
 		return KNOT_EINVAL;
 	}
 
-	uint8_t *raw_hash = NULL;
-	uint8_t raw_length = 0;
-	knot_nsec3_next_hashed(a_rrs, 0, &raw_hash, &raw_length);
+	uint8_t raw_length = knot_nsec3_next_len(a_rrs->rdata);
+	uint8_t *raw_hash = (uint8_t *)knot_nsec3_next(a_rrs->rdata);
 	if (raw_hash == NULL) {
 		return KNOT_EINVAL;
 	}
@@ -447,7 +458,7 @@ static int connect_nsec3_nodes2(zone_node_t *a, zone_node_t *b,
 	// connect the copied rrset
 	int ret = connect_nsec3_base(&acopy->rrs, b->owner);
 	if (ret != KNOT_EOK || knot_rrset_equal(&aorig, acopy, KNOT_RRSET_COMPARE_WHOLE)) {
-		knot_rrset_free(&acopy, NULL);
+		knot_rrset_free(acopy, NULL);
 		return ret;
 	}
 
@@ -460,7 +471,7 @@ static int connect_nsec3_nodes2(zone_node_t *a, zone_node_t *b,
 	if (ret == KNOT_EOK) {
 		ret = changeset_add_addition(data->changeset, acopy, CHANGESET_CHECK | CHANGESET_CHECK_CANCELOUT);
 	}
-	knot_rrset_free(&acopy, NULL);
+	knot_rrset_free(acopy, NULL);
 	return ret;
 }
 
@@ -563,8 +574,12 @@ static int fix_nsec3_for_node(zone_update_t *update, const dnssec_nsec3_params_t
 		return KNOT_ENORECORD;
 	}
 
-	int ret = KNOT_EOK;
-	knot_dname_t *for_node_hashed = knot_create_nsec3_owner(for_node, update->new_cont->apex->owner, params);
+	uint8_t for_node_hashed[KNOT_DNAME_MAXLEN];
+	int ret = knot_create_nsec3_owner(for_node_hashed, sizeof(for_node_hashed),
+	                                  for_node, update->new_cont->apex->owner, params);
+	if (ret != KNOT_EOK) {
+		return ret;
+	}
 
 	// saved hash of next node
 	uint8_t *next_hash = NULL;
@@ -590,10 +605,10 @@ static int fix_nsec3_for_node(zone_update_t *update, const dnssec_nsec3_params_t
 					ret = changeset_add_removal(chgset, &rem_rrsig, 0);
 				}
 			}
-			knot_nsec3_next_hashed(&rem_nsec3.rrs, 0, &next_hash, &next_length);
+			next_hash = (uint8_t *)knot_nsec3_next(rem_nsec3.rrs.rdata);
+			next_length = knot_nsec3_next_len(rem_nsec3.rrs.rdata);
 		}
 	}
-	free(for_node_hashed);
 
 	// add NSEC3 with correct bitmap
 	if (add_nsec3 && ret == KNOT_EOK) {
@@ -606,9 +621,8 @@ static int fix_nsec3_for_node(zone_update_t *update, const dnssec_nsec3_params_t
 
 		// copy hash of next element from removed record
 		if (next_hash != NULL) {
-			uint8_t *raw_hash = NULL;
-			uint8_t raw_length = 0;
-			knot_nsec3_next_hashed(&add_nsec3.rrs, 0, &raw_hash, &raw_length);
+			uint8_t *raw_hash = (uint8_t *)knot_nsec3_next(add_nsec3.rrs.rdata);
+			uint8_t raw_length = knot_nsec3_next_len(add_nsec3.rrs.rdata);
 			assert(raw_hash != NULL);
 			if (raw_length != next_length) {
 				ret = KNOT_EMALF;
@@ -624,7 +638,7 @@ static int fix_nsec3_for_node(zone_update_t *update, const dnssec_nsec3_params_t
 			}
 		}
 		node_free_rrsets(new_nsec3_n, NULL);
-		node_free(&new_nsec3_n, NULL);
+		node_free(new_nsec3_n, NULL);
 	}
 
 	return ret;
