@@ -1,4 +1,4 @@
-/*  Copyright (C) 2017 CZ.NIC, z.s.p.o. <knot-dns@labs.nic.cz>
+/*  Copyright (C) 2018 CZ.NIC, z.s.p.o. <knot-dns@labs.nic.cz>
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -17,7 +17,7 @@
 #include <assert.h>
 
 #include "contrib/wire_ctx.h"
-#include "dnssec/error.h"
+#include "libdnssec/error.h"
 #include "knot/dnssec/rrset-sign.h"
 #include "libknot/libknot.h"
 
@@ -52,9 +52,7 @@ static size_t rrsig_rdata_header_size(const dnssec_key_t *key)
 
 	// variable part
 
-	const uint8_t *signer = dnssec_key_get_dname(key);
-	assert(signer);
-	size += knot_dname_size(signer);
+	size += knot_dname_size(dnssec_key_get_dname(key));
 
 	return size;
 }
@@ -226,11 +224,9 @@ static int rrsigs_create_rdata(knot_rrset_t *rrsigs, dnssec_sign_ctx_t *ctx,
 	}
 
 	uint8_t header[header_size];
-	const knot_rdata_t *covered_data = knot_rdataset_at(&covered->rrs, 0);
 	int res = rrsig_write_rdata(header, header_size,
 	                            key, covered->type, owner_labels,
-	                            knot_rdata_ttl(covered_data),
-	                            sig_incepted, sig_expires);
+	                            covered->ttl, sig_incepted, sig_expires);
 	assert(res == KNOT_EOK);
 
 	res = dnssec_sign_init(ctx);
@@ -257,8 +253,7 @@ static int rrsigs_create_rdata(knot_rrset_t *rrsigs, dnssec_sign_ctx_t *ctx,
 
 	dnssec_binary_free(&signature);
 
-	return knot_rrset_add_rdata(rrsigs, rrsig, rrsig_size,
-	                            knot_rdata_ttl(covered_data), mm);
+	return knot_rrset_add_rdata(rrsigs, rrsig, rrsig_size, mm);
 }
 
 int knot_sign_rrset(knot_rrset_t *rrsigs, const knot_rrset_t *covered,
@@ -280,28 +275,29 @@ int knot_sign_rrset(knot_rrset_t *rrsigs, const knot_rrset_t *covered,
 }
 
 int knot_synth_rrsig(uint16_t type, const knot_rdataset_t *rrsig_rrs,
-                knot_rdataset_t *out_sig, knot_mm_t *mm)
+                     knot_rdataset_t *out_sig, knot_mm_t *mm)
 {
 	if (rrsig_rrs == NULL) {
 		return KNOT_ENOENT;
 	}
 
-	if (out_sig == NULL || out_sig->rr_count > 0) {
+	if (out_sig == NULL || out_sig->count > 0) {
 		return KNOT_EINVAL;
 	}
 
-	for (int i = 0; i < rrsig_rrs->rr_count; ++i) {
-		if (type == knot_rrsig_type_covered(rrsig_rrs, i)) {
-			const knot_rdata_t *rr_to_copy = knot_rdataset_at(rrsig_rrs, i);
+	knot_rdata_t *rr_to_copy = rrsig_rrs->rdata;
+	for (int i = 0; i < rrsig_rrs->count; ++i) {
+		if (type == knot_rrsig_type_covered(rr_to_copy)) {
 			int ret = knot_rdataset_add(out_sig, rr_to_copy, mm);
 			if (ret != KNOT_EOK) {
 				knot_rdataset_clear(out_sig, mm);
 				return ret;
 			}
 		}
+		rr_to_copy = knot_rdataset_next(rr_to_copy);
 	}
 
-	return out_sig->rr_count > 0 ? KNOT_EOK : KNOT_ENOENT;
+	return out_sig->count > 0 ? KNOT_EOK : KNOT_ENOENT;
 }
 
 /*- Verification of signatures -----------------------------------------------*/
@@ -309,19 +305,17 @@ int knot_synth_rrsig(uint16_t type, const knot_rdataset_t *rrsig_rrs,
 /*!
  * \brief Check if the signature is expired.
  *
- * \param rrsigs  RR set with RRSIGs.
- * \param pos     Number of RR in the RR set.
+ * \param rrsig   RRSIG rdata.
  * \param policy  DNSSEC policy.
  *
  * \return Signature is expired or should be replaced soon.
  */
-static bool is_expired_signature(const knot_rrset_t *rrsigs, size_t pos,
-                                 uint32_t now, uint32_t refresh_before)
+static bool is_expired_signature(const knot_rdata_t *rrsig, uint32_t now,
+                                 uint32_t refresh_before)
 {
-	assert(!knot_rrset_empty(rrsigs));
-	assert(rrsigs->type == KNOT_RRTYPE_RRSIG);
+	assert(rrsig);
 
-	uint32_t expire_at = knot_rrsig_sig_expiration(&rrsigs->rrs, pos);
+	uint32_t expire_at = knot_rrsig_sig_expiration(rrsig);
 	uint32_t expire_in = expire_at > now ? expire_at - now : 0;
 
 	return expire_in <= refresh_before;
@@ -338,22 +332,21 @@ int knot_check_signature(const knot_rrset_t *covered,
 		return KNOT_EINVAL;
 	}
 
-	if (is_expired_signature(rrsigs, pos, dnssec_ctx->now,
+	knot_rdata_t *rrsig = knot_rdataset_at(&rrsigs->rrs, pos);
+	assert(rrsig);
+
+	if (is_expired_signature(rrsig, dnssec_ctx->now,
 	                         dnssec_ctx->policy->rrsig_refresh_before)) {
 		return DNSSEC_INVALID_SIGNATURE;
 	}
 
 	// identify fields in the signature being validated
 
-	const knot_rdata_t *rr_data = knot_rdataset_at(&rrsigs->rrs, pos);
-	uint8_t *rdata = knot_rdata_data(rr_data);
-	if (!rdata) {
-		return KNOT_EINVAL;
-	}
-
-	dnssec_binary_t signature = { 0 };
-	knot_rrsig_signature(&rrsigs->rrs, pos, &signature.data, &signature.size);
-	if (!signature.data) {
+	dnssec_binary_t signature = {
+		.size = knot_rrsig_signature_len(rrsig),
+		.data = (uint8_t *)knot_rrsig_signature(rrsig)
+	};
+	if (signature.data == NULL) {
 		return KNOT_EINVAL;
 	}
 
@@ -364,7 +357,7 @@ int knot_check_signature(const knot_rrset_t *covered,
 		return result;
 	}
 
-	result = knot_sign_ctx_add_data(sign_ctx, rdata, covered);
+	result = knot_sign_ctx_add_data(sign_ctx, rrsig->data, covered);
 	if (result != KNOT_EOK) {
 		return result;
 	}
