@@ -1,4 +1,4 @@
-/*  Copyright (C) 2017 CZ.NIC, z.s.p.o. <knot-dns@labs.nic.cz>
+/*  Copyright (C) 2018 CZ.NIC, z.s.p.o. <knot-dns@labs.nic.cz>
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -16,8 +16,8 @@
 
 #pragma once
 
-#include "dnssec/key.h"
-#include "dnssec/sign.h"
+#include "libdnssec/key.h"
+#include "libdnssec/sign.h"
 #include "knot/dnssec/context.h"
 #include "libknot/rrset.h"
 
