@@ -1,4 +1,4 @@
-/*  Copyright (C) 2017 CZ.NIC, z.s.p.o. <knot-dns@labs.nic.cz>
+/*  Copyright (C) 2018 CZ.NIC, z.s.p.o. <knot-dns@labs.nic.cz>
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -16,8 +16,8 @@
 
 #include <assert.h>
 
-#include "dnssec/error.h"
-#include "dnssec/random.h"
+#include "libdnssec/error.h"
+#include "libdnssec/random.h"
 #include "libknot/libknot.h"
 #include "knot/conf/conf.h"
 #include "knot/common/log.h"
@@ -201,7 +201,7 @@ int knot_dnssec_zone_sign(zone_update_t *update,
 		goto done;
 	}
 
-	if (!(flags & ZONE_SIGN_KEEP_SERIAL)) {
+	if (!(flags & ZONE_SIGN_KEEP_SERIAL) && zone_update_to(update) == NULL) {
 		result = zone_update_increment_soa(update, conf());
 		if (result == KNOT_EOK) {
 			result = knot_zone_sign_soa(update, &keyset, &ctx);
@@ -269,8 +269,8 @@ int knot_dnssec_sign_update(zone_update_t *update, zone_sign_reschedule_t *resch
 		goto done;
 	}
 
-	bool soa_changed = (knot_soa_serial(node_rdataset(update->zone->contents->apex, KNOT_RRTYPE_SOA)) !=
-			    knot_soa_serial(node_rdataset(update->new_cont->apex, KNOT_RRTYPE_SOA)));
+	bool soa_changed = (knot_soa_serial(node_rdataset(update->zone->contents->apex, KNOT_RRTYPE_SOA)->rdata) !=
+			    knot_soa_serial(node_rdataset(update->new_cont->apex, KNOT_RRTYPE_SOA)->rdata));
 
 	if (zone_update_no_change(update) && !soa_changed &&
 	    !knot_zone_sign_soa_expired(update->new_cont, &keyset, &ctx)) {
@@ -304,5 +304,5 @@ done:
 	free_zone_keys(&keyset);
 	kdnssec_ctx_deinit(&ctx);
 
-	return KNOT_EOK;
+	return result;
 }
