@@ -1,4 +1,4 @@
-/*  Copyright (C) 2017 CZ.NIC, z.s.p.o. <knot-dns@labs.nic.cz>
+/*  Copyright (C) 2018 CZ.NIC, z.s.p.o. <knot-dns@labs.nic.cz>
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -17,10 +17,10 @@
 #include <assert.h>
 #include <sys/types.h>
 
-#include "dnssec/error.h"
-#include "dnssec/key.h"
-#include "dnssec/keytag.h"
-#include "dnssec/sign.h"
+#include "libdnssec/error.h"
+#include "libdnssec/key.h"
+#include "libdnssec/keytag.h"
+#include "libdnssec/sign.h"
 #include "knot/dnssec/key-events.h"
 #include "knot/dnssec/rrset-sign.h"
 #include "knot/dnssec/zone-sign.h"
@@ -49,7 +49,7 @@ static knot_rrset_t rrset_init_from(const knot_rrset_t *src, uint16_t type)
 {
 	assert(src);
 	knot_rrset_t rrset;
-	knot_rrset_init(&rrset, src->owner, type, src->rclass);
+	knot_rrset_init(&rrset, src->owner, type, src->rclass, src->ttl);
 	return rrset;
 }
 
@@ -110,10 +110,12 @@ static bool valid_signature_exists(const knot_rrset_t *covered,
 		return false;
 	}
 
-	uint16_t rrsigs_rdata_count = rrsigs->rrs.rr_count;
+	uint16_t rrsigs_rdata_count = rrsigs->rrs.count;
+	knot_rdata_t *rdata = rrsigs->rrs.rdata;
 	for (uint16_t i = 0; i < rrsigs_rdata_count; i++) {
-		uint16_t rr_keytag = knot_rrsig_key_tag(&rrsigs->rrs, i);
-		uint16_t rr_covered = knot_rrsig_type_covered(&rrsigs->rrs, i);
+		uint16_t rr_keytag = knot_rrsig_key_tag(rdata);
+		uint16_t rr_covered = knot_rrsig_type_covered(rdata);
+		rdata = knot_rdataset_next(rdata);
 
 		uint16_t keytag = dnssec_key_get_keytag(key);
 		if (rr_keytag != keytag || rr_covered != covered->type) {
@@ -129,31 +131,6 @@ static bool valid_signature_exists(const knot_rrset_t *covered,
 	return false;
 }
 
-/*!
- * \brief Check if key can be used to sign given RR.
- *
- * \param key      Zone key.
- * \param covered  RR to be checked.
- *
- * \return The RR should be signed.
- */
-static bool use_key(const zone_key_t *key, const knot_rrset_t *covered)
-{
-	assert(key);
-	assert(covered);
-
-	if (!key->is_active) {
-		return false;
-	}
-
-	bool is_apex = knot_dname_is_equal(covered->owner,
-	                                   dnssec_key_get_dname(key->key));
-
-	bool is_zone_key = is_apex && covered->type == KNOT_RRTYPE_DNSKEY;
-
-	return (key->is_ksk && is_zone_key) || (key->is_zsk && !is_zone_key);
-}
-
 /*!
  * \brief Check if valid signature exist for all keys for a given RR set.
  *
@@ -174,7 +151,7 @@ static bool all_signatures_exist(const knot_rrset_t *covered,
 
 	for (int i = 0; i < zone_keys->count; i++) {
 		zone_key_t *key = &zone_keys->keys[i];
-		if (!use_key(key, covered)) {
+		if (!knot_zone_sign_use_key(key, covered)) {
 			continue;
 		}
 
@@ -187,40 +164,18 @@ static bool all_signatures_exist(const knot_rrset_t *covered,
 	return true;
 }
 
-/*!
- * \brief Get zone key for given RRSIG (checks key tag only).
- *
- * \param rrsigs  RR set with RRSIGs.
- * \param pos     Number of RR in RR set.
- * \param keys    Zone keys.
- *
- * \return Dynarray of such keys.
- */
-static keyptr_dynarray_t get_matching_zone_keys(const knot_rrset_t *rrsigs,
-                                                     size_t pos, const zone_keyset_t *keys)
-{
-	assert(rrsigs && rrsigs->type == KNOT_RRTYPE_RRSIG);
-	assert(keys);
-
-	uint16_t keytag = knot_rrsig_key_tag(&rrsigs->rrs, pos);
-
-	return get_zone_keys(keys, keytag);
-}
-
 /*!
  * \brief Note earliest expiration of a signature.
  *
- * \param rrsigs      RR set with RRSIGs.
- * \param pos         Position of RR in rrsigs.
+ * \param rrsig       RRSIG rdata.
  * \param expires_at  Current earliest expiration, will be updated.
  */
-static void note_earliest_expiration(const knot_rrset_t *rrsigs, size_t pos,
-                                     knot_time_t *expires_at)
+static void note_earliest_expiration(const knot_rdata_t *rrsig, knot_time_t *expires_at)
 {
-	assert(rrsigs);
+	assert(rrsig);
 	assert(expires_at);
 
-	uint32_t curr_rdata = knot_rrsig_sig_expiration(&rrsigs->rrs, pos);
+	uint32_t curr_rdata = knot_rrsig_sig_expiration(rrsig);
 	knot_time_t current = knot_time_from_u32(curr_rdata);
 	*expires_at = knot_time_min(current, *expires_at);
 }
@@ -265,9 +220,12 @@ static int remove_expired_rrsigs(const knot_rrset_t *covered,
 		return KNOT_EOK;
 	}
 
-	uint16_t rrsig_rdata_count = synth_rrsig.rrs.rr_count;
+	uint16_t rrsig_rdata_count = synth_rrsig.rrs.count;
 	for (uint16_t i = 0; i < rrsig_rdata_count; i++) {
-		struct keyptr_dynarray keys = get_matching_zone_keys(&synth_rrsig, i, zone_keys);
+		knot_rdata_t *rr = knot_rdataset_at(&synth_rrsig.rrs, i);
+		uint16_t keytag = knot_rrsig_key_tag(rr);
+
+		struct keyptr_dynarray keys = get_zone_keys(zone_keys, keytag);
 		int endloop = 0; // 1 - continue; 2 - break
 
 		dynarray_foreach(keyptr, zone_key_t *, key, keys) {
@@ -278,7 +236,7 @@ static int remove_expired_rrsigs(const knot_rrset_t *covered,
 			                              (*key)->key, (*key)->ctx, dnssec_ctx);
 			if (result == KNOT_EOK) {
 				// valid signature
-				note_earliest_expiration(&synth_rrsig, i, expires_at);
+				note_earliest_expiration(rr, expires_at);
 				endloop = 1;
 				break;
 			} else if (result != DNSSEC_INVALID_SIGNATURE) {
@@ -298,8 +256,7 @@ static int remove_expired_rrsigs(const knot_rrset_t *covered,
 			to_remove = create_empty_rrsigs_for(&synth_rrsig);
 		}
 
-		knot_rdata_t *rr_rem = knot_rdataset_at(&synth_rrsig.rrs, i);
-		result = knot_rdataset_add(&to_remove.rrs, rr_rem, NULL);
+		result = knot_rdataset_add(&to_remove.rrs, rr, NULL);
 		if (result != KNOT_EOK) {
 			break;
 		}
@@ -342,7 +299,7 @@ static int add_missing_rrsigs(const knot_rrset_t *covered,
 
 	for (int i = 0; i < zone_keys->count; i++) {
 		const zone_key_t *key = &zone_keys->keys[i];
-		if (!use_key(key, covered)) {
+		if (!knot_zone_sign_use_key(key, covered)) {
 			continue;
 		}
 
@@ -385,7 +342,7 @@ static int remove_rrset_rrsigs(const knot_dname_t *owner, uint16_t type,
 	assert(changeset);
 	knot_rrset_t synth_rrsig;
 	knot_rrset_init(&synth_rrsig, (knot_dname_t *)owner,
-	                KNOT_RRTYPE_RRSIG, rrsigs->rclass);
+	                KNOT_RRTYPE_RRSIG, rrsigs->rclass, rrsigs->ttl);
 	int ret = knot_synth_rrsig(type, &rrsigs->rrs, &synth_rrsig.rrs, NULL);
 	if (ret != KNOT_EOK) {
 		if (ret != KNOT_ENOENT) {
@@ -468,15 +425,15 @@ static int remove_standalone_rrsigs(const zone_node_t *node,
 		return KNOT_EOK;
 	}
 
-	uint16_t rrsigs_rdata_count = rrsigs->rrs.rr_count;
+	uint16_t rrsigs_rdata_count = rrsigs->rrs.count;
+	knot_rdata_t *rdata = rrsigs->rrs.rdata;
 	for (uint16_t i = 0; i < rrsigs_rdata_count; ++i) {
-		uint16_t type_covered = knot_rrsig_type_covered(&rrsigs->rrs, i);
+		uint16_t type_covered = knot_rrsig_type_covered(rdata);
 		if (!node_rrtype_exists(node, type_covered)) {
 			knot_rrset_t to_remove;
 			knot_rrset_init(&to_remove, rrsigs->owner, rrsigs->type,
-			                rrsigs->rclass);
-			knot_rdata_t *rr_rem = knot_rdataset_at(&rrsigs->rrs, i);
-			int ret = knot_rdataset_add(&to_remove.rrs, rr_rem, NULL);
+			                rrsigs->rclass, rrsigs->ttl);
+			int ret = knot_rdataset_add(&to_remove.rrs, rdata, NULL);
 			if (ret != KNOT_EOK) {
 				return ret;
 			}
@@ -486,6 +443,7 @@ static int remove_standalone_rrsigs(const zone_node_t *node,
 				return ret;
 			}
 		}
+		rdata = knot_rdataset_next(rdata);
 	}
 
 	return KNOT_EOK;
@@ -627,9 +585,7 @@ typedef struct {
 
 /*- private API - DNSKEY handling --------------------------------------------*/
 
-static int rrset_add_zone_key(knot_rrset_t *rrset,
-                              zone_key_t *zone_key,
-                              uint32_t ttl)
+static int rrset_add_zone_key(knot_rrset_t *rrset, zone_key_t *zone_key)
 {
 	assert(rrset);
 	assert(zone_key);
@@ -637,13 +593,10 @@ static int rrset_add_zone_key(knot_rrset_t *rrset,
 	dnssec_binary_t dnskey_rdata = { 0 };
 	dnssec_key_get_rdata(zone_key->key, &dnskey_rdata);
 
-	return knot_rrset_add_rdata(rrset, dnskey_rdata.data,
-	                            dnskey_rdata.size, ttl, NULL);
+	return knot_rrset_add_rdata(rrset, dnskey_rdata.data, dnskey_rdata.size, NULL);
 }
 
-static int rrset_add_zone_ds(knot_rrset_t *rrset,
-                             zone_key_t *zone_key,
-                             uint32_t ttl)
+static int rrset_add_zone_ds(knot_rrset_t *rrset, zone_key_t *zone_key)
 {
 	assert(rrset);
 	assert(zone_key);
@@ -651,8 +604,7 @@ static int rrset_add_zone_ds(knot_rrset_t *rrset,
 	dnssec_binary_t cds_rdata = { 0 };
 	zone_key_calculate_ds(zone_key, &cds_rdata);
 
-	return knot_rrset_add_rdata(rrset, cds_rdata.data,
-	                            cds_rdata.size, ttl, NULL);
+	return knot_rrset_add_rdata(rrset, cds_rdata.data, cds_rdata.size, NULL);
 }
 
 /*!
@@ -711,8 +663,9 @@ static int rr_already_signed(const knot_rrset_t *rrset, trie_t *t,
 	assert(t);
 	*rr_signed = false;
 	// Create a key = RRSet owner converted to sortable format
-	uint8_t lf[KNOT_DNAME_MAXLEN];
-	knot_dname_lf(lf, rrset->owner, NULL);
+	knot_dname_storage_t lf_storage;
+	uint8_t *lf = knot_dname_lf(rrset->owner, lf_storage);
+	assert(lf);
 	trie_val_t stored_info = (signed_info_t *)trie_get_try(t, (char *)lf+1,
 	                                                      *lf);
 	if (stored_info == NULL) {
@@ -841,8 +794,8 @@ static int free_helper_trie_node(trie_val_t *val, void *d)
 		WALK_LIST_FREE(*(info->type_list));
 	}
 	free(info->type_list);
-	knot_dname_free(&info->dname, NULL);
-	knot_dname_free(&info->hashed_dname, NULL);
+	knot_dname_free(info->dname, NULL);
+	knot_dname_free(info->hashed_dname, NULL);
 	free(info);
 	return KNOT_EOK;
 }
@@ -939,9 +892,12 @@ int knot_zone_sign_update_dnskeys(zone_update_t *update,
 	CHECK_RET;
 
 	// add DNSKEYs, CDNSKEYs and CDSs
-	add_dnskeys = knot_rrset_new(apex->owner, KNOT_RRTYPE_DNSKEY, soa.rclass, NULL);
-	add_cdnskeys = knot_rrset_new(apex->owner, KNOT_RRTYPE_CDNSKEY, soa.rclass, NULL);
-	add_cdss = knot_rrset_new(apex->owner, KNOT_RRTYPE_CDS, soa.rclass, NULL);
+	add_dnskeys = knot_rrset_new(apex->owner, KNOT_RRTYPE_DNSKEY, soa.rclass,
+	                             dnskey_ttl, NULL);
+	add_cdnskeys = knot_rrset_new(apex->owner, KNOT_RRTYPE_CDNSKEY, soa.rclass,
+	                              0, NULL);
+	add_cdss = knot_rrset_new(apex->owner, KNOT_RRTYPE_CDS, soa.rclass,
+	                          0, NULL);
 	if (add_dnskeys == NULL || add_cdnskeys == NULL || add_cdss == NULL) {
 		ret = KNOT_ENOMEM;
 		CHECK_RET;
@@ -952,7 +908,7 @@ int knot_zone_sign_update_dnskeys(zone_update_t *update,
 	for (int i = 0; i < zone_keys->count; i++) {
 		zone_key_t *key = &zone_keys->keys[i];
 		if (key->is_public) {
-			ret = rrset_add_zone_key(add_dnskeys, key, dnskey_ttl);
+			ret = rrset_add_zone_key(add_dnskeys, key);
 			CHECK_RET;
 		}
 
@@ -964,9 +920,9 @@ int knot_zone_sign_update_dnskeys(zone_update_t *update,
 	}
 
 	if (ksk_for_cds != NULL) {
-		ret = rrset_add_zone_key(add_cdnskeys, ksk_for_cds, 0);
+		ret = rrset_add_zone_key(add_cdnskeys, ksk_for_cds);
 		CHECK_RET;
-		ret = rrset_add_zone_ds(add_cdss, ksk_for_cds, 0);
+		ret = rrset_add_zone_ds(add_cdss, ksk_for_cds);
 		CHECK_RET;
 	}
 
@@ -974,10 +930,10 @@ int knot_zone_sign_update_dnskeys(zone_update_t *update,
 		const uint8_t cdnskey_empty[5] = { 0, 0, 3, 0, 0 };
 		const uint8_t cds_empty[5] = { 0, 0, 0, 0, 0 };
 		ret = knot_rrset_add_rdata(add_cdnskeys, cdnskey_empty,
-		                           sizeof(cdnskey_empty), 0, NULL);
+		                           sizeof(cdnskey_empty), NULL);
 		CHECK_RET;
 		ret = knot_rrset_add_rdata(add_cdss, cds_empty,
-		                           sizeof(cds_empty), 0, NULL);
+		                           sizeof(cds_empty), NULL);
 		CHECK_RET;
 	}
 
@@ -1004,13 +960,44 @@ int knot_zone_sign_update_dnskeys(zone_update_t *update,
 #undef CHECK_RET
 
 cleanup:
-	knot_rrset_free(&add_dnskeys, NULL);
-	knot_rrset_free(&add_cdnskeys, NULL);
-	knot_rrset_free(&add_cdss, NULL);
+	knot_rrset_free(add_dnskeys, NULL);
+	knot_rrset_free(add_cdnskeys, NULL);
+	knot_rrset_free(add_cdss, NULL);
 	changeset_clear(&ch);
 	return ret;
 }
 
+bool knot_zone_sign_use_key(const zone_key_t *key, const knot_rrset_t *covered)
+{
+	if (key == NULL || covered == NULL) {
+		return KNOT_EINVAL;
+	}
+
+	if (!key->is_active) {
+		return false;
+	}
+
+	// this may be a problem with offline KSK
+	bool cds_sign_by_ksk = true;
+
+	assert(key->is_zsk || key->is_ksk);
+	bool is_apex = knot_dname_is_equal(covered->owner,
+	                                   dnssec_key_get_dname(key->key));
+	if (!is_apex) {
+		return key->is_zsk;
+	}
+
+	switch (covered->type) {
+	case KNOT_RRTYPE_DNSKEY:
+		return key->is_ksk;
+	case KNOT_RRTYPE_CDS:
+	case KNOT_RRTYPE_CDNSKEY:
+		return (cds_sign_by_ksk ? key->is_ksk : key->is_zsk);
+	default:
+		return key->is_zsk;
+	}
+}
+
 bool knot_zone_sign_soa_expired(const zone_contents_t *zone,
                                 const zone_keyset_t *zone_keys,
                                 const kdnssec_ctx_t *dnssec_ctx)
