@@ -1,4 +1,4 @@
-/*  Copyright (C) 2017 CZ.NIC, z.s.p.o. <knot-dns@labs.nic.cz>
+/*  Copyright (C) 2018 CZ.NIC, z.s.p.o. <knot-dns@labs.nic.cz>
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -15,7 +15,6 @@
  */
 
 #include <assert.h>
-#include <urcu.h>
 
 #include "knot/common/log.h"
 #include "knot/conf/conf.h"
@@ -43,7 +42,6 @@ void event_dnssec_reschedule(conf_t *conf, zone_t *zone,
 	log_dnssec_next(zone->name, (time_t)refresh_at);
 
 	if (refresh->plan_ds_query) {
-		log_zone_notice(zone->name, "DNSSEC, KSK submission, waiting for confirmation");
 		zone->timers.next_parent_ds_q = now;
 	}
 
