@@ -18,7 +18,7 @@
 #include <stdint.h>
 
 #include "contrib/mempattern.h"
-#include "dnssec/random.h"
+#include "libdnssec/random.h"
 #include "knot/common/log.h"
 #include "knot/conf/conf.h"
 #include "knot/dnssec/zone-events.h"
@@ -205,7 +205,8 @@ static int axfr_init(struct refresh_data *data)
 
 static void axfr_cleanup(struct refresh_data *data)
 {
-	zone_contents_deep_free(&data->axfr.zone);
+	zone_contents_deep_free(data->axfr.zone);
+	data->axfr.zone = NULL;
 }
 
 static int axfr_finalize(struct refresh_data *data)
@@ -240,6 +241,9 @@ static int axfr_finalize(struct refresh_data *data)
 		return ret;
 	}
 
+	// Seized by zone_update. Don't free the contents again in axfr_cleanup.
+	data->axfr.zone = NULL;
+
 	conf_val_t val = conf_zone_get(data->conf, C_DNSSEC_SIGNING, data->zone->name);
 	bool dnssec_enable = conf_bool(&val);
 	if (dnssec_enable) {
@@ -271,7 +275,6 @@ static int axfr_finalize(struct refresh_data *data)
 
 	xfr_log_publish(data->zone->name, data->remote, old_serial,
 	                zone_contents_serial(new_zone), bootstrap);
-	data->axfr.zone = NULL; // seized
 
 	return KNOT_EOK;
 }
@@ -355,7 +358,7 @@ static int axfr_consume(knot_pkt_t *pkt, struct refresh_data *data)
 	// Process saved SOA if fallback from IXFR
 	if (data->initial_soa_copy != NULL) {
 		next = axfr_consume_rr(data->initial_soa_copy, data);
-		knot_rrset_free(&data->initial_soa_copy, data->mm);
+		knot_rrset_free(data->initial_soa_copy, data->mm);
 		if (next != KNOT_STATE_CONSUME) {
 			return next;
 		}
@@ -400,7 +403,7 @@ static void ixfr_cleanup(struct refresh_data *data)
 		return;
 	}
 
-	knot_rrset_free(&data->ixfr.final_soa, data->mm);
+	knot_rrset_free(data->ixfr.final_soa, data->mm);
 	mm_free(data->mm, data->ixfr.proc);
 	data->ixfr.proc = NULL;
 
@@ -417,15 +420,15 @@ static int ixfr_finalize(struct refresh_data *data)
 	uint32_t old_serial = local_serial;
 	changeset_t *chs = NULL;
 	WALK_LIST(chs, data->ixfr.changesets) {
-		master_serial = knot_soa_serial(&chs->soa_to->rrs);
-		knot_soa_serial_set(&chs->soa_from->rrs, local_serial);
+		master_serial = knot_soa_serial(chs->soa_to->rrs.rdata);
+		knot_soa_serial_set(chs->soa_from->rrs.rdata, local_serial);
 		if (have_lastsigned && (serial_compare(lastsigned_serial, local_serial) & SERIAL_MASK_GEQ)) {
 			local_serial = lastsigned_serial;
 		}
 		if (serial_compare(master_serial, local_serial) != SERIAL_GREATER) {
 			conf_val_t val = conf_zone_get(data->conf, C_SERIAL_POLICY, data->zone->name);
 			local_serial = serial_next(local_serial, conf_opt(&val));
-			knot_soa_serial_set(&chs->soa_to->rrs, local_serial);
+			knot_soa_serial_set(chs->soa_to->rrs.rdata, local_serial);
 		} else {
 			local_serial = master_serial;
 		}
@@ -528,7 +531,7 @@ static int ixfr_solve_soa_del(const knot_rrset_t *rr, struct refresh_data *data)
 	// Store SOA into changeset.
 	change->soa_from = knot_rrset_copy(rr, NULL);
 	if (change->soa_from == NULL) {
-		changeset_clear(change);
+		changeset_free(change);
 		return KNOT_ENOMEM;
 	}
 
@@ -633,7 +636,7 @@ static int ixfr_step(const knot_rrset_t *rr, struct refresh_data *data)
 
 static int ixfr_consume_rr(const knot_rrset_t *rr, struct refresh_data *data)
 {
-	if (!knot_dname_in(data->zone->name, rr->owner)) {
+	if (knot_dname_in_bailiwick(rr->owner, data->zone->name) < 0) {
 		return KNOT_STATE_CONSUME;
 	}
 
@@ -695,7 +698,7 @@ static enum xfr_type determine_xfr_type(const knot_pktsection_t *answer,
 
 	if (answer->count == 1) {
 		if (rr_one->type == KNOT_RRTYPE_SOA) {
-			return serial_is_current(zone_serial, knot_soa_serial(&rr_one->rrs)) ?
+			return serial_is_current(zone_serial, knot_soa_serial(rr_one->rrs.rdata)) ?
 			       XFR_TYPE_UPTODATE : XFR_TYPE_UNDETERMINED;
 		}
 		return XFR_TYPE_ERROR;
@@ -728,7 +731,7 @@ static int ixfr_consume(knot_pkt_t *pkt, struct refresh_data *data)
 	if (data->ixfr.proc == NULL) {
 		const knot_pktsection_t *answer = knot_pkt_section(pkt, KNOT_ANSWER);
 
-		uint32_t master_serial = knot_soa_serial(&data->soa->rrs);
+		uint32_t master_serial = knot_soa_serial(data->soa->rrs.rdata);
 		(void)zone_get_master_serial(data->zone, &master_serial);
 		data->xfr_type = determine_xfr_type(answer, master_serial,
 		                                    data->initial_soa_copy);
@@ -779,7 +782,7 @@ static int ixfr_consume(knot_pkt_t *pkt, struct refresh_data *data)
 	// Process saved SOA if existing
 	if (data->initial_soa_copy != NULL) {
 		next = ixfr_consume_rr(data->initial_soa_copy, data);
-		knot_rrset_free(&data->initial_soa_copy, data->mm);
+		knot_rrset_free(data->initial_soa_copy, data->mm);
 		if (next != KNOT_STATE_CONSUME) {
 			return next;
 		}
@@ -829,15 +832,15 @@ static int soa_query_consume(knot_layer_t *layer, knot_pkt_t *pkt)
 
 	const knot_pktsection_t *answer = knot_pkt_section(pkt, KNOT_ANSWER);
 	const knot_rrset_t *rr = answer->count == 1 ? knot_pkt_rr(answer, 0) : NULL;
-	if (!rr || rr->type != KNOT_RRTYPE_SOA || rr->rrs.rr_count != 1) {
+	if (!rr || rr->type != KNOT_RRTYPE_SOA || rr->rrs.count != 1) {
 		REFRESH_LOG(LOG_WARNING, data->zone->name, data->remote,
 		            "malformed message");
 		return KNOT_STATE_FAIL;
 	}
 
-	uint32_t local_serial = knot_soa_serial(&data->soa->rrs);
+	uint32_t local_serial = knot_soa_serial(data->soa->rrs.rdata);
 	(void)zone_get_master_serial(data->zone, &local_serial);
-	uint32_t remote_serial = knot_soa_serial(&rr->rrs);
+	uint32_t remote_serial = knot_soa_serial(rr->rrs.rdata);
 	bool current = serial_is_current(local_serial, remote_serial);
 	bool master_uptodate = serial_is_current(remote_serial, local_serial);
 
@@ -870,13 +873,13 @@ static int transfer_produce(knot_layer_t *layer, knot_pkt_t *pkt)
 		uint32_t master_serial;
 		int ret = zone_get_master_serial(data->zone, &master_serial);
 		if (sending_soa == NULL || ret != KNOT_EOK) {
-			knot_rrset_free(&sending_soa, data->mm);
+			knot_rrset_free(sending_soa, data->mm);
 			return KNOT_STATE_FAIL;
 		}
-		knot_soa_serial_set(&sending_soa->rrs, master_serial);
+		knot_soa_serial_set(sending_soa->rrs.rdata, master_serial);
 		knot_pkt_begin(pkt, KNOT_AUTHORITY);
 		knot_pkt_put(pkt, KNOT_COMPR_HINT_QNAME, sending_soa, 0);
-		knot_rrset_free(&sending_soa, data->mm);
+		knot_rrset_free(sending_soa, data->mm);
 	}
 
 	query_put_edns(pkt, &data->edns);
@@ -1111,13 +1114,13 @@ int event_refresh(conf_t *conf, zone_t *zone)
 	const knot_rdataset_t *soa = zone_soa(zone);
 
 	if (ret == KNOT_EOK) {
-		zone->timers.soa_expire = knot_soa_expire(soa);
+		zone->timers.soa_expire = knot_soa_expire(soa->rdata);
 		zone->timers.last_refresh = now;
-		zone->timers.next_refresh = now + knot_soa_refresh(soa);
+		zone->timers.next_refresh = now + knot_soa_refresh(soa->rdata);
 	} else {
 		time_t next = 0;
 		if (soa) {
-			next = knot_soa_retry(soa);
+			next = knot_soa_retry(soa->rdata);
 		} else {
 			next = bootstrap_next(&zone->timers);
 		}
