@@ -1,4 +1,4 @@
-/*  Copyright (C) 2017 CZ.NIC, z.s.p.o. <knot-dns@labs.nic.cz>
+/*  Copyright (C) 2018 CZ.NIC, z.s.p.o. <knot-dns@labs.nic.cz>
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -19,13 +19,12 @@
 #include <string.h>
 
 #include "contrib/string.h"
-#include "dnssec/error.h"
+#include "libdnssec/error.h"
 #include "knot/include/module.h"
 #include "knot/modules/onlinesign/nsec_next.h"
 // Next dependencies force static module!
+#include "knot/dnssec/ds_query.h"
 #include "knot/dnssec/key-events.h"
-#include "knot/dnssec/rrset-sign.h"
-#include "knot/dnssec/zone-keys.h"
 #include "knot/dnssec/zone-events.h"
 #include "knot/nameserver/query_module.h"
 #include "knot/nameserver/process_query.h"
@@ -47,23 +46,6 @@ const yp_item_t online_sign_conf[] = {
 	{ NULL }
 };
 
-#define RRSIG_LIFETIME (25*60*60)
-
-/*
- * TODO:
- *
- * - Fix delegation signing:
- *   + The NSEC proof can decsend into the child zone.
- *   + Out-of-zone glue records can be signed.
- *
- * - Fix CNAME handling:
- *   + Owner name of synthesized records can be incorrect.
- *   + Combination with wildcards results in invalid signatures.
- *
- * - Add support for CDS/CDSKEY synthesis.
- *
- */
-
 /*!
  * \brief RR types to force in synthesised NSEC maps.
  *
@@ -85,8 +67,11 @@ static const uint16_t NSEC_FORCE_TYPES[] = {
 };
 
 typedef struct {
-	dnssec_key_t *key;
-	uint32_t rrsig_lifetime;
+	knot_time_t event_rollover;
+	knot_time_t event_parent_ds_q;
+	pthread_mutex_t event_mutex;
+
+	bool zone_doomed;
 } online_sign_ctx_t;
 
 static bool want_dnssec(knotd_qdata_t *qdata)
@@ -97,13 +82,13 @@ static bool want_dnssec(knotd_qdata_t *qdata)
 static uint32_t dnskey_ttl(knotd_qdata_t *qdata)
 {
 	knot_rrset_t soa = knotd_qdata_zone_apex_rrset(qdata, KNOT_RRTYPE_SOA);
-	return knot_rrset_ttl(&soa);
+	return soa.ttl;
 }
 
 static uint32_t nsec_ttl(knotd_qdata_t *qdata)
 {
 	knot_rrset_t soa = knotd_qdata_zone_apex_rrset(qdata, KNOT_RRTYPE_SOA);
-	return knot_soa_minimum(&soa.rrs);
+	return knot_soa_minimum(soa.rrs.rdata);
 }
 
 /*!
@@ -166,7 +151,7 @@ static void bitmap_add_section(dnssec_nsec_bitmap_t *map, const knot_pktsection_
  * - For non-ANY query, the bitmap will contain types from zone and forced
  *   types which can be potentionally synthesized by other query modules.
  */
-static dnssec_nsec_bitmap_t *synth_bitmap(knot_pkt_t *pkt, const knotd_qdata_t *qdata)
+static dnssec_nsec_bitmap_t *synth_bitmap(knot_pkt_t *pkt, const knotd_qdata_t *qdata, bool add_forced)
 {
 	dnssec_nsec_bitmap_t *map = dnssec_nsec_bitmap_new();
 	if (!map) {
@@ -185,7 +170,7 @@ static dnssec_nsec_bitmap_t *synth_bitmap(knot_pkt_t *pkt, const knotd_qdata_t *
 		bitmap_add_section(map, answer);
 	} else {
 		bitmap_add_zone(map, qdata->extra->node);
-		if (!node_rrtype_exists(qdata->extra->node, KNOT_RRTYPE_CNAME)) {
+		if (add_forced && !node_rrtype_exists(qdata->extra->node, KNOT_RRTYPE_CNAME)) {
 			bitmap_add_forced(map, qtype);
 		}
 	}
@@ -193,24 +178,30 @@ static dnssec_nsec_bitmap_t *synth_bitmap(knot_pkt_t *pkt, const knotd_qdata_t *
 	return map;
 }
 
+static bool is_deleg(const knot_pkt_t *pkt)
+{
+	return !knot_wire_get_aa(pkt->wire);
+}
+
 static knot_rrset_t *synth_nsec(knot_pkt_t *pkt, knotd_qdata_t *qdata, knot_mm_t *mm)
 {
-	knot_rrset_t *nsec = knot_rrset_new(qdata->name, KNOT_RRTYPE_NSEC,
-	                                    KNOT_CLASS_IN, mm);
+	const knot_dname_t *nsec_owner = is_deleg(pkt) ? qdata->extra->encloser->owner : qdata->name;
+	knot_rrset_t *nsec = knot_rrset_new(nsec_owner, KNOT_RRTYPE_NSEC,
+	                                    KNOT_CLASS_IN, nsec_ttl(qdata), mm);
 	if (!nsec) {
 		return NULL;
 	}
 
-	knot_dname_t *next = online_nsec_next(qdata->name, knotd_qdata_zone_name(qdata));
+	knot_dname_t *next = online_nsec_next(nsec_owner, knotd_qdata_zone_name(qdata));
 	if (!next) {
-		knot_rrset_free(&nsec, mm);
+		knot_rrset_free(nsec, mm);
 		return NULL;
 	}
 
-	dnssec_nsec_bitmap_t *bitmap = synth_bitmap(pkt, qdata);
+	dnssec_nsec_bitmap_t *bitmap = synth_bitmap(pkt, qdata, !is_deleg(pkt));
 	if (!bitmap) {
 		free(next);
-		knot_rrset_free(&nsec, mm);
+		knot_rrset_free(nsec, mm);
 		return NULL;
 	}
 
@@ -220,13 +211,11 @@ static knot_rrset_t *synth_nsec(knot_pkt_t *pkt, knotd_qdata_t *qdata, knot_mm_t
 	int written = knot_dname_to_wire(rdata, next, size);
 	dnssec_nsec_bitmap_write(bitmap, rdata + written);
 
-	knot_dname_free(&next, NULL);
+	knot_dname_free(next, NULL);
 	dnssec_nsec_bitmap_free(bitmap);
 
-	uint32_t ttl = nsec_ttl(qdata);
-
-	if (knot_rrset_add_rdata(nsec, rdata, size, ttl, mm) != KNOT_EOK) {
-		knot_rrset_free(&nsec, mm);
+	if (knot_rrset_add_rdata(nsec, rdata, size, mm) != KNOT_EOK) {
+		knot_rrset_free(nsec, mm);
 		return NULL;
 	}
 
@@ -235,96 +224,109 @@ static knot_rrset_t *synth_nsec(knot_pkt_t *pkt, knotd_qdata_t *qdata, knot_mm_t
 
 static knot_rrset_t *sign_rrset(const knot_dname_t *owner,
                                 const knot_rrset_t *cover,
-                                online_sign_ctx_t *module_ctx,
-                                dnssec_sign_ctx_t *sign_ctx,
+                                knotd_mod_t *mod,
                                 knot_mm_t *mm)
 {
 	// copy of RR set with replaced owner name
 
-	knot_rrset_t *copy = knot_rrset_new(owner, cover->type, cover->rclass, NULL);
+	knot_rrset_t *copy = knot_rrset_new(owner, cover->type, cover->rclass,
+	                                    cover->ttl, NULL);
 	if (!copy) {
 		return NULL;
 	}
 
 	if (knot_rdataset_copy(&copy->rrs, &cover->rrs, NULL) != KNOT_EOK) {
-		knot_rrset_free(&copy, NULL);
+		knot_rrset_free(copy, NULL);
 		return NULL;
 	}
 
 	// resulting RRSIG
 
-	knot_rrset_t *rrsig = knot_rrset_new(owner, KNOT_RRTYPE_RRSIG, copy->rclass, mm);
+	knot_rrset_t *rrsig = knot_rrset_new(owner, KNOT_RRTYPE_RRSIG, copy->rclass,
+	                                     copy->ttl, mm);
 	if (!rrsig) {
-		knot_rrset_free(&copy, NULL);
+		knot_rrset_free(copy, NULL);
 		return NULL;
 	}
 
-	// compatible context
-
-	knot_kasp_policy_t policy = {
-		.rrsig_lifetime = module_ctx->rrsig_lifetime
-	};
-
-	kdnssec_ctx_t ksign_ctx = {
-		.now = time(NULL),
-		.policy = &policy
-	};
-
-	int r = knot_sign_rrset(rrsig, copy, module_ctx->key, sign_ctx, &ksign_ctx, mm);
-
-	knot_rrset_free(&copy, NULL);
-
-	if (r != KNOT_EOK) {
-		knot_rrset_free(&rrsig, mm);
+	int ret = knotd_mod_dnssec_sign_rrset(mod, rrsig, copy, mm);
+	if (ret != KNOT_EOK) {
+		knot_rrset_free(copy, NULL);
+		knot_rrset_free(rrsig, mm);
 		return NULL;
 	}
 
+	knot_rrset_free(copy, NULL);
+
 	return rrsig;
 }
 
+static glue_t *find_glue_for(const knot_rrset_t *rr, const knot_pkt_t *pkt)
+{
+	for (int i = KNOT_ANSWER; i <= KNOT_AUTHORITY; i++) {
+		const knot_pktsection_t *section = knot_pkt_section(pkt, i);
+		for (int j = 0; j < section->count; j++) {
+			const knot_rrset_t *attempt = knot_pkt_rr(section, j);
+			const additional_t *a = attempt->additional;
+			for (int k = 0; a != NULL && k < a->count; k++) {
+				// no need for knot_dname_cmp because the pointers are assigned
+				if (a->glues[k].node->owner == rr->owner) {
+					return &a->glues[k];
+				}
+			}
+		}
+	}
+	return NULL;
+}
+
+static bool shall_sign_rr(const knot_rrset_t *rr, const knot_pkt_t *pkt)
+{
+	if (pkt->current == KNOT_ADDITIONAL) {
+		glue_t *g = find_glue_for(rr, pkt);
+		assert(g); // finds actually the node which is rr in
+		return !(g->node->flags & NODE_FLAGS_NONAUTH);
+	} else {
+		return !is_deleg(pkt) || rr->type == KNOT_RRTYPE_NSEC;
+	}
+}
+
 static knotd_in_state_t sign_section(knotd_in_state_t state, knot_pkt_t *pkt,
                                      knotd_qdata_t *qdata, knotd_mod_t *mod)
 {
-	online_sign_ctx_t *module_ctx = knotd_mod_ctx(mod);
-
 	if (!want_dnssec(qdata)) {
 		return state;
 	}
 
-	dnssec_sign_ctx_t *sign_ctx = NULL;
-	int r = dnssec_sign_new(&sign_ctx, module_ctx->key);
-	if (r != DNSSEC_EOK) {
-		return KNOTD_IN_STATE_ERROR;
-	}
-
 	const knot_pktsection_t *section = knot_pkt_section(pkt, pkt->current);
 	assert(section);
 
 	uint16_t count_unsigned = section->count;
 	for (int i = 0; i < count_unsigned; i++) {
 		const knot_rrset_t *rr = knot_pkt_rr(section, i);
+		if (!shall_sign_rr(rr, pkt)) {
+			continue;
+		}
+
 		uint16_t rr_pos = knot_pkt_rr_offset(section, i);
 
 		uint8_t owner[KNOT_DNAME_MAXLEN] = { 0 };
 		knot_dname_unpack(owner, pkt->wire + rr_pos, sizeof(owner), pkt->wire);
 		knot_dname_to_lower(owner);
 
-		knot_rrset_t *rrsig = sign_rrset(owner, rr, module_ctx, sign_ctx, &pkt->mm);
+		knot_rrset_t *rrsig = sign_rrset(owner, rr, mod, &pkt->mm);
 		if (!rrsig) {
 			state = KNOTD_IN_STATE_ERROR;
 			break;
 		}
 
-		r = knot_pkt_put(pkt, KNOT_COMPR_HINT_NONE, rrsig, KNOT_PF_FREE);
+		int r = knot_pkt_put(pkt, KNOT_COMPR_HINT_NONE, rrsig, KNOT_PF_FREE);
 		if (r != KNOT_EOK) {
-			knot_rrset_free(&rrsig, &pkt->mm);
+			knot_rrset_free(rrsig, &pkt->mm);
 			state = KNOTD_IN_STATE_ERROR;
 			break;
 		}
 	}
 
-	dnssec_sign_free(sign_ctx);
-
 	return state;
 }
 
@@ -341,7 +343,7 @@ static knotd_in_state_t synth_authority(knotd_in_state_t state, knot_pkt_t *pkt,
 		knot_rrset_t *nsec = synth_nsec(pkt, qdata, &pkt->mm);
 		int r = knot_pkt_put(pkt, KNOT_COMPR_HINT_NONE, nsec, KNOT_PF_FREE);
 		if (r != DNSSEC_EOK) {
-			knot_rrset_free(&nsec, &pkt->mm);
+			knot_rrset_free(nsec, &pkt->mm);
 			return KNOTD_IN_STATE_ERROR;
 		}
 	}
@@ -357,30 +359,108 @@ static knotd_in_state_t synth_authority(knotd_in_state_t state, knot_pkt_t *pkt,
 	return state;
 }
 
-static knot_rrset_t *synth_dnskey(knotd_qdata_t *qdata, const dnssec_key_t *key,
+static knot_rrset_t *synth_dnskey(knotd_qdata_t *qdata, const zone_keyset_t *keyset,
                                   knot_mm_t *mm)
 {
 	knot_rrset_t *dnskey = knot_rrset_new(knotd_qdata_zone_name(qdata),
-	                                      KNOT_RRTYPE_DNSKEY, KNOT_CLASS_IN, mm);
+	                                      KNOT_RRTYPE_DNSKEY, KNOT_CLASS_IN,
+	                                      dnskey_ttl(qdata), mm);
 	if (!dnskey) {
 		return 0;
 	}
 
 	dnssec_binary_t rdata = { 0 };
-	dnssec_key_get_rdata(key, &rdata);
+	for (size_t i = 0; i < keyset->count; i++) {
+		if (!keyset->keys[i].is_public) {
+			continue;
+		}
+
+		dnssec_key_get_rdata(keyset->keys[i].key, &rdata);
+		assert(rdata.size > 0 && rdata.data);
+
+		int r = knot_rrset_add_rdata(dnskey, rdata.data, rdata.size, mm);
+		if (r != KNOT_EOK) {
+			knot_rrset_free(dnskey, mm);
+			return NULL;
+		}
+	}
+
+	return dnskey;
+}
+
+/* copied from the middle of zone-sign.c */
+static zone_key_t *ksk_for_cds(knotd_mod_t *mod)
+{
+	zone_key_t *res = NULL;
+	unsigned crp = mod->dnssec->policy->child_records_publish;
+	int kfc_prio = (crp == CHILD_RECORDS_ALWAYS ?
+	                0 : (crp == CHILD_RECORDS_ROLLOVER ? 1 : 2));
+	for (int i = 0; i < mod->keyset->count; i++) {
+		zone_key_t *key = &mod->keyset->keys[i];
+		if (key->is_ksk && key->cds_priority > kfc_prio) {
+			res = key;
+			kfc_prio = key->cds_priority;
+		}
+	}
+	return res;
+}
+
+static knot_rrset_t *synth_cdnskey(knotd_qdata_t *qdata, knotd_mod_t *mod,
+                                   knot_mm_t *mm)
+{
+	knot_rrset_t *dnskey = knot_rrset_new(knotd_qdata_zone_name(qdata),
+	                                      KNOT_RRTYPE_CDNSKEY, KNOT_CLASS_IN,
+	                                      0, mm);
+	if (dnskey == NULL) {
+		return 0;
+	}
+
+	dnssec_binary_t rdata = { 0 };
+	zone_key_t *key = ksk_for_cds(mod);
+	if (key == NULL) {
+		knot_rrset_free(dnskey, mm);
+		return NULL;
+	}
+	dnssec_key_get_rdata(key->key, &rdata);
 	assert(rdata.size > 0 && rdata.data);
 
-	uint32_t ttl = dnskey_ttl(qdata);
-
-	int r = knot_rrset_add_rdata(dnskey, rdata.data, rdata.size, ttl, mm);
-	if (r != KNOT_EOK) {
-		knot_rrset_free(&dnskey, mm);
+	int ret = knot_rrset_add_rdata(dnskey, rdata.data, rdata.size, mm);
+	if (ret != KNOT_EOK) {
+		knot_rrset_free(dnskey, mm);
 		return NULL;
 	}
 
 	return dnskey;
 }
 
+static knot_rrset_t *synth_cds(knotd_qdata_t *qdata, knotd_mod_t *mod,
+                               knot_mm_t *mm)
+{
+	knot_rrset_t *ds = knot_rrset_new(knotd_qdata_zone_name(qdata),
+	                                  KNOT_RRTYPE_CDS, KNOT_CLASS_IN,
+	                                  0, mm);
+	if (ds == NULL) {
+		return 0;
+	}
+
+	dnssec_binary_t rdata = { 0 };
+	zone_key_t *key = ksk_for_cds(mod);
+	if (key == NULL) {
+		knot_rrset_free(ds, mm);
+		return NULL;
+	}
+	zone_key_calculate_ds(key, &rdata);
+	assert(rdata.size > 0 && rdata.data);
+
+	int ret = knot_rrset_add_rdata(ds, rdata.data, rdata.size, mm);
+	if (ret != KNOT_EOK) {
+		knot_rrset_free(ds, mm);
+		return NULL;
+	}
+
+	return ds;
+}
+
 static bool qtype_match(knotd_qdata_t *qdata, uint16_t type)
 {
 	uint16_t qtype = knot_pkt_qtype(qdata->query);
@@ -392,11 +472,56 @@ static bool is_apex_query(knotd_qdata_t *qdata)
 	return knot_dname_is_equal(qdata->name, knotd_qdata_zone_name(qdata));
 }
 
+static knotd_in_state_t pre_routine(knotd_in_state_t state, knot_pkt_t *pkt,
+                                    knotd_qdata_t *qdata, knotd_mod_t *mod)
+{
+	online_sign_ctx_t *ctx = knotd_mod_ctx(mod);
+	zone_sign_reschedule_t resch = { .allow_rollover = true };
+
+	(void)pkt, (void)qdata;
+
+	pthread_mutex_lock(&ctx->event_mutex);
+	if (ctx->zone_doomed) {
+		pthread_mutex_unlock(&ctx->event_mutex);
+		return KNOTD_IN_STATE_ERROR;
+	}
+	mod->dnssec->now = time(NULL);
+	int ret = KNOT_ESEMCHECK;
+	if (knot_time_cmp(ctx->event_parent_ds_q, mod->dnssec->now) <= 0) {
+		ret = knot_parent_ds_query(mod->dnssec, mod->keyset, 1000);
+		if (ret != KNOT_EOK && mod->dnssec->policy->ksk_sbm_check_interval > 0) {
+			ctx->event_parent_ds_q = mod->dnssec->now + mod->dnssec->policy->ksk_sbm_check_interval;
+		} else {
+			ctx->event_parent_ds_q = 0;
+		}
+	}
+	if (ret == KNOT_EOK || knot_time_cmp(ctx->event_rollover, mod->dnssec->now) <= 0) {
+		ret = knot_dnssec_key_rollover(mod->dnssec, &resch);
+	}
+	if (ret == KNOT_EOK) {
+		if (resch.plan_ds_query && mod->dnssec->policy->ksk_sbm_check_interval > 0) {
+			ctx->event_parent_ds_q = mod->dnssec->now + mod->dnssec->policy->ksk_sbm_check_interval;
+		} else {
+			ctx->event_parent_ds_q = 0;
+		}
+		ctx->event_rollover = resch.next_rollover;
+
+		free_zone_keys(mod->keyset);
+		free(mod->keyset);
+		ret = knotd_mod_dnssec_load_keyset(mod, true);
+		if (ret != KNOT_EOK) {
+			ctx->zone_doomed = true;
+			state = KNOTD_IN_STATE_ERROR;
+		}
+	}
+	pthread_mutex_unlock(&ctx->event_mutex);
+
+	return state;
+}
+
 static knotd_in_state_t synth_answer(knotd_in_state_t state, knot_pkt_t *pkt,
                                      knotd_qdata_t *qdata, knotd_mod_t *mod)
 {
-	online_sign_ctx_t *ctx = knotd_mod_ctx(mod);
-
 	// disallowed queries
 
 	if (knot_pkt_qtype(pkt) == KNOT_RRTYPE_RRSIG) {
@@ -407,17 +532,44 @@ static knotd_in_state_t synth_answer(knotd_in_state_t state, knot_pkt_t *pkt,
 	// synthesized DNSSEC answers
 
 	if (qtype_match(qdata, KNOT_RRTYPE_DNSKEY) && is_apex_query(qdata)) {
-		knot_rrset_t *dnskey = synth_dnskey(qdata, ctx->key, &pkt->mm);
+		knot_rrset_t *dnskey = synth_dnskey(qdata, mod->keyset, &pkt->mm);
 		if (!dnskey) {
 			return KNOTD_IN_STATE_ERROR;
 		}
 
 		int r = knot_pkt_put(pkt, KNOT_COMPR_HINT_QNAME, dnskey, KNOT_PF_FREE);
 		if (r != DNSSEC_EOK) {
-			knot_rrset_free(&dnskey, &pkt->mm);
+			knot_rrset_free(dnskey, &pkt->mm);
+			return KNOTD_IN_STATE_ERROR;
+		}
+		state = KNOTD_IN_STATE_HIT;
+	}
+
+	if (qtype_match(qdata, KNOT_RRTYPE_CDNSKEY) && is_apex_query(qdata)) {
+		knot_rrset_t *dnskey = synth_cdnskey(qdata, mod, &pkt->mm);
+		if (!dnskey) {
 			return KNOTD_IN_STATE_ERROR;
 		}
 
+		int r = knot_pkt_put(pkt, KNOT_COMPR_HINT_QNAME, dnskey, KNOT_PF_FREE);
+		if (r != DNSSEC_EOK) {
+			knot_rrset_free(dnskey, &pkt->mm);
+			return KNOTD_IN_STATE_ERROR;
+		}
+		state = KNOTD_IN_STATE_HIT;
+	}
+
+	if (qtype_match(qdata, KNOT_RRTYPE_CDS) && is_apex_query(qdata)) {
+		knot_rrset_t *ds = synth_cds(qdata, mod, &pkt->mm);
+		if (!ds) {
+			return KNOTD_IN_STATE_ERROR;
+		}
+
+		int r = knot_pkt_put(pkt, KNOT_COMPR_HINT_QNAME, ds, KNOT_PF_FREE);
+		if (r != DNSSEC_EOK) {
+			knot_rrset_free(ds, &pkt->mm);
+			return KNOTD_IN_STATE_ERROR;
+		}
 		state = KNOTD_IN_STATE_HIT;
 	}
 
@@ -429,7 +581,7 @@ static knotd_in_state_t synth_answer(knotd_in_state_t state, knot_pkt_t *pkt,
 
 		int r = knot_pkt_put(pkt, KNOT_COMPR_HINT_QNAME, nsec, KNOT_PF_FREE);
 		if (r != DNSSEC_EOK) {
-			knot_rrset_free(&nsec, &pkt->mm);
+			knot_rrset_free(nsec, &pkt->mm);
 			return KNOTD_IN_STATE_ERROR;
 		}
 
@@ -439,52 +591,9 @@ static knotd_in_state_t synth_answer(knotd_in_state_t state, knot_pkt_t *pkt,
 	return state;
 }
 
-static int get_online_key(dnssec_key_t **key_ptr, knotd_mod_t *mod)
-{
-	kdnssec_ctx_t kctx = { 0 };
-
-	int r = kdnssec_ctx_init(mod->config, &kctx, mod->zone, mod->id);
-	if (r != DNSSEC_EOK) {
-		return r;
-	}
-
-	// Force Singe-Type signing scheme.
-	kctx.policy->singe_type_signing = true;
-
-	zone_sign_reschedule_t ignore_out = { 0 };
-	ignore_out.allow_rollover = true;
-	r = knot_dnssec_key_rollover(&kctx, &ignore_out);
-	if (r != DNSSEC_EOK) {
-		goto fail;
-	}
-
-	assert(kctx.zone->num_keys > 0);
-	knot_kasp_key_t *kasp_key = &kctx.zone->keys[0];
-	assert(kasp_key);
-
-	dnssec_key_t *key = dnssec_key_dup(kasp_key->key);
-	if (!key) {
-		r = KNOT_ENOMEM;
-		goto fail;
-	}
-
-	r = dnssec_key_import_keystore(key, kctx.keystore, kasp_key->id);
-	if (r != DNSSEC_EOK) {
-		dnssec_key_free(key);
-		goto fail;
-	}
-
-	*key_ptr = key;
-
-fail:
-	kdnssec_ctx_deinit(&kctx);
-
-	return r;
-}
-
 static void online_sign_ctx_free(online_sign_ctx_t *ctx)
 {
-	dnssec_key_free(ctx->key);
+	pthread_mutex_destroy(&ctx->event_mutex);
 
 	free(ctx);
 }
@@ -496,21 +605,37 @@ static int online_sign_ctx_new(online_sign_ctx_t **ctx_ptr, knotd_mod_t *mod)
 		return KNOT_ENOMEM;
 	}
 
-	int r = get_online_key(&ctx->key, mod);
-	if (r != KNOT_EOK) {
-		online_sign_ctx_free(ctx);
-		return r;
+	int ret = knotd_mod_dnssec_init(mod);
+	if (ret != KNOT_EOK) {
+		free(ctx);
+		return ret;
 	}
 
-	ctx->rrsig_lifetime = RRSIG_LIFETIME;
-	knotd_conf_t policy = knotd_conf_mod(mod, MOD_POLICY);
-	if (policy.count != 0) {
-		knotd_conf_t conf = knotd_conf(mod, C_POLICY, C_RRSIG_LIFETIME, &policy);
-		if (conf.count != 0) {
-			ctx->rrsig_lifetime = conf.single.integer;
-		}
+	// Force Singe-Type signing scheme. This is only important for compatibility with older versions.
+	mod->dnssec->policy->singe_type_signing = true;
+
+	zone_sign_reschedule_t resch = {
+		.allow_rollover = true
+	};
+	ret = knot_dnssec_key_rollover(mod->dnssec, &resch);
+	if (ret != KNOT_EOK) {
+		free(ctx);
+		return ret;
 	}
 
+	if (resch.plan_ds_query) {
+		ctx->event_parent_ds_q = time(NULL);
+	}
+	ctx->event_rollover = resch.next_rollover;
+
+	ret = knotd_mod_dnssec_load_keyset(mod, true);
+	if (ret != KNOT_EOK) {
+		free(ctx);
+		return ret;
+	}
+
+	pthread_mutex_init(&ctx->event_mutex, NULL);
+
 	*ctx_ptr = ctx;
 
 	return KNOT_EOK;
@@ -535,6 +660,8 @@ int online_sign_load(knotd_mod_t *mod)
 
 	knotd_mod_ctx_set(mod, ctx);
 
+	knotd_mod_in_hook(mod, KNOTD_STAGE_ANSWER, pre_routine);
+
 	knotd_mod_in_hook(mod, KNOTD_STAGE_ANSWER, synth_answer);
 	knotd_mod_in_hook(mod, KNOTD_STAGE_ANSWER, sign_section);
 
