@@ -1,4 +1,4 @@
-/*  Copyright (C) 2017 CZ.NIC, z.s.p.o. <knot-dns@labs.nic.cz>
+/*  Copyright (C) 2018 CZ.NIC, z.s.p.o. <knot-dns@labs.nic.cz>
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -59,7 +59,7 @@ static const knot_dname_t *name_from_rrsig(const knot_rrset_t *rr)
 	}
 
 	// This is a signature.
-	return knot_rrsig_signer_name(&rr->rrs, 0);
+	return knot_rrsig_signer_name(rr->rrs.rdata);
 }
 
 static const knot_dname_t *name_from_authrr(const knot_rrset_t *rr)
@@ -111,6 +111,11 @@ static knotd_state_t ratelimit_apply(knotd_state_t state, knot_pkt_t *pkt,
 		return state;
 	}
 
+	// Rate limit is not applied to responses with a valid cookie.
+	if (qdata->params->flags & KNOTD_QUERY_FLAG_COOKIE) {
+		return state;
+	}
+
 	// Exempt clients.
 	if (addr_range_match(&ctx->whitelist, qdata->params->remote)) {
 		return state;
@@ -184,37 +189,23 @@ int rrl_load(knotd_mod_t *mod)
 	}
 
 	// Create table.
-	knotd_conf_t conf = knotd_conf_mod(mod, MOD_TBL_SIZE);
-	ctx->rrl = rrl_create(conf.single.integer);
+	knotd_conf_t rate = knotd_conf_mod(mod, MOD_RATE_LIMIT);
+	knotd_conf_t size = knotd_conf_mod(mod, MOD_TBL_SIZE);
+	ctx->rrl = rrl_create(size.single.integer, rate.single.integer);
 	if (ctx->rrl == NULL) {
 		ctx_free(ctx);
 		return KNOT_ENOMEM;
 	}
 
-	// Set locks.
-	int ret = rrl_setlocks(ctx->rrl, RRL_LOCK_GRANULARITY);
-	if (ret != KNOT_EOK) {
-		ctx_free(ctx);
-		return ret;
-	}
-
-	// Set rate limit.
-	conf = knotd_conf_mod(mod, MOD_RATE_LIMIT);
-	ret = rrl_setrate(ctx->rrl, conf.single.integer);
-	if (ret != KNOT_EOK) {
-		ctx_free(ctx);
-		return ret;
-	}
-
 	// Get slip.
-	conf = knotd_conf_mod(mod, MOD_SLIP);
+	knotd_conf_t conf = knotd_conf_mod(mod, MOD_SLIP);
 	ctx->slip = conf.single.integer;
 
 	// Get whitelist.
 	ctx->whitelist = knotd_conf_mod(mod, MOD_WHITELIST);
 
 	// Set up statistics counters.
-	ret = knotd_mod_stats_add(mod, "slipped", 1, NULL);
+	int ret = knotd_mod_stats_add(mod, "slipped", 1, NULL);
 	if (ret != KNOT_EOK) {
 		ctx_free(ctx);
 		return ret;
