@@ -1,4 +1,4 @@
-/*  Copyright (C) 2017 CZ.NIC, z.s.p.o. <knot-dns@labs.nic.cz>
+/*  Copyright (C) 2018 CZ.NIC, z.s.p.o. <knot-dns@labs.nic.cz>
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -16,7 +16,7 @@
 
 #include <urcu.h>
 
-#include "dnssec/tsig.h"
+#include "libdnssec/tsig.h"
 #include "knot/common/log.h"
 #include "knot/dnssec/rrset-sign.h"
 #include "knot/nameserver/process_query.h"
@@ -266,7 +266,7 @@ static int answer_edns_init(const knot_pkt_t *query, knot_pkt_t *resp,
 	}
 
 	/* Append NSID if requested and available. */
-	if (knot_edns_has_option(query->opt_rr, KNOT_EDNS_OPTION_NSID)) {
+	if (knot_pkt_edns_option(query, KNOT_EDNS_OPTION_NSID) != NULL) {
 		conf_val_t *nsid = &conf()->cache.srv_nsid;
 		size_t nsid_len;
 		const uint8_t *nsid_data = conf_bin(nsid, &nsid_len);
@@ -291,6 +291,34 @@ static int answer_edns_init(const knot_pkt_t *query, knot_pkt_t *resp,
 		}
 	}
 
+	/* Initialize EDNS Client Subnet if configured and present in query. */
+	if (conf()->cache.use_ecs) {
+		uint8_t *ecs_opt = knot_pkt_edns_option(query, KNOT_EDNS_OPTION_CLIENT_SUBNET);
+		if (ecs_opt != NULL) {
+			qdata->ecs = mm_alloc(qdata->mm, sizeof(knot_edns_client_subnet_t));
+			if (qdata->ecs == NULL) {
+				return KNOT_ENOMEM;
+			}
+			const uint8_t *ecs_data = knot_edns_opt_get_data(ecs_opt);
+			uint16_t ecs_len = knot_edns_opt_get_length(ecs_opt);
+			ret = knot_edns_client_subnet_parse(qdata->ecs, ecs_data, ecs_len);
+			if (ret != KNOT_EOK) {
+				qdata->rcode = KNOT_RCODE_FORMERR;
+				return ret;
+			}
+			qdata->ecs->scope_len = 0;
+
+			/* Reserve space for the option in the answer. */
+			ret = knot_edns_reserve_option(&qdata->opt_rr, KNOT_EDNS_OPTION_CLIENT_SUBNET,
+			                               ecs_len, NULL, qdata->mm);
+			if (ret != KNOT_EOK) {
+				return ret;
+			}
+		}
+	} else {
+		qdata->ecs = NULL;
+	}
+
 	return answer_edns_reserve(resp, qdata);
 }
 
@@ -300,8 +328,22 @@ static int answer_edns_put(knot_pkt_t *resp, knotd_qdata_t *qdata)
 		return KNOT_EOK;
 	}
 
+	/* Add ECS if present. */
+	int ret = KNOT_EOK;
+	if (qdata->ecs != NULL) {
+		uint8_t *ecs_opt = knot_edns_get_option(&qdata->opt_rr, KNOT_EDNS_OPTION_CLIENT_SUBNET);
+		if (ecs_opt != NULL) {
+			uint8_t *ecs_data = knot_edns_opt_get_data(ecs_opt);
+			uint16_t ecs_len = knot_edns_opt_get_length(ecs_opt);
+			ret = knot_edns_client_subnet_write(ecs_data, ecs_len, qdata->ecs);
+			if (ret != KNOT_EOK) {
+				return ret;
+			}
+		}
+	}
+
 	/* Reclaim reserved size. */
-	int ret = knot_pkt_reclaim(resp, knot_edns_wire_size(&qdata->opt_rr));
+	ret = knot_pkt_reclaim(resp, knot_edns_wire_size(&qdata->opt_rr));
 	if (ret != KNOT_EOK) {
 		return ret;
 	}
@@ -320,7 +362,7 @@ static int answer_edns_put(knot_pkt_t *resp, knotd_qdata_t *qdata)
 }
 
 /*! \brief Initialize response, sizes and find zone from which we're going to answer. */
-static int prepare_answer(const knot_pkt_t *query, knot_pkt_t *resp, knot_layer_t *ctx)
+static int prepare_answer(knot_pkt_t *query, knot_pkt_t *resp, knot_layer_t *ctx)
 {
 	knotd_qdata_t *qdata = QUERY_DATA(ctx);
 	server_t *server = qdata->params->server;
@@ -332,24 +374,6 @@ static int prepare_answer(const knot_pkt_t *query, knot_pkt_t *resp, knot_layer_
 	}
 	knot_wire_clear_cd(resp->wire);
 
-	/* Query MUST carry a question. */
-	const knot_dname_t *qname = knot_pkt_qname(query);
-	if (qname == NULL) {
-		qdata->rcode = KNOT_RCODE_FORMERR;
-		return KNOT_EMALF;
-	}
-
-	/* Convert query QNAME to lowercase, but keep original QNAME case.
-	 * Already checked for absence of compression and length.
-	 */
-	memcpy(qdata->extra->orig_qname, qname, query->qname_size);
-	ret = process_query_qname_case_lower((knot_pkt_t *)query);
-	if (ret != KNOT_EOK) {
-		return ret;
-	}
-	/* Find zone for QNAME. */
-	qdata->extra->zone = answer_zone_find(query, server->zone_db);
-
 	/* Setup EDNS. */
 	ret = answer_edns_init(query, resp, qdata);
 	if (ret != KNOT_EOK || qdata->rcode != 0) {
@@ -380,7 +404,23 @@ static int prepare_answer(const knot_pkt_t *query, knot_pkt_t *resp, knot_layer_
 		resp->max_size = KNOT_WIRE_MAX_PKTSIZE;
 	}
 
-	return ret;
+	/* Query MUST carry a question. */
+	const knot_dname_t *qname = knot_pkt_qname(query);
+	if (qname == NULL) {
+		qdata->rcode = KNOT_RCODE_FORMERR;
+		return KNOT_EMALF;
+	}
+
+	/* Convert query QNAME to lowercase, but keep original QNAME case.
+	 * Already checked for absence of compression and length.
+	 */
+	memcpy(qdata->extra->orig_qname, qname, query->qname_size);
+	process_query_qname_case_lower(query);
+
+	/* Find zone for QNAME. */
+	qdata->extra->zone = answer_zone_find(query, server->zone_db);
+
+	return KNOT_EOK;
 }
 
 static void set_rcode_to_packet(knot_pkt_t *pkt, knotd_qdata_t *qdata)
@@ -549,6 +589,10 @@ finish:
 		/* Error processing. */
 		next_state = process_query_err(ctx, pkt);
 		break;
+	case KNOT_STATE_FINAL:
+		/* Just skipped postprocessing. */
+		next_state = KNOT_STATE_DONE;
+		break;
 	default:
 		set_rcode_to_packet(pkt, qdata);
 	}
@@ -735,22 +779,6 @@ fail:
 	return ret;
 }
 
-void process_query_qname_case_restore(knot_pkt_t *pkt, knotd_qdata_t *qdata)
-{
-	/* If original QNAME is empty, Query is either unparsed or for root domain.
-	 * Either way, letter case doesn't matter. */
-	if (qdata->extra->orig_qname[0] != '\0') {
-		memcpy(pkt->wire + KNOT_WIRE_HEADER_SIZE,
-		       qdata->extra->orig_qname, qdata->query->qname_size);
-	}
-}
-
-int process_query_qname_case_lower(knot_pkt_t *pkt)
-{
-	knot_dname_t *qname = (knot_dname_t *)knot_pkt_qname(pkt);
-	return knot_dname_to_lower(qname);
-}
-
 /*! \brief Synthesize RRSIG for given parameters, store in 'qdata' for later use */
 static int put_rrsig(const knot_dname_t *sig_owner, uint16_t type,
                      const knot_rrset_t *rrsigs, knot_rrinfo_t *rrinfo,
@@ -781,7 +809,8 @@ static int put_rrsig(const knot_dname_t *sig_owner, uint16_t type,
 		knot_rdataset_clear(&synth_rrs, qdata->mm);
 		return KNOT_ENOMEM;
 	}
-	knot_rrset_init(&info->synth_rrsig, owner_copy, rrsigs->type, rrsigs->rclass);
+	knot_rrset_init(&info->synth_rrsig, owner_copy, rrsigs->type,
+	                rrsigs->rclass, rrsigs->ttl);
 	/* Store filtered signature. */
 	info->synth_rrsig.rrs = synth_rrs;
 
@@ -795,7 +824,7 @@ int process_query_put_rr(knot_pkt_t *pkt, knotd_qdata_t *qdata,
                          const knot_rrset_t *rr, const knot_rrset_t *rrsigs,
                          uint16_t compr_hint, uint32_t flags)
 {
-	if (rr->rrs.rr_count < 1) {
+	if (rr->rrs.count < 1) {
 		return KNOT_EMALF;
 	}
 
@@ -817,10 +846,10 @@ int process_query_put_rr(knot_pkt_t *pkt, knotd_qdata_t *qdata,
 		if (qname_cpy == NULL) {
 			return KNOT_ENOMEM;
 		}
-		knot_rrset_init(&to_add, qname_cpy, rr->type, rr->rclass);
+		knot_rrset_init(&to_add, qname_cpy, rr->type, rr->rclass, rr->ttl);
 		ret = knot_rdataset_copy(&to_add.rrs, &rr->rrs, &pkt->mm);
 		if (ret != KNOT_EOK) {
-			knot_dname_free(&qname_cpy, &pkt->mm);
+			knot_dname_free(qname_cpy, &pkt->mm);
 			return ret;
 		}
 		to_add.additional = rr->additional;
