@@ -1,4 +1,4 @@
-/*  Copyright (C) 2017 CZ.NIC, z.s.p.o. <knot-dns@labs.nic.cz>
+/*  Copyright (C) 2018 CZ.NIC, z.s.p.o. <knot-dns@labs.nic.cz>
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -18,6 +18,8 @@
 
 #include "libknot/libknot.h"
 #include "knot/conf/conf.h"
+#include "knot/dnssec/context.h"
+#include "knot/dnssec/zone-keys.h"
 #include "knot/include/module.h"
 #include "contrib/ucw/lists.h"
 
@@ -44,12 +46,11 @@ struct query_step {
  *  assembly phase, for example 'before processing', 'answer section' and so on.
  */
 struct query_plan {
-	knot_mm_t *mm;
 	list_t stage[KNOTD_STAGES];
 };
 
 /*! \brief Create an empty query plan. */
-struct query_plan *query_plan_create(knot_mm_t *mm);
+struct query_plan *query_plan_create(void);
 
 /*! \brief Free query plan and all planned steps. */
 void query_plan_free(struct query_plan *plan);
@@ -60,8 +61,7 @@ int query_plan_step(struct query_plan *plan, knotd_stage_t stage,
 
 /*! \brief Open query module identified by name. */
 knotd_mod_t *query_module_open(conf_t *conf, conf_mod_id_t *mod_id,
-                               struct query_plan *plan, const knot_dname_t *zone,
-                               knot_mm_t *mm);
+                               struct query_plan *plan, const knot_dname_t *zone);
 
 /*! \brief Close query module. */
 void query_module_close(knotd_mod_t *module);
@@ -82,12 +82,13 @@ typedef struct {
 
 struct knotd_mod {
 	node_t node;
-	knot_mm_t *mm;
 	conf_t *config;
 	conf_mod_id_t *id;
 	struct query_plan *plan;
 	const knot_dname_t *zone;
 	const knotd_mod_api_t *api;
+	kdnssec_ctx_t *dnssec;
+	zone_keyset_t *keyset;
 	mod_ctr_t *stats;
 	uint32_t stats_count;
 	void *ctx;
