@@ -1,4 +1,4 @@
-/*  Copyright (C) 2016 CZ.NIC, z.s.p.o. <knot-dns@labs.nic.cz>
+/*  Copyright (C) 2018 CZ.NIC, z.s.p.o. <knot-dns@labs.nic.cz>
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -18,9 +18,8 @@
 
 #include <stdint.h>
 
-#include "contrib/wire.h"
 #include "contrib/wire_ctx.h"
-#include "dnssec/random.h"
+#include "libdnssec/random.h"
 #include "knot/conf/conf.h"
 #include "libknot/yparser/yptrafo.h"
 #include "libknot/rrset.h"
