@@ -28,6 +28,7 @@
 #include "knot/conf/confio.h"
 #include "knot/conf/migration.h"
 #include "knot/conf/module.h"
+#include "knot/dnssec/kasp/kasp_db.h"
 #include "knot/server/server.h"
 #include "knot/server/udp-handler.h"
 #include "knot/server/tcp-handler.h"
@@ -189,8 +190,8 @@ static int server_init_iface(iface_t *new_if, struct sockaddr_storage *addr, int
 
 	/* Initialize the sockets to ensure safe early deinitialization. */
 	for (int i = 0; i < udp_socket_count; i++) {
-		new_if->fd_udp[new_if->fd_udp_count] = -1;
-	};
+		new_if->fd_udp[i] = -1;
+	}
 	new_if->fd_tcp = -1;
 
 	bool warn_bind = false;
