@@ -26,9 +26,9 @@
 
 #include <urcu.h>
 
-static int init_incremental(zone_update_t *update, zone_t *zone)
+static int init_incremental(zone_update_t *update, zone_t *zone, zone_contents_t *old_contents, bool deep_copy)
 {
-	if (zone->contents == NULL) {
+	if (old_contents == NULL) {
 		return KNOT_EINVAL;
 	}
 
@@ -37,12 +37,16 @@ static int init_incremental(zone_update_t *update, zone_t *zone)
 		return ret;
 	}
 
-	update->new_cont_deep_copy = false;
-
-	ret = apply_prepare_zone_copy(zone->contents, &update->new_cont);
-	if (ret != KNOT_EOK) {
-		changeset_clear(&update->change);
-		return ret;
+	if (deep_copy) {
+		update->new_cont_deep_copy = true;
+		update->new_cont = old_contents;
+	} else {
+		update->new_cont_deep_copy = false;
+		ret = apply_prepare_zone_copy(old_contents, &update->new_cont);
+		if (ret != KNOT_EOK) {
+			changeset_clear(&update->change);
+			return ret;
+		}
 	}
 
 	uint32_t apply_flags = update->flags & UPDATE_STRICT ? APPLY_STRICT : 0;
@@ -50,9 +54,9 @@ static int init_incremental(zone_update_t *update, zone_t *zone)
 
 	/* Copy base SOA RR. */
 	update->change.soa_from =
-		node_create_rrset(zone->contents->apex, KNOT_RRTYPE_SOA);
+		node_create_rrset(old_contents->apex, KNOT_RRTYPE_SOA);
 	if (update->change.soa_from == NULL) {
-		zone_contents_free(&update->new_cont);
+		zone_contents_free(update->new_cont);
 		changeset_clear(&update->change);
 		return KNOT_ENOMEM;
 	}
@@ -88,14 +92,18 @@ static int replace_soa(zone_contents_t *contents, const knot_rrset_t *rr)
 		return ret;
 	}
 
-	return zone_contents_add_rr(contents, rr, &n);
+	ret = zone_contents_add_rr(contents, rr, &n);
+	if (ret == KNOT_ETTL) {
+		return KNOT_EOK;
+	}
+
+	return ret;
 }
 
-/* ------------------------------- API -------------------------------------- */
-
-int zone_update_init(zone_update_t *update, zone_t *zone, zone_update_flags_t flags)
+int init_base(zone_update_t *update, zone_t *zone, zone_contents_t *old_contents,
+              zone_update_flags_t flags)
 {
-	if (update == NULL || zone == NULL) {
+	if (update == NULL || zone == NULL || (old_contents == NULL && (flags & UPDATE_INCREMENTAL))) {
 		return KNOT_EINVAL;
 	}
 
@@ -112,7 +120,7 @@ int zone_update_init(zone_update_t *update, zone_t *zone, zone_update_flags_t fl
 
 	int ret = KNOT_EINVAL;
 	if (flags & UPDATE_INCREMENTAL) {
-		ret = init_incremental(update, zone);
+		ret = init_incremental(update, zone, old_contents, flags & UPDATE_JOURNAL);
 	} else if (flags & UPDATE_FULL) {
 		ret = init_full(update, zone);
 	}
@@ -123,43 +131,58 @@ int zone_update_init(zone_update_t *update, zone_t *zone, zone_update_flags_t fl
 	return ret;
 }
 
+/* ------------------------------- API -------------------------------------- */
+
+int zone_update_init(zone_update_t *update, zone_t *zone, zone_update_flags_t flags)
+{
+	return init_base(update, zone, zone->contents, flags);
+}
+
 int zone_update_from_differences(zone_update_t *update, zone_t *zone, zone_contents_t *old_cont,
-                                 zone_contents_t *new_cont, zone_update_flags_t flags)
+				 zone_contents_t *new_cont, zone_update_flags_t flags, bool ignore_dnssec)
 {
 	if (update == NULL || zone == NULL || new_cont == NULL ||
 	    !(flags & UPDATE_INCREMENTAL) || (flags & UPDATE_FULL)) {
 		return KNOT_EINVAL;
 	}
 
-	memset(update, 0, sizeof(*update));
-	update->zone = zone;
-
-	mm_ctx_mempool(&update->mm, MM_DEFAULT_BLKSIZE);
-	update->flags = flags;
-
-	update->new_cont = new_cont;
-	update->new_cont_deep_copy = true;
-
-	update->a_ctx = calloc(1, sizeof(*update->a_ctx));
-	if (update->a_ctx == NULL) {
-		return KNOT_ENOMEM;
-	}
-
-	int ret = changeset_init(&update->change, zone->name);
+	changeset_t diff;
+	int ret = changeset_init(&diff, zone->name);
 	if (ret != KNOT_EOK) {
-		free(update->a_ctx);
 		return ret;
 	}
 
-	ret = zone_contents_diff(old_cont, new_cont, &update->change);
-	if (ret != KNOT_EOK && ret != KNOT_ENODIFF) {
-		free(update->a_ctx);
-		changeset_clear(&update->change);
+	ret = zone_contents_diff(old_cont, new_cont, &diff, ignore_dnssec);
+	if (ret != KNOT_EOK && ret != KNOT_ENODIFF && ret != KNOT_ESEMCHECK) {
+		changeset_clear(&diff);
 		return ret;
 	}
 
-	uint32_t apply_flags = update->flags & UPDATE_STRICT ? APPLY_STRICT : 0;
-	apply_init_ctx(update->a_ctx, update->new_cont, apply_flags);
+	// True if nonempty changes were made but the serial
+	// remained the same and has to be incremented.
+	bool diff_semcheck = (ret == KNOT_ESEMCHECK);
+
+	ret = init_base(update, zone, old_cont, flags);
+	if (ret != KNOT_EOK) {
+		changeset_clear(&diff);
+		return ret;
+	}
+
+	ret = zone_update_apply_changeset(update, &diff);
+	changeset_clear(&diff);
+	if (ret != KNOT_EOK) {
+		zone_update_clear(update);
+		return ret;
+	}
+
+	if (diff_semcheck) {
+		ret = zone_update_increment_soa(update, conf());
+		if (ret != KNOT_EOK) {
+			zone_update_clear(update);
+			return ret;
+		}
+		log_zone_info(zone->name, "automatic SOA serial increment");
+	}
 
 	return KNOT_EOK;
 }
@@ -228,7 +251,7 @@ uint32_t zone_update_current_serial(zone_update_t *update)
 {
 	const zone_node_t *apex = zone_update_get_apex(update);
 	if (apex != NULL) {
-		return knot_soa_serial(node_rdataset(apex, KNOT_RRTYPE_SOA));
+		return knot_soa_serial(node_rdataset(apex, KNOT_RRTYPE_SOA)->rdata);
 	} else {
 		return 0;
 	}
@@ -277,15 +300,15 @@ void zone_update_clear(zone_update_t *update)
 		/* Revert any changes on error, do nothing on success. */
 		if (update->new_cont_deep_copy) {
 			update_cleanup(update->a_ctx);
-			zone_contents_deep_free(&update->new_cont);
+			zone_contents_deep_free(update->new_cont);
 		} else {
 			update_rollback(update->a_ctx);
-			update_free_zone(&update->new_cont);
+			update_free_zone(update->new_cont);
 		}
 		changeset_clear(&update->change);
 	} else if (update->flags & UPDATE_FULL) {
 		assert(update->new_cont_deep_copy);
-		zone_contents_deep_free(&update->new_cont);
+		zone_contents_deep_free(update->new_cont);
 	}
 	free(update->a_ctx);
 	mp_delete(update->mm.ctx);
@@ -327,7 +350,22 @@ int zone_update_add(zone_update_t *update, const knot_rrset_t *rrset)
 		}
 
 		zone_node_t *n = NULL;
-		return zone_contents_add_rr(update->new_cont, rrset, &n);
+		int ret = zone_contents_add_rr(update->new_cont, rrset, &n);
+		if (ret == KNOT_ETTL) {
+			char buff[KNOT_DNAME_TXT_MAXLEN + 1];
+			char *owner = knot_dname_to_str(buff, rrset->owner, sizeof(buff));
+			if (owner == NULL) {
+				owner = "";
+			}
+			char type[16] = { '\0' };
+			knot_rrtype_to_string(rrset->type, type, sizeof(type));
+			log_zone_notice(update->new_cont->apex->owner,
+			                "TTL mismatch, owner %s, type %s, "
+			                "TTL set to %u", owner, type, rrset->ttl);
+			return KNOT_EOK;
+		}
+
+		return ret;
 	} else {
 		return KNOT_EINVAL;
 	}
@@ -361,7 +399,7 @@ int zone_update_remove(zone_update_t *update, const knot_rrset_t *rrset)
 		zone_node_t *n = NULL;
 		knot_rrset_t *rrs_copy = knot_rrset_copy(rrset, &update->mm);
 		int ret = zone_contents_remove_rr(update->new_cont, rrs_copy, &n);
-		knot_rrset_free(&rrs_copy, &update->mm);
+		knot_rrset_free(rrs_copy, &update->mm);
 		return ret;
 	} else {
 		return KNOT_EINVAL;
@@ -493,6 +531,16 @@ int zone_update_apply_changeset_fix(zone_update_t *update, changeset_t *changes)
 	return ret;
 }
 
+int zone_update_apply_changeset_reverse(zone_update_t *update, const changeset_t *changes)
+{
+	changeset_t reverse;
+	reverse.remove = changes->add;
+	reverse.add = changes->remove;
+	reverse.soa_from = changes->soa_to;
+	reverse.soa_to = changes->soa_from;
+	return zone_update_apply_changeset(update, &reverse);
+}
+
 static int set_new_soa(zone_update_t *update, unsigned serial_policy)
 {
 	assert(update);
@@ -505,21 +553,23 @@ static int set_new_soa(zone_update_t *update, unsigned serial_policy)
 
 	int ret = zone_update_remove(update, soa_cpy);
 	if (ret != KNOT_EOK) {
-		knot_rrset_free(&soa_cpy, NULL);
+		knot_rrset_free(soa_cpy, NULL);
+		return ret;
 	}
 
-	uint32_t old_serial = knot_soa_serial(&soa_cpy->rrs);
+	uint32_t old_serial = knot_soa_serial(soa_cpy->rrs.rdata);
 	uint32_t new_serial = serial_next(old_serial, serial_policy);
 	if (serial_compare(old_serial, new_serial) != SERIAL_LOWER) {
-		log_zone_warning(update->zone->name, "updated serial is lower "
+		log_zone_warning(update->zone->name, "updated SOA serial is lower "
 		                 "than current, serial %u -> %u",
 		                 old_serial, new_serial);
+		ret = KNOT_ESOAINVAL;
+	} else {
+		knot_soa_serial_set(soa_cpy->rrs.rdata, new_serial);
+
+		ret = zone_update_add(update, soa_cpy);
 	}
-
-	knot_soa_serial_set(&soa_cpy->rrs, new_serial);
-
-	ret = zone_update_add(update, soa_cpy);
-	knot_rrset_free(&soa_cpy, NULL);
+	knot_rrset_free(soa_cpy, NULL);
 
 	return ret;
 }
@@ -619,18 +669,14 @@ typedef struct {
 	struct rcu_head rcuhead;
 	void (*callback)(void *);
 	void *ctx;
-	bool call_ctx_addr;
+	bool free_ctx;
 } callrcu_wrapper_t;
 
 static void callrcu_wrapper_cb(struct rcu_head *param)
 {
 	callrcu_wrapper_t *wrap = (callrcu_wrapper_t *)param;
-	if (wrap->call_ctx_addr) {
-		// Paradigm free_fun(type_t **t) freeing the pointer completely.
-		wrap->callback(&wrap->ctx);
-	} else {
-		// Paradigm clear_fun(type_t *t) just clearing the contents of type_t.
-		wrap->callback(wrap->ctx);
+	wrap->callback(wrap->ctx);
+	if (wrap->free_ctx) {
 		free(wrap->ctx);
 	}
 	free(wrap);
@@ -640,13 +686,13 @@ static void callrcu_wrapper_cb(struct rcu_head *param)
 }
 
 /* NOTE: Does nothing if not enough memory. */
-static void callrcu_wrapper(void *ctx, void (*callback)(void *), bool call_ctx_addr)
+static void callrcu_wrapper(void *ctx, void *callback, bool free_ctx)
 {
 	callrcu_wrapper_t *wrap = calloc(1, sizeof(callrcu_wrapper_t));
 	if (wrap != NULL) {
 		wrap->callback = callback;
 		wrap->ctx = ctx;
-		wrap->call_ctx_addr = call_ctx_addr;
+		wrap->free_ctx = free_ctx;
 		call_rcu((struct rcu_head *)wrap, callrcu_wrapper_cb);
 	}
 }
@@ -682,6 +728,14 @@ int zone_update_commit(conf_t *conf, zone_update_t *update)
 		return KNOT_EZONESIZE;
 	}
 
+	/* Check if the zone was re-signed upon zone load to ensure proper flush
+	 * even if the SOA serial wasn't incremented by re-signing. */
+	val = conf_zone_get(conf, C_DNSSEC_SIGNING, update->zone->name);
+	bool dnssec = conf_bool(&val);
+	if (!changeset_empty(&update->change) && dnssec) {
+		update->zone->zonefile.resigned = true;
+	}
+
 	/* Switch zone contents. */
 	zone_contents_t *old_contents;
 	old_contents = zone_switch_contents(update->zone, new_contents);
@@ -689,16 +743,16 @@ int zone_update_commit(conf_t *conf, zone_update_t *update)
 	/* Sync RCU. */
 	if (update->flags & UPDATE_FULL) {
 		assert(update->new_cont_deep_copy);
-		callrcu_wrapper(old_contents, (void (*)(void *))zone_contents_deep_free, true);
+		callrcu_wrapper(old_contents, zone_contents_deep_free, false);
 	} else if (update->flags & UPDATE_INCREMENTAL) {
 		if (update->new_cont_deep_copy) {
-			callrcu_wrapper(old_contents, (void (*)(void *))zone_contents_deep_free, true);
+			callrcu_wrapper(old_contents, zone_contents_deep_free, false);
 		} else {
-			callrcu_wrapper(old_contents, (void (*)(void *))update_free_zone, true);
+			callrcu_wrapper(old_contents, update_free_zone, false);
 		}
 		changeset_clear(&update->change);
 	}
-	callrcu_wrapper(update->a_ctx, (void (*)(void *))update_cleanup, false);
+	callrcu_wrapper(update->a_ctx, update_cleanup, true);
 	update->a_ctx = NULL;
 	update->new_cont = NULL;
 
