@@ -1,4 +1,4 @@
-/*  Copyright (C) 2016 CZ.NIC, z.s.p.o. <knot-dns@labs.nic.cz>
+/*  Copyright (C) 2018 CZ.NIC, z.s.p.o. <knot-dns@labs.nic.cz>
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -16,7 +16,7 @@
 
 #include <assert.h>
 
-#include "dnssec/error.h"
+#include "libdnssec/error.h"
 #include "knot/zone/contents.h"
 #include "knot/common/log.h"
 #include "knot/dnssec/zone-nsec.h"
@@ -65,7 +65,7 @@ static int check_node(const zone_contents_t *contents, const zone_node_t *node)
 	assert(contents->apex != NULL);
 	assert(node);
 
-	if (!knot_dname_is_sub(node->owner, contents->apex->owner)) {
+	if (knot_dname_in_bailiwick(node->owner, contents->apex->owner) <= 0) {
 		return KNOT_EOUTOFZONE;
 	}
 
@@ -87,30 +87,26 @@ static int destroy_node_rrsets_from_tree(zone_node_t **node, void *data)
 
 	if (*node != NULL) {
 		node_free_rrsets(*node, NULL);
-		node_free(node, NULL);
+		node_free(*node, NULL);
 	}
 
 	return KNOT_EOK;
 }
 
-static int create_nsec3_name(const zone_contents_t *zone,
-                             const knot_dname_t *name,
-                             knot_dname_t **nsec3_name)
+static int create_nsec3_name(uint8_t *out, size_t out_size,
+                             const zone_contents_t *zone,
+                             const knot_dname_t *name)
 {
+	assert(out);
 	assert(zone);
-	assert(nsec3_name);
+	assert(name);
 
 	if (!knot_is_nsec3_enabled(zone)) {
 		return KNOT_ENSEC3PAR;
 	}
 
-	*nsec3_name = knot_create_nsec3_owner(name, zone->apex->owner,
-	                                      &zone->nsec3_params);
-	if (*nsec3_name == NULL) {
-		return KNOT_ERROR;
-	}
-
-	return KNOT_EOK;
+	return knot_create_nsec3_owner(out, out_size, name, zone->apex->owner,
+	                               &zone->nsec3_params);
 }
 
 /*! \brief Link pointers to additional nodes for this RRSet. */
@@ -124,7 +120,7 @@ static int discover_additionals(const knot_dname_t *owner, struct rr_data *rr_da
 	rr_data->additional = NULL;
 
 	const knot_rdataset_t *rrs = &rr_data->rrs;
-	uint16_t rdcount = rrs->rr_count;
+	uint16_t rdcount = rrs->count;
 
 	uint16_t mandatory_count = 0;
 	uint16_t others_count = 0;
@@ -133,7 +129,8 @@ static int discover_additionals(const knot_dname_t *owner, struct rr_data *rr_da
 
 	/* Scan new additional nodes. */
 	for (uint16_t i = 0; i < rdcount; i++) {
-		const knot_dname_t *dname = knot_rdata_name(rrs, i, rr_data->type);
+		knot_rdata_t *rdata = knot_rdataset_at(rrs, i);
+		const knot_dname_t *dname = knot_rdata_name(rdata, rr_data->type);
 		const zone_node_t *node = NULL, *encloser = NULL, *prev = NULL;
 
 		/* Try to find node for the dname in the RDATA. */
@@ -152,7 +149,7 @@ static int discover_additionals(const knot_dname_t *owner, struct rr_data *rr_da
 		glue_t *glue;
 		if ((node->flags & (NODE_FLAGS_DELEG | NODE_FLAGS_NONAUTH)) &&
 		    rr_data->type == KNOT_RRTYPE_NS &&
-		    knot_dname_in(owner, node->owner)) {
+		    knot_dname_in_bailiwick(node->owner, owner) >= 0) {
 			glue = &mandatory[mandatory_count++];
 			glue->optional = false;
 		} else {
@@ -240,22 +237,35 @@ static int adjust_nsec3_pointers(zone_node_t **tnode, void *data)
 
 	zone_adjust_arg_t *args = (zone_adjust_arg_t *)data;
 	zone_node_t *node = *tnode;
+	const zone_node_t *ignored;
 
 	// Connect to NSEC3 node (only if NSEC3 tree is not empty)
-	zone_node_t *nsec3 = NULL;
-	knot_dname_t *nsec3_name = NULL;
-	int ret = create_nsec3_name(args->zone, node->owner, &nsec3_name);
+	node->nsec3_wildcard_prev = NULL;
+	uint8_t nsec3_name[KNOT_DNAME_MAXLEN];
+	int ret = create_nsec3_name(nsec3_name, sizeof(nsec3_name), args->zone,
+	                            node->owner);
 	if (ret == KNOT_EOK) {
-		assert(nsec3_name);
-		zone_tree_get(args->zone->nsec3_nodes, nsec3_name, &nsec3);
-		node->nsec3_node = nsec3;
+		node->nsec3_node = zone_tree_get(args->zone->nsec3_nodes, nsec3_name);
+
+		// Connect to NSEC3 node proving nonexistence of wildcard.
+		size_t wildcard_size = knot_dname_size(node->owner) + 2;
+		if (wildcard_size <= KNOT_DNAME_MAXLEN) {
+			assert(wildcard_size > 2);
+			knot_dname_t wildcard[wildcard_size];
+			memcpy(wildcard, "\x01""*", 2);
+			memcpy(wildcard + 2, node->owner, wildcard_size - 2);
+			ret = zone_contents_find_nsec3_for_name(args->zone, wildcard, &ignored,
+			                                        (const zone_node_t **)&node->nsec3_wildcard_prev);
+			if (ret == ZONE_NAME_FOUND) {
+				node->nsec3_wildcard_prev = NULL;
+				ret = KNOT_EOK;
+			}
+		}
 	} else if (ret == KNOT_ENSEC3PAR) {
 		node->nsec3_node = NULL;
 		ret = KNOT_EOK;
 	}
 
-	knot_dname_free(&nsec3_name, NULL);
-
 	return ret;
 }
 
@@ -270,6 +280,22 @@ static int measure_size(zone_node_t *node, void *data){
 	return KNOT_EOK;
 }
 
+static bool nsec3_params_match(const knot_rdataset_t *rrs,
+                               const dnssec_nsec3_params_t *params,
+                               size_t rdata_pos)
+{
+	assert(rrs != NULL);
+	assert(params != NULL);
+
+	knot_rdata_t *rdata = knot_rdataset_at(rrs, rdata_pos);
+
+	return (knot_nsec3_alg(rdata) == params->algorithm
+	        && knot_nsec3_iters(rdata) == params->iterations
+	        && knot_nsec3_salt_len(rdata) == params->salt.size
+	        && memcmp(knot_nsec3_salt(rdata), params->salt.data,
+	                  params->salt.size) == 0);
+}
+
 /*!
  * \brief Adjust normal (non NSEC3) node.
  *
@@ -328,6 +354,14 @@ static int adjust_nsec3_node(zone_node_t **tnode, void *data)
 
 	measure_size(*tnode, &args->zone->size);
 
+	// check if this node belongs to correct chain
+	const knot_rdataset_t *nsec3_rrs = node_rdataset(node, KNOT_RRTYPE_NSEC3);
+	for (uint16_t i = 0; nsec3_rrs != NULL && i < nsec3_rrs->count; i++) {
+		if (nsec3_params_match(nsec3_rrs, &args->zone->nsec3_params, i)) {
+			node->flags |= NODE_FLAGS_IN_NSEC3_CHAIN;
+		}
+	}
+
 	return KNOT_EOK;
 }
 
@@ -392,20 +426,6 @@ static bool find_in_tree(zone_tree_t *tree, const knot_dname_t *name,
 	return match > 0;
 }
 
-static bool nsec3_params_match(const knot_rdataset_t *rrs,
-                               const dnssec_nsec3_params_t *params,
-                               size_t rdata_pos)
-{
-	assert(rrs != NULL);
-	assert(params != NULL);
-
-	return (knot_nsec3_algorithm(rrs, rdata_pos) == params->algorithm
-		&& knot_nsec3_iterations(rrs, rdata_pos) == params->iterations
-		&& knot_nsec3_salt_length(rrs, rdata_pos) == params->salt.size
-		&& memcmp(knot_nsec3_salt(rrs, rdata_pos), params->salt.data,
-		          params->salt.size) == 0);
-}
-
 zone_contents_t *zone_contents_new(const knot_dname_t *apex_name)
 {
 	if (apex_name == NULL) {
@@ -443,17 +463,10 @@ cleanup:
 
 static zone_node_t *get_node(const zone_contents_t *zone, const knot_dname_t *name)
 {
-	if (zone == NULL || name == NULL) {
-		return NULL;
-	}
+	assert(zone);
+	assert(name);
 
-	zone_node_t *n;
-	int ret = zone_tree_get(zone->nodes, name, &n);
-	if (ret != KNOT_EOK) {
-		return NULL;
-	}
-
-	return n;
+	return zone_tree_get(zone->nodes, name);
 }
 
 static int add_node(zone_contents_t *zone, zone_node_t *node, bool create_parents)
@@ -484,7 +497,7 @@ static int add_node(zone_contents_t *zone, zone_node_t *node, bool create_parent
 	zone_node_t *next_node = NULL;
 	const uint8_t *parent = knot_wire_next_label(node->owner, NULL);
 
-	if (knot_dname_cmp(zone->apex->owner, parent) == 0) {
+	if (knot_dname_is_equal(zone->apex->owner, parent)) {
 		node_set_parent(node, zone->apex);
 
 		// check if the node is not wildcard child of the parent
@@ -503,7 +516,7 @@ static int add_node(zone_contents_t *zone, zone_node_t *node, bool create_parent
 			/* Insert node to a tree. */
 			ret = zone_tree_insert(zone->nodes, next_node);
 			if (ret != KNOT_EOK) {
-				node_free(&next_node, NULL);
+				node_free(next_node, NULL);
 				return ret;
 			}
 
@@ -563,17 +576,10 @@ static int add_nsec3_node(zone_contents_t *zone, zone_node_t *node)
 static zone_node_t *get_nsec3_node(const zone_contents_t *zone,
                                    const knot_dname_t *name)
 {
-	if (zone == NULL || name == NULL) {
-		return NULL;
-	}
+	assert(zone);
+	assert(name);
 
-	zone_node_t *n;
-	int ret = zone_tree_get(zone->nsec3_nodes, name, &n);
-	if (ret != KNOT_EOK) {
-		return NULL;
-	}
-
-	return n;
+	return zone_tree_get(zone->nsec3_nodes, name);
 }
 
 static int insert_rr(zone_contents_t *z, const knot_rrset_t *rr,
@@ -584,12 +590,10 @@ static int insert_rr(zone_contents_t *z, const knot_rrset_t *rr,
 	}
 
 	// check if the RRSet belongs to the zone
-	if (!knot_dname_is_sub(rr->owner, z->apex->owner) &&
-	    !knot_dname_is_equal(rr->owner, z->apex->owner)) {
+	if (knot_dname_in_bailiwick(rr->owner, z->apex->owner) < 0) {
 		return KNOT_EOUTOFZONE;
 	}
 
-	int ret = KNOT_EOK;
 	if (*n == NULL) {
 		*n = nsec3 ? get_nsec3_node(z, rr->owner) : get_node(z, rr->owner);
 		if (*n == NULL) {
@@ -598,9 +602,10 @@ static int insert_rr(zone_contents_t *z, const knot_rrset_t *rr,
 			if (*n == NULL) {
 				return KNOT_ENOMEM;
 			}
-			ret = nsec3 ? add_nsec3_node(z, *n) : add_node(z, *n, true);
+			int ret = nsec3 ? add_nsec3_node(z, *n) : add_node(z, *n, true);
 			if (ret != KNOT_EOK) {
-				node_free(n, NULL);
+				node_free(*n, NULL);
+				*n = NULL;
 			}
 		}
 	}
@@ -616,8 +621,7 @@ static int remove_rr(zone_contents_t *z, const knot_rrset_t *rr,
 	}
 
 	// check if the RRSet belongs to the zone
-	if (!knot_dname_is_sub(rr->owner, z->apex->owner) &&
-	    !knot_dname_is_equal(rr->owner, z->apex->owner)) {
+	if (knot_dname_in_bailiwick(rr->owner, z->apex->owner) < 0) {
 		return KNOT_EOUTOFZONE;
 	}
 
@@ -633,17 +637,17 @@ static int remove_rr(zone_contents_t *z, const knot_rrset_t *rr,
 
 	knot_rdataset_t *node_rrs = node_rdataset(node, rr->type);
 	// Subtract changeset RRS from node RRS.
-	int ret = knot_rdataset_subtract(node_rrs, &rr->rrs, false, NULL);
+	int ret = knot_rdataset_subtract(node_rrs, &rr->rrs, NULL);
 	if (ret != KNOT_EOK) {
 		return ret;
 	}
 
-	if (node_rrs->rr_count == 0) {
+	if (node_rrs->count == 0) {
 		// RRSet is empty now, remove it from node, all data freed.
 		node_remove_rdataset(node, rr->type);
 		// If node is empty now, delete it from zone tree.
 		if (node->rrset_count == 0 && node != z->apex) {
-			zone_tree_delete_empty_node(nsec3 ? z->nsec3_nodes : z->nodes, node);
+			zone_tree_delete_empty(nsec3 ? z->nsec3_nodes : z->nodes, node);
 		}
 	}
 
@@ -667,7 +671,7 @@ static int recreate_normal_tree(const zone_contents_t *z, zone_contents_t *out)
 	// Normal additions need apex ... so we need to insert directly.
 	int ret = zone_tree_insert(out->nodes, apex_cpy);
 	if (ret != KNOT_EOK) {
-		node_free(&apex_cpy, NULL);
+		node_free(apex_cpy, NULL);
 		return ret;
 	}
 
@@ -693,7 +697,7 @@ static int recreate_normal_tree(const zone_contents_t *z, zone_contents_t *out)
 
 		int ret = add_node(out, to_add, true);
 		if (ret != KNOT_EOK) {
-			node_free(&to_add, NULL);
+			node_free(to_add, NULL);
 			trie_it_free(itt);
 			return ret;
 		}
@@ -727,7 +731,7 @@ static int recreate_nsec3_tree(const zone_contents_t *z, zone_contents_t *out)
 		int ret = add_nsec3_node(out, to_add);
 		if (ret != KNOT_EOK) {
 			trie_it_free(itt);
-			node_free(&to_add, NULL);
+			node_free(to_add, NULL);
 			return ret;
 		}
 
@@ -774,7 +778,7 @@ zone_node_t *zone_contents_get_node_for_rr(zone_contents_t *zone, const knot_rrs
 		node = node_new(rrset->owner, NULL);
 		int ret = nsec3 ? add_nsec3_node(zone, node) : add_node(zone, node, true);
 		if (ret != KNOT_EOK) {
-			node_free(&node, NULL);
+			node_free(node, NULL);
 			return NULL;
 		}
 
@@ -786,6 +790,10 @@ zone_node_t *zone_contents_get_node_for_rr(zone_contents_t *zone, const knot_rrs
 
 const zone_node_t *zone_contents_find_node(const zone_contents_t *zone, const knot_dname_t *name)
 {
+	if (zone == NULL || name == NULL) {
+		return NULL;
+	}
+
 	return get_node(zone, name);
 }
 
@@ -810,7 +818,7 @@ int zone_contents_find_dname(const zone_contents_t *zone,
 		return KNOT_EINVAL;
 	}
 
-	if (!knot_dname_in(zone->apex->owner, name)) {
+	if (knot_dname_in_bailiwick(name, zone->apex->owner) < 0) {
 		return KNOT_EOUTOFZONE;
 	}
 
@@ -837,7 +845,7 @@ int zone_contents_find_dname(const zone_contents_t *zone,
 		assert(!node && prev);
 
 		node = prev;
-		int matched_labels = knot_dname_matched_labels(node->owner, name);
+		size_t matched_labels = knot_dname_matched_labels(node->owner, name);
 		while (matched_labels < knot_dname_labels(node->owner, NULL)) {
 			node = node->parent;
 			assert(node);
@@ -854,6 +862,10 @@ int zone_contents_find_dname(const zone_contents_t *zone,
 const zone_node_t *zone_contents_find_nsec3_node(const zone_contents_t *zone,
                                                  const knot_dname_t *name)
 {
+	if (zone == NULL || name == NULL) {
+		return NULL;
+	}
+
 	return get_nsec3_node(zone, name);
 }
 
@@ -872,8 +884,8 @@ int zone_contents_find_nsec3_for_name(const zone_contents_t *zone,
 		return KNOT_ENSEC3CHAIN;
 	}
 
-	knot_dname_t *nsec3_name = NULL;
-	int ret = create_nsec3_name(zone, name, &nsec3_name);
+	uint8_t nsec3_name[KNOT_DNAME_MAXLEN];
+	int ret = create_nsec3_name(nsec3_name, sizeof(nsec3_name), zone, name);
 	if (ret != KNOT_EOK) {
 		return ret;
 	}
@@ -881,8 +893,6 @@ int zone_contents_find_nsec3_for_name(const zone_contents_t *zone,
 	zone_node_t *found = NULL, *prev = NULL;
 	bool match = find_in_tree(zone->nsec3_nodes, nsec3_name, &found, &prev);
 
-	knot_dname_free(&nsec3_name, NULL);
-
 	*nsec3_node = found;
 
 	if (prev == NULL) {
@@ -896,33 +906,18 @@ int zone_contents_find_nsec3_for_name(const zone_contents_t *zone,
 		*nsec3_previous = prev;
 	}
 
-	/* The previous may be from wrong NSEC3 chain. Search for previous
-	 * from the right chain. Check iterations, hash algorithm and salt
-	 * values and compare them to the ones from NSEC3PARAM.
-	 */
-	const knot_rdataset_t *nsec3_rrs = node_rdataset(*nsec3_previous, KNOT_RRTYPE_NSEC3);
+	// The previous may be from wrong NSEC3 chain. Search for previous from the right chain.
 	const zone_node_t *original_prev = *nsec3_previous;
-
-	ret = match ? ZONE_NAME_FOUND : ZONE_NAME_NOT_FOUND;
-
-	while (nsec3_rrs) {
-		for (uint16_t i = 0; i < nsec3_rrs->rr_count; i++) {
-			if (nsec3_params_match(nsec3_rrs, &zone->nsec3_params, i)) {
-				return ret;
-			}
-		}
-
-		/* This RRSET was not a match, try the one from previous node. */
+	while (!((*nsec3_previous)->flags & NODE_FLAGS_IN_NSEC3_CHAIN)) {
 		*nsec3_previous = (*nsec3_previous)->prev;
-		nsec3_rrs = node_rdataset(*nsec3_previous, KNOT_RRTYPE_NSEC3);
-		if (*nsec3_previous == original_prev || nsec3_rrs == NULL) {
+		if (*nsec3_previous == original_prev || *nsec3_previous == NULL) {
 			// cycle
 			*nsec3_previous = NULL;
 			break;
 		}
 	}
 
-	return ret;
+	return (match ? ZONE_NAME_FOUND : ZONE_NAME_NOT_FOUND);
 }
 
 const zone_node_t *zone_contents_find_wildcard_child(const zone_contents_t *contents,
@@ -972,14 +967,13 @@ static int load_nsec3param(zone_contents_t *contents)
 		return KNOT_EOK;
 	}
 
-	if (rrs->rr_count < 1) {
+	if (rrs->count < 1) {
 		return KNOT_EINVAL;
 	}
 
-	const knot_rdata_t *rr = knot_rdataset_at(rrs, 0);
 	dnssec_binary_t rdata = {
-		.size = knot_rdata_rdlen(rr),
-		.data = knot_rdata_data(rr)
+		.size = rrs->rdata->len,
+		.data = rrs->rdata->data,
 	};
 
 	dnssec_nsec3_params_t new_params = { 0 };
@@ -1012,14 +1006,17 @@ static int contents_adjust(zone_contents_t *contents, bool normal)
 	};
 
 	contents->size = 0;
+	contents->dnssec = node_rrtype_is_signed(contents->apex, KNOT_RRTYPE_SOA);
 
-	ret = adjust_nodes(contents->nodes, &arg,
-	                   normal ? adjust_normal_node : adjust_pointers);
+	// NSEC3 nodes must be adjusted first, because we already need the NSEC3 chain
+	// to be closed before we adjust NSEC3 pointers in adjust_normal_node
+	ret = adjust_nodes(contents->nsec3_nodes, &arg, adjust_nsec3_node);
 	if (ret != KNOT_EOK) {
 		return ret;
 	}
 
-	ret = adjust_nodes(contents->nsec3_nodes, &arg, adjust_nsec3_node);
+	ret = adjust_nodes(contents->nodes, &arg,
+	                   normal ? adjust_normal_node : adjust_pointers);
 	if (ret != KNOT_EOK) {
 		return ret;
 	}
@@ -1106,34 +1103,35 @@ int zone_contents_shallow_copy(const zone_contents_t *from, zone_contents_t **to
 	return KNOT_EOK;
 }
 
-void zone_contents_free(zone_contents_t **contents)
+void zone_contents_free(zone_contents_t *contents)
 {
-	if (contents == NULL || *contents == NULL) {
+	if (contents == NULL) {
 		return;
 	}
 
 	// free the zone tree, but only the structure
-	zone_tree_free(&(*contents)->nodes);
-	zone_tree_free(&(*contents)->nsec3_nodes);
+	zone_tree_free(&contents->nodes);
+	zone_tree_free(&contents->nsec3_nodes);
 
-	dnssec_nsec3_params_free(&(*contents)->nsec3_params);
+	dnssec_nsec3_params_free(&contents->nsec3_params);
 
-	free(*contents);
-	*contents = NULL;
+	free(contents);
 }
 
-void zone_contents_deep_free(zone_contents_t **contents)
+void zone_contents_deep_free(zone_contents_t *contents)
 {
-	if (contents == NULL || *contents == NULL) {
+	if (contents == NULL) {
 		return;
 	}
 
-	if (*contents != NULL) {
-		// Delete NSEC3 tree
-		zone_tree_apply((*contents)->nsec3_nodes, destroy_node_rrsets_from_tree, NULL);
+	if (contents != NULL) {
+		// Delete NSEC3 tree.
+		(void)zone_tree_apply(contents->nsec3_nodes,
+		                      destroy_node_rrsets_from_tree, NULL);
 
-		// Delete normal tree
-		zone_tree_apply((*contents)->nodes, destroy_node_rrsets_from_tree, NULL);
+		// Delete the normal tree.
+		(void)zone_tree_apply(contents->nodes,
+		                      destroy_node_rrsets_from_tree, NULL);
 	}
 
 	zone_contents_free(contents);
@@ -1150,22 +1148,17 @@ uint32_t zone_contents_serial(const zone_contents_t *zone)
 		return 0;
 	}
 
-	return knot_soa_serial(soa);
+	return knot_soa_serial(soa->rdata);
 }
 
 void zone_contents_set_soa_serial(zone_contents_t *zone, uint32_t new_serial)
 {
 	knot_rdataset_t *soa;
 	if (zone != NULL && (soa = node_rdataset(zone->apex, KNOT_RRTYPE_SOA)) != NULL) {
-		knot_soa_serial_set(soa, new_serial);
+		knot_soa_serial_set(soa->rdata, new_serial);
 	}
 }
 
-bool zone_contents_is_signed(const zone_contents_t *zone)
-{
-	return node_rrtype_is_signed(zone->apex, KNOT_RRTYPE_SOA);
-}
-
 bool zone_contents_is_empty(const zone_contents_t *zone)
 {
 	if (zone == NULL) {
