@@ -1,4 +1,4 @@
-/*  Copyright (C) 2017 CZ.NIC, z.s.p.o. <knot-dns@labs.nic.cz>
+/*  Copyright (C) 2018 CZ.NIC, z.s.p.o. <knot-dns@labs.nic.cz>
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -24,7 +24,7 @@
 
 #pragma once
 
-#include "dnssec/nsec.h"
+#include "libdnssec/nsec.h"
 #include "libknot/rrtype/nsec3param.h"
 #include "knot/zone/node.h"
 #include "knot/zone/zone-tree.h"
@@ -42,6 +42,7 @@ typedef struct zone_contents {
 
 	dnssec_nsec3_params_t nsec3_params;
 	size_t size;
+	bool dnssec;
 } zone_contents_t;
 
 /*!
@@ -93,9 +94,6 @@ zone_node_t *zone_contents_get_node_for_rr(zone_contents_t *zone, const knot_rrs
 /*!
  * \brief Tries to find a node with the specified name in the zone.
  *
- * \note This function is identical to zone_contents_get_node(), only it returns
- *       constant reference.
- *
  * \param contents Zone where the name should be searched for.
  * \param name Name to find.
  *
@@ -234,14 +232,14 @@ int zone_contents_shallow_copy(const zone_contents_t *from, zone_contents_t **to
  *
  * \param contents  Zone contents to free.
  */
-void zone_contents_free(zone_contents_t **contents);
+void zone_contents_free(zone_contents_t *contents);
 
 /*!
  * \brief Deallocate node RRSets inside the trees, then call zone_contents_free.
  *
  * \param contents  Zone contents to free.
  */
-void zone_contents_deep_free(zone_contents_t **contents);
+void zone_contents_deep_free(zone_contents_t *contents);
 
 /*!
  * \brief Fetch zone serial.
@@ -262,11 +260,6 @@ uint32_t zone_contents_serial(const zone_contents_t *zone);
  */
 void zone_contents_set_soa_serial(zone_contents_t *zone, uint32_t new_serial);
 
-/*!
- * \brief Return true if zone is signed.
- */
-bool zone_contents_is_signed(const zone_contents_t *zone);
-
 /*!
  * \brief Return true if zone is empty.
  */
