@@ -1,4 +1,4 @@
-/*  Copyright (C) 2016 CZ.NIC, z.s.p.o. <knot-dns@labs.nic.cz>
+/*  Copyright (C) 2018 CZ.NIC, z.s.p.o. <knot-dns@labs.nic.cz>
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -103,10 +103,11 @@ static int flush_journal(conf_t *conf, zone_t *zone, bool allow_empty_zone)
 		return KNOT_EOK;
 	}
 
-	/* Check for difference against zonefile serial. */
+	/* Check for updated zone. */
 	zone_contents_t *contents = zone->contents;
 	uint32_t serial_to = zone_contents_serial(contents);
-	if (!force && zone->zonefile.exists && zone->zonefile.serial == serial_to) {
+	if (!force && zone->zonefile.exists && zone->zonefile.serial == serial_to &&
+	    !zone->zonefile.resigned) {
 		ret = KNOT_EOK; /* No differences. */
 		goto flush_journal_replan;
 	}
@@ -142,10 +143,11 @@ static int flush_journal(conf_t *conf, zone_t *zone, bool allow_empty_zone)
 
 	free(zonefile);
 
-	/* Update zone file serial and journal. */
+	/* Update zone file attributes. */
 	zone->zonefile.exists = true;
 	zone->zonefile.mtime = st.st_mtime;
 	zone->zonefile.serial = serial_to;
+	zone->zonefile.resigned = false;
 
 	/* Flush journal. */
 	if (zone->journal && journal_exists(zone->journal_db, zone->name)) {
@@ -194,7 +196,7 @@ zone_t* zone_new(const knot_dname_t *name)
 	// Journal
 	zone->journal = journal_new();
 	if (zone->journal == NULL) {
-		knot_dname_free(&zone->name, NULL);
+		knot_dname_free(zone->name, NULL);
 		free(zone);
 		return NULL;
 	}
@@ -242,7 +244,7 @@ void zone_free(zone_t **zone_ptr)
 
 	zone_events_deinit(zone);
 
-	knot_dname_free(&zone->name, NULL);
+	knot_dname_free(zone->name, NULL);
 
 	journal_free(&zone->journal);
 
@@ -258,7 +260,7 @@ void zone_free(zone_t **zone_ptr)
 	free(zone->preferred_master);
 
 	/* Free zone contents. */
-	zone_contents_deep_free(&zone->contents);
+	zone_contents_deep_free(zone->contents);
 
 	conf_deactivate_modules(&zone->query_modules, &zone->query_plan);
 
@@ -293,32 +295,6 @@ int zone_change_store(conf_t *conf, zone_t *zone, changeset_t *change)
 	return ret;
 }
 
-int zone_changes_store(conf_t *conf, zone_t *zone, list_t *chgs)
-{
-	if (conf == NULL || zone == NULL || chgs == NULL) {
-		return KNOT_EINVAL;
-	}
-
-	JOURNAL_LOCK_RW
-
-	int ret = open_journal(zone);
-	if (ret == KNOT_EOK) {
-		ret = journal_store_changesets(zone->journal, chgs);
-		if (ret == KNOT_EBUSY) {
-			log_zone_notice(zone->name, "journal is full, flushing");
-			/* Transaction rolled back, journal released, we may flush. */
-			ret = flush_journal(conf, zone, true);
-			if (ret == KNOT_EOK) {
-				ret = journal_store_changesets(zone->journal, chgs);
-			}
-		}
-	}
-
-	JOURNAL_UNLOCK_RW
-
-	return ret;
-}
-
 int zone_changes_load(conf_t *conf, zone_t *zone, list_t *dst, uint32_t from)
 {
 	if (conf == NULL || zone == NULL || dst == NULL) {
@@ -338,6 +314,25 @@ int zone_changes_load(conf_t *conf, zone_t *zone, list_t *dst, uint32_t from)
 	return ret;
 }
 
+int zone_chgset_ctx_load(conf_t *conf, zone_t *zone, chgset_ctx_list_t *dst, uint32_t from)
+{
+	if (conf == NULL || zone == NULL || dst == NULL) {
+		return KNOT_EINVAL;
+	}
+
+	int ret = KNOT_ENOENT;
+
+	if (journal_exists(zone->journal_db, zone->name)) {
+		ret = open_journal(zone);
+	}
+
+	if (ret == KNOT_EOK) {
+		ret = journal_load_chgset_ctx(zone->journal, dst, from);
+	}
+
+	return ret;
+}
+
 int zone_in_journal_load(conf_t *conf, zone_t *zone, list_t *dst)
 {
 	if (conf == NULL || zone == NULL || dst == NULL) {
@@ -396,7 +391,7 @@ int zone_journal_serial(conf_t *conf, zone_t *zone, bool *is_empty, uint32_t *se
 	int ret = open_journal(zone);
 	if (ret == KNOT_EOK) {
 		kserial_t ks;
-		journal_metadata_info(zone->journal, is_empty, NULL, NULL, NULL, &ks, NULL);
+		journal_metadata_info(zone->journal, is_empty, NULL, NULL, NULL, &ks, NULL, NULL);
 		*serial_to = (ks.valid ? ks.serial : 0);
 	}
 
@@ -543,11 +538,19 @@ int zone_master_try(conf_t *conf, zone_t *zone, zone_master_cb callback,
 				preferred.addr.ss_family = AF_UNSPEC;
 				continue;
 			}
+
 			int ret = callback(conf, zone, &master, callback_data);
 			if (ret == KNOT_EOK) {
 				success = true;
 				break;
 			}
+
+			char addr_str[SOCKADDR_STRLEN] = { 0 };
+			sockaddr_tostr(addr_str, sizeof(addr_str),
+			               (struct sockaddr *)&master.addr);
+			log_zone_debug(zone->name, "%s, remote %s, address %s, failed (%s)",
+			               err_str, conf_str(&masters), addr_str,
+			               knot_strerror(ret));
 		}
 
 		if (!success) {
@@ -581,7 +584,7 @@ int zone_update_enqueue(zone_t *zone, knot_pkt_t *pkt, knotd_qdata_params_t *par
 	req->query = knot_pkt_new(NULL, pkt->max_size, NULL);
 	int ret = knot_pkt_copy(req->query, pkt);
 	if (ret != KNOT_EOK) {
-		knot_pkt_free(&req->query);
+		knot_pkt_free(req->query);
 		free(req);
 		return ret;
 	}
