@@ -1,4 +1,4 @@
-/*  Copyright (C) 2011 CZ.NIC, z.s.p.o. <knot-dns@labs.nic.cz>
+/*  Copyright (C) 2018 CZ.NIC, z.s.p.o. <knot-dns@labs.nic.cz>
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -19,9 +19,6 @@
 #include <stdint.h>
 
 #include "libknot/attribute.h"
-#include "libknot/consts.h"
-#include "libknot/descriptor.h"
-#include "libknot/dname.h"
 #include "libknot/errcode.h"
 #include "libknot/rrset.h"
 #include "libknot/rrtype/naptr.h"
@@ -30,7 +27,7 @@
 
 _public_
 knot_rrset_t *knot_rrset_new(const knot_dname_t *owner, uint16_t type,
-                             uint16_t rclass, knot_mm_t *mm)
+                             uint16_t rclass, uint32_t ttl, knot_mm_t *mm)
 {
 	knot_dname_t *owner_cpy = knot_dname_copy(owner, mm);
 	if (owner_cpy == NULL) {
@@ -39,32 +36,15 @@ knot_rrset_t *knot_rrset_new(const knot_dname_t *owner, uint16_t type,
 
 	knot_rrset_t *ret = mm_alloc(mm, sizeof(knot_rrset_t));
 	if (ret == NULL) {
-		knot_dname_free(&owner_cpy, mm);
+		knot_dname_free(owner_cpy, mm);
 		return NULL;
 	}
 
-	knot_rrset_init(ret, owner_cpy, type, rclass);
+	knot_rrset_init(ret, owner_cpy, type, rclass, ttl);
 
 	return ret;
 }
 
-_public_
-void knot_rrset_init(knot_rrset_t *rrset, knot_dname_t *owner, uint16_t type,
-                     uint16_t rclass)
-{
-	rrset->owner = owner;
-	rrset->type = type;
-	rrset->rclass = rclass;
-	knot_rdataset_init(&rrset->rrs);
-	rrset->additional = NULL;
-}
-
-_public_
-void knot_rrset_init_empty(knot_rrset_t *rrset)
-{
-	knot_rrset_init(rrset, NULL, 0, KNOT_CLASS_IN);
-}
-
 _public_
 knot_rrset_t *knot_rrset_copy(const knot_rrset_t *src, knot_mm_t *mm)
 {
@@ -73,14 +53,14 @@ knot_rrset_t *knot_rrset_copy(const knot_rrset_t *src, knot_mm_t *mm)
 	}
 
 	knot_rrset_t *rrset = knot_rrset_new(src->owner, src->type,
-	                                     src->rclass, mm);
+	                                     src->rclass, src->ttl, mm);
 	if (rrset == NULL) {
 		return NULL;
 	}
 
 	int ret = knot_rdataset_copy(&rrset->rrs, &src->rrs, mm);
 	if (ret != KNOT_EOK) {
-		knot_rrset_free(&rrset, mm);
+		knot_rrset_free(rrset, mm);
 		return NULL;
 	}
 
@@ -88,40 +68,41 @@ knot_rrset_t *knot_rrset_copy(const knot_rrset_t *src, knot_mm_t *mm)
 }
 
 _public_
-void knot_rrset_free(knot_rrset_t **rrset, knot_mm_t *mm)
+void knot_rrset_free(knot_rrset_t *rrset, knot_mm_t *mm)
 {
-	if (rrset == NULL || *rrset == NULL) {
+	if (rrset == NULL) {
 		return;
 	}
 
-	knot_rrset_clear(*rrset, mm);
-
-	mm_free(mm, *rrset);
-	*rrset = NULL;
+	knot_rrset_clear(rrset, mm);
+	mm_free(mm, rrset);
 }
 
 _public_
 void knot_rrset_clear(knot_rrset_t *rrset, knot_mm_t *mm)
 {
-	if (rrset) {
-		knot_rdataset_clear(&rrset->rrs, mm);
-		knot_dname_free(&rrset->owner, mm);
+	if (rrset == NULL) {
+		return;
 	}
+
+	knot_rdataset_clear(&rrset->rrs, mm);
+	knot_dname_free(rrset->owner, mm);
+	rrset->owner = NULL;
 }
 
 _public_
-int knot_rrset_add_rdata(knot_rrset_t *rrset,
-                         const uint8_t *rdata, const uint16_t size,
-                         const uint32_t ttl, knot_mm_t *mm)
+int knot_rrset_add_rdata(knot_rrset_t *rrset, const uint8_t *data, uint16_t len,
+                         knot_mm_t *mm)
 {
-	if (rrset == NULL || (rdata == NULL && size > 0)) {
+	if (rrset == NULL || (data == NULL && len > 0)) {
 		return KNOT_EINVAL;
 	}
 
-	knot_rdata_t rr[knot_rdata_array_size(size)];
-	knot_rdata_init(rr, size, rdata, ttl);
+	uint8_t buf[knot_rdata_size(len)];
+	knot_rdata_t *rdata = (knot_rdata_t *)buf;
+	knot_rdata_init(rdata, len, data);
 
-	return knot_rdataset_add(&rrset->rrs, rr, mm);
+	return knot_rdataset_add(&rrset->rrs, rdata, mm);
 }
 
 _public_
@@ -152,23 +133,6 @@ bool knot_rrset_equal(const knot_rrset_t *r1,
 	return true;
 }
 
-_public_
-bool knot_rrset_empty(const knot_rrset_t *rrset)
-{
-	if (rrset) {
-		uint16_t rr_count = rrset->rrs.rr_count;
-		return rr_count == 0;
-	} else {
-		return true;
-	}
-}
-
-_public_
-uint32_t knot_rrset_ttl(const knot_rrset_t *rrset)
-{
-	return knot_rdata_ttl(knot_rdataset_at(&(rrset->rrs), 0));
-}
-
 _public_
 bool knot_rrset_is_nsec3rel(const knot_rrset_t *rr)
 {
@@ -179,21 +143,18 @@ bool knot_rrset_is_nsec3rel(const knot_rrset_t *rr)
 	/* Is NSEC3 or non-empty RRSIG covering NSEC3. */
 	return ((rr->type == KNOT_RRTYPE_NSEC3) ||
 	        (rr->type == KNOT_RRTYPE_RRSIG
-	         && knot_rrsig_type_covered(&rr->rrs, 0) == KNOT_RRTYPE_NSEC3));
+	         && knot_rrsig_type_covered(rr->rrs.rdata) == KNOT_RRTYPE_NSEC3));
 }
 
 _public_
 int knot_rrset_rr_to_canonical(knot_rrset_t *rrset)
 {
-	if (rrset == NULL || rrset->rrs.rr_count != 1) {
+	if (rrset == NULL || rrset->rrs.count != 1) {
 		return KNOT_EINVAL;
 	}
 
 	/* Convert owner for all RRSets. */
-	int ret = knot_dname_to_lower(rrset->owner);
-	if (ret != KNOT_EOK) {
-		return ret;
-	}
+	knot_dname_to_lower(rrset->owner);
 
 	/* Convert DNAMEs in RDATA only for RFC4034 types. */
 	if (!knot_rrtype_should_be_lowercased(rrset->type)) {
@@ -205,10 +166,8 @@ int knot_rrset_rr_to_canonical(knot_rrset_t *rrset)
 		desc = knot_get_obsolete_rdata_descriptor(rrset->type);
 	}
 
-	knot_rdata_t *rdata = knot_rdataset_at(&rrset->rrs, 0);
-	assert(rdata);
-	uint16_t rdlen = knot_rdata_rdlen(rdata);
-	uint8_t *pos = knot_rdata_data(rdata);
+	uint16_t rdlen = rrset->rrs.rdata->len;
+	uint8_t *pos = rrset->rrs.rdata->data;
 	uint8_t *endpos = pos + rdlen;
 
 	/* No RDATA */
@@ -223,20 +182,11 @@ int knot_rrset_rr_to_canonical(knot_rrset_t *rrset)
 		case KNOT_RDATA_WF_COMPRESSIBLE_DNAME:
 		case KNOT_RDATA_WF_DECOMPRESSIBLE_DNAME:
 		case KNOT_RDATA_WF_FIXED_DNAME:
-			ret = knot_dname_to_lower(pos);
-			if (ret != KNOT_EOK) {
-				return ret;
-			}
-
-			ret = knot_dname_size(pos);
-			if (ret < 0) {
-				return ret;
-			}
-
-			pos += ret;
+			knot_dname_to_lower(pos);
+			pos += knot_dname_size(pos);
 			break;
 		case KNOT_RDATA_WF_NAPTR_HEADER:
-			ret = knot_naptr_header_size(pos, endpos);
+			; int ret = knot_naptr_header_size(pos, endpos);
 			if (ret < 0) {
 				return ret;
 			}
@@ -262,16 +212,15 @@ size_t knot_rrset_size(const knot_rrset_t *rrset)
 		return 0;
 	}
 
-	uint16_t rr_count = rrset->rrs.rr_count;
+	uint16_t rr_count = rrset->rrs.count;
 
-	assert(rrset->owner);
 	size_t total_size = knot_dname_size(rrset->owner) * rr_count;
 
+	knot_rdata_t *rr = rrset->rrs.rdata;
 	for (size_t i = 0; i < rr_count; ++i) {
-		const knot_rdata_t *rr = knot_rdataset_at(&rrset->rrs, i);
-		assert(rr);
 		/* 10B = TYPE + CLASS + TTL + RDLENGTH */
-		total_size += knot_rdata_rdlen(rr) + 10;
+		total_size += rr->len + 10;
+		rr = knot_rdataset_next(rr);
 	}
 
 	return total_size;
