@@ -1,4 +1,4 @@
-/*  Copyright (C) 2016 CZ.NIC, z.s.p.o. <knot-dns@labs.nic.cz>
+/*  Copyright (C) 2018 CZ.NIC, z.s.p.o. <knot-dns@labs.nic.cz>
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -19,6 +19,7 @@
 #include <string.h>
 #include <gnutls/gnutls.h>
 #include <gnutls/x509.h>
+#include <poll.h>
 
 #include "utils/common/tls.h"
 #include "utils/common/cert.h"
@@ -223,6 +224,9 @@ static int verify_certificate(gnutls_session_t session)
 		  .data = (void *)ctx->params->hostname }
 	};
 	size_t data_count = (ctx->params->hostname != NULL) ? 2 : 1;
+	if (data_count == 1) {
+		WARN("TLS, no hostname provided, will not verify certificate owner\n")
+	}
 
 	unsigned int status;
 	ret = gnutls_certificate_verify_peers(session, data, data_count, &status);
@@ -251,6 +255,7 @@ int tls_ctx_init(tls_ctx_t *ctx, const tls_params_t *params, int wait)
 	memset(ctx, 0, sizeof(*ctx));
 	ctx->params = params;
 	ctx->wait = wait;
+	ctx->sockfd = -1;
 
 	int ret = gnutls_certificate_allocate_credentials(&ctx->credentials);
 	if (ret != GNUTLS_E_SUCCESS) {
@@ -258,7 +263,8 @@ int tls_ctx_init(tls_ctx_t *ctx, const tls_params_t *params, int wait)
 	}
 
 	// Import system certificates.
-	if (ctx->params->system_ca) {
+	if (ctx->params->system_ca ||
+	    (ctx->params->hostname != NULL && EMPTY_LIST(ctx->params->ca_files))) {
 		ret = gnutls_certificate_set_x509_system_trust(ctx->credentials);
 		if (ret < 0) {
 			WARN("TLS, failed to import system certificates (%s)\n",
@@ -323,9 +329,22 @@ int tls_ctx_connect(tls_ctx_t *ctx, int sockfd,  const char *remote)
 	gnutls_transport_set_int(ctx->session, sockfd);
 	gnutls_handshake_set_timeout(ctx->session, 1000 * ctx->wait);
 
+	// Initialize poll descriptor structure.
+	struct pollfd pfd = {
+		.fd = sockfd,
+		.events = POLLIN,
+		.revents = 0,
+	};
+
 	// Perform the TLS handshake
 	do {
 		ret = gnutls_handshake(ctx->session);
+		if (ret != GNUTLS_E_SUCCESS && gnutls_error_is_fatal(ret) == 0) {
+			if (poll(&pfd, 1, 1000 * ctx->wait) != 1) {
+				WARN("TLS, peer took too long to respond\n");
+				return KNOT_NET_ETIMEOUT;
+			}
+		}
 	} while (ret != GNUTLS_E_SUCCESS && gnutls_error_is_fatal(ret) == 0);
 	if (ret != GNUTLS_E_SUCCESS) {
 		WARN("TLS, handshake failed (%s)\n", gnutls_strerror(ret));
@@ -333,6 +352,9 @@ int tls_ctx_connect(tls_ctx_t *ctx, int sockfd,  const char *remote)
 		return KNOT_NET_ESOCKET;
 	}
 
+	// Save the socket descriptor.
+	ctx->sockfd = sockfd;
+
 	return KNOT_EOK;
 }
 
@@ -372,9 +394,16 @@ int tls_ctx_receive(tls_ctx_t *ctx, uint8_t *buf, const size_t buf_len)
 		return KNOT_EINVAL;
 	}
 
-	uint32_t total = 0;
+	// Initialize poll descriptor structure.
+	struct pollfd pfd = {
+		.fd = ctx->sockfd,
+		.events = POLLIN,
+		.revents = 0,
+	};
 
+	uint32_t total = 0;
 	uint16_t msg_len = 0;
+
 	// Receive message header.
 	while (total < sizeof(msg_len)) {
 		ssize_t ret = gnutls_record_recv(ctx->session, &msg_len + total,
@@ -388,6 +417,9 @@ int tls_ctx_receive(tls_ctx_t *ctx, uint8_t *buf, const size_t buf_len)
 			WARN("TLS, failed to receive reply (%s)\n",
 			     gnutls_strerror(ret));
 			return KNOT_NET_ERECV;
+		} else if (poll(&pfd, 1, 1000 * ctx->wait) != 1) {
+			WARN("TLS, peer took too long to respond\n");
+			return KNOT_NET_ETIMEOUT;
 		}
 	}
 
@@ -412,6 +444,9 @@ int tls_ctx_receive(tls_ctx_t *ctx, uint8_t *buf, const size_t buf_len)
 			WARN("TLS, failed to receive reply (%s)\n",
 			     gnutls_strerror(ret));
 			return KNOT_NET_ERECV;
+		} else if (poll(&pfd, 1, 1000 * ctx->wait) != 1) {
+			WARN("TLS, peer took too long to respond\n");
+			return KNOT_NET_ETIMEOUT;
 		}
 	}
 
@@ -425,6 +460,7 @@ void tls_ctx_close(tls_ctx_t *ctx)
 	}
 
 	gnutls_bye(ctx->session, GNUTLS_SHUT_RDWR);
+	gnutls_deinit(ctx->session);
 }
 
 void tls_ctx_deinit(tls_ctx_t *ctx)
@@ -433,10 +469,6 @@ void tls_ctx_deinit(tls_ctx_t *ctx)
 		return;
 	}
 
-	if (ctx->session != NULL) {
-		gnutls_deinit(ctx->session);
-	}
-
 	if (ctx->credentials != NULL) {
 		gnutls_certificate_free_credentials(ctx->credentials);
 	}
