@@ -1,4 +1,4 @@
-/*  Copyright (C) 2016 CZ.NIC, z.s.p.o. <knot-dns@labs.nic.cz>
+/*  Copyright (C) 2018 CZ.NIC, z.s.p.o. <knot-dns@labs.nic.cz>
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -22,6 +22,7 @@
 
 #include "utils/kdig/kdig_params.h"
 #include "utils/common/cert.h"
+#include "utils/common/hex.h"
 #include "utils/common/msg.h"
 #include "utils/common/params.h"
 #include "utils/common/resolv.h"
@@ -29,8 +30,11 @@
 #include "libknot/libknot.h"
 #include "contrib/base64.h"
 #include "contrib/sockaddr.h"
+#include "contrib/string.h"
 #include "contrib/strtonum.h"
 #include "contrib/ucw/lists.h"
+#include "libdnssec/error.h"
+#include "libdnssec/random.h"
 
 #define PROGRAM_NAME "kdig"
 
@@ -56,6 +60,7 @@ static const style_t DEFAULT_STYLE_DIG = {
 		.show_class = true,
 		.show_ttl = true,
 		.verbose = false,
+		.original_ttl = false,
 		.empty_ttl = false,
 		.human_ttl = false,
 		.human_tmstamp = true,
@@ -742,6 +747,79 @@ static int opt_nobufsize(const char *arg, void *query)
 	return KNOT_EOK;
 }
 
+static int opt_cookie(const char *arg, void *query)
+{
+	query_t *q = query;
+
+	if (arg != NULL) {
+		uint8_t *input = NULL;
+		size_t input_len;
+
+		int ret = hex_decode(arg, &input, &input_len);
+		if (ret != KNOT_EOK) {
+			ERR("invalid +cookie=%s\n", arg);
+			return KNOT_EINVAL;
+		}
+
+		if (input_len < KNOT_EDNS_COOKIE_CLNT_SIZE) {
+			ERR("too short client +cookie=%s\n", arg);
+			free(input);
+			return KNOT_EINVAL;
+		}
+		q->cc.len = KNOT_EDNS_COOKIE_CLNT_SIZE;
+		memcpy(q->cc.data, input, q->cc.len);
+
+		input_len -= q->cc.len;
+		if (input_len > 0) {
+			if (input_len < KNOT_EDNS_COOKIE_SRVR_MIN_SIZE) {
+				ERR("too short server +cookie=%s\n", arg);
+				free(input);
+				return KNOT_EINVAL;
+			}
+			if (input_len > KNOT_EDNS_COOKIE_SRVR_MAX_SIZE) {
+				ERR("too long server +cookie=%s\n", arg);
+				free(input);
+				return KNOT_EINVAL;
+			}
+			q->sc.len = input_len;
+			memcpy(q->sc.data, input + q->cc.len, q->sc.len);
+		}
+
+		free(input);
+	} else {
+		q->cc.len = KNOT_EDNS_COOKIE_CLNT_SIZE;
+
+		int ret = dnssec_random_buffer(q->cc.data, q->cc.len);
+		if (ret != DNSSEC_EOK) {
+			return knot_error_from_libdnssec(ret);
+		}
+	}
+
+	return KNOT_EOK;
+}
+
+static int opt_nocookie(const char *arg, void *query)
+{
+	query_t *q = query;
+	q->cc.len = 0;
+	q->sc.len = 0;
+	return KNOT_EOK;
+}
+
+static int opt_badcookie(const char *arg, void *query)
+{
+	query_t *q = query;
+	q->badcookie = true;
+	return KNOT_EOK;
+}
+
+static int opt_nobadcookie(const char *arg, void *query)
+{
+	query_t *q = query;
+	q->badcookie = false;
+	return KNOT_EOK;
+}
+
 static int opt_padding(const char *arg, void *query)
 {
 	query_t *q = query;
@@ -807,11 +885,6 @@ static int opt_subnet(const char *arg, void *query)
 	const char   *arg_end = arg + arg_len;
 	size_t       addr_len = 0;
 
-	knot_edns_client_subnet_t *subnet = calloc(1, sizeof(*subnet));
-	if (subnet == NULL) {
-		return KNOT_ENOMEM;
-	}
-
 	// Separate address and network mask.
 	if ((sep = strchr(arg, '/')) != NULL) {
 		addr_len = sep - arg;
@@ -828,7 +901,6 @@ static int opt_subnet(const char *arg, void *query)
 	char *addr_str = strndup(arg, addr_len);
 	if (getaddrinfo(addr_str, NULL, &hints, &ai) != 0) {
 		free(addr_str);
-		free(subnet);
 		ERR("invalid address +subnet=%s\n", arg);
 		return KNOT_EINVAL;
 	}
@@ -837,8 +909,7 @@ static int opt_subnet(const char *arg, void *query)
 	freeaddrinfo(ai);
 	free(addr_str);
 
-	if (knot_edns_client_subnet_set_addr(subnet, &ss) != KNOT_EOK) {
-		free(subnet);
+	if (knot_edns_client_subnet_set_addr(&q->subnet, &ss) != KNOT_EOK) {
 		ERR("invalid address +subnet=%s\n", arg);
 		return KNOT_EINVAL;
 	}
@@ -848,17 +919,13 @@ static int opt_subnet(const char *arg, void *query)
 	if (mask + addr_len < arg_end) {
 		mask += addr_len + 1;
 		uint8_t num = 0;
-		if (str_to_u8(mask, &num) != KNOT_EOK || num > subnet->source_len) {
-			free(subnet);
+		if (str_to_u8(mask, &num) != KNOT_EOK || num > q->subnet.source_len) {
 			ERR("invalid network mask +subnet=%s\n", arg);
 			return KNOT_EINVAL;
 		}
-		subnet->source_len = num;
+		q->subnet.source_len = num;
 	}
 
-	free(q->subnet);
-	q->subnet = subnet;
-
 	return KNOT_EOK;
 }
 
@@ -866,8 +933,7 @@ static int opt_nosubnet(const char *arg, void *query)
 {
 	query_t *q = query;
 
-	free(q->subnet);
-	q->subnet = NULL;
+	q->subnet.family = AF_UNSPEC;
 
 	return KNOT_EOK;
 }
@@ -899,6 +965,7 @@ static int opt_noedns(const char *arg, void *query)
 	opt_nodoflag(arg, query);
 	opt_nonsid(arg, query);
 	opt_nobufsize(arg, query);
+	opt_nocookie(arg, query);
 	opt_nopadding(arg, query);
 	opt_noalignment(arg, query);
 	opt_nosubnet(arg, query);
@@ -948,6 +1015,67 @@ static int opt_noretry(const char *arg, void *query)
 	return KNOT_EOK;
 }
 
+static int parse_ednsopt(const char *arg, ednsopt_t **opt_ptr)
+{
+	errno = 0;
+	char *end = NULL;
+	unsigned long code = strtoul(arg, &end, 10);
+	if (errno != 0 || arg == end || code > UINT16_MAX) {
+		return KNOT_EINVAL;
+	}
+
+	size_t length = 0;
+	uint8_t *data = NULL;
+	if (end[0] == ':') {
+		if (end[1] != '\0') {
+			int ret = hex_decode(end + 1, &data, &length);
+			if (ret != KNOT_EOK) {
+				return ret;
+			}
+			if (length > UINT16_MAX) {
+				free(data);
+				return KNOT_ERANGE;
+			}
+		}
+	} else if (end[0] != '\0') {
+		return KNOT_EINVAL;
+	}
+
+	ednsopt_t *opt = ednsopt_create(code, length, data);
+	if (opt == NULL) {
+		free(data);
+		return KNOT_ENOMEM;
+	}
+
+	*opt_ptr = opt;
+	return KNOT_EOK;
+}
+
+static int opt_ednsopt(const char *arg, void *query)
+{
+	query_t *q = query;
+
+	ednsopt_t *opt = NULL;
+	int ret = parse_ednsopt(arg, &opt);
+	if (ret != KNOT_EOK) {
+		ERR("invalid +ednsopt=%s\n", arg);
+		return KNOT_EINVAL;
+	}
+
+	add_tail(&q->edns_opts, &opt->n);
+
+	return KNOT_EOK;
+}
+
+static int opt_noednsopt(const char *arg, void *query)
+{
+	query_t *q = query;
+
+	ednsopt_list_deinit(&q->edns_opts);
+
+	return KNOT_EOK;
+}
+
 static int opt_noidn(const char *arg, void *query)
 {
 	query_t *q = query;
@@ -1083,6 +1211,15 @@ static const param_t kdig_opts2[] = {
 	{ "retry",          ARG_REQUIRED, opt_retry },
 	{ "noretry",        ARG_NONE,     opt_noretry },
 
+	{ "cookie",         ARG_OPTIONAL, opt_cookie },
+	{ "nocookie",       ARG_NONE,     opt_nocookie },
+
+	{ "badcookie",      ARG_NONE,     opt_badcookie },
+	{ "nobadcookie",    ARG_NONE,     opt_nobadcookie },
+
+	{ "ednsopt",        ARG_REQUIRED, opt_ednsopt },
+	{ "noednsopt",      ARG_NONE,     opt_noednsopt },
+
 	/* "idn" doesn't work since it must be called before query creation. */
 	{ "noidn",          ARG_NONE,     opt_noidn },
 
@@ -1099,17 +1236,6 @@ query_t *query_create(const char *owner, const query_t *conf)
 		return NULL;
 	}
 
-	// Set the query owner if any.
-	if (owner != NULL) {
-		if ((query->owner = strdup(owner)) == NULL) {
-			query_free(query);
-			return NULL;
-		}
-	}
-
-	// Initialize list of servers.
-	init_list(&query->servers);
-
 	// Initialization with defaults or with reference query.
 	if (conf == NULL) {
 		query->conf = NULL;
@@ -1132,16 +1258,21 @@ query_t *query_create(const char *owner, const query_t *conf)
 		query->idn = true;
 		query->nsid = false;
 		query->edns = -1;
+		query->cc.len = 0;
+		query->sc.len = 0;
+		query->badcookie = true;
 		query->padding = -1;
 		query->alignment = 0;
 		tls_params_init(&query->tls);
 		//query->tsig_key
-		query->subnet = NULL;
+		query->subnet.family = AF_UNSPEC;
+		ednsopt_list_init(&query->edns_opts);
 #if USE_DNSTAP
 		query->dt_reader = NULL;
 		query->dt_writer = NULL;
 #endif // USE_DNSTAP
 	} else {
+		*query = *conf;
 		query->conf = conf;
 		if (conf->local != NULL) {
 			query->local = srv_info_create(conf->local->name,
@@ -1153,26 +1284,7 @@ query_t *query_create(const char *owner, const query_t *conf)
 		} else {
 			query->local = NULL;
 		}
-		query->operation = conf->operation;
-		query->ip = conf->ip;
-		query->protocol = conf->protocol;
-		query->fastopen = conf->fastopen;
 		query->port = strdup(conf->port);
-		query->udp_size = conf->udp_size;
-		query->retries = conf->retries;
-		query->wait = conf->wait;
-		query->ignore_tc = conf->ignore_tc;
-		query->class_num = conf->class_num;
-		query->type_num = conf->type_num;
-		query->serial = conf->serial;
-		query->notify = conf->notify;
-		query->flags = conf->flags;
-		query->style = conf->style;
-		query->idn = conf->idn;
-		query->nsid = conf->nsid;
-		query->edns = conf->edns;
-		query->padding = conf->padding;
-		query->alignment = conf->alignment;
 		tls_params_copy(&query->tls, &conf->tls);
 		if (conf->tsig_key.name != NULL) {
 			int ret = knot_tsig_key_copy(&query->tsig_key,
@@ -1182,22 +1294,30 @@ query_t *query_create(const char *owner, const query_t *conf)
 				return NULL;
 			}
 		}
-		if (conf->subnet != NULL) {
-			query->subnet = malloc(sizeof(*query->subnet));
-			if (query->subnet == NULL) {
-				query_free(query);
-				return NULL;
-			}
-			*(query->subnet) = *(conf->subnet);
-		} else {
-			query->subnet = NULL;
+
+		int ret = ednsopt_list_dup(&query->edns_opts, &conf->edns_opts);
+		if (ret != KNOT_EOK) {
+			query_free(query);
+			return NULL;
 		}
+
 #if USE_DNSTAP
 		query->dt_reader = conf->dt_reader;
 		query->dt_writer = conf->dt_writer;
 #endif // USE_DNSTAP
 	}
 
+	// Initialize list of servers.
+	init_list(&query->servers);
+
+	// Set the query owner if any.
+	if (owner != NULL) {
+		if ((query->owner = strdup(owner)) == NULL) {
+			query_free(query);
+			return NULL;
+		}
+	}
+
 	// Check dynamic allocation.
 	if (query->port == NULL) {
 		query_free(query);
@@ -1231,6 +1351,9 @@ void query_free(query_t *query)
 	// Cleanup signing key.
 	knot_tsig_key_deinit(&query->tsig_key);
 
+	// Cleanup EDNS options.
+	ednsopt_list_deinit(&query->edns_opts);
+
 #if USE_DNSTAP
 	if (query->dt_reader != NULL) {
 		dt_reader_free(query->dt_reader);
@@ -1246,10 +1369,93 @@ void query_free(query_t *query)
 
 	free(query->owner);
 	free(query->port);
-	free(query->subnet);
 	free(query);
 }
 
+ednsopt_t *ednsopt_create(uint16_t code, uint16_t length, uint8_t *data)
+{
+	ednsopt_t *opt = calloc(1, sizeof(*opt));
+	if (opt == NULL) {
+		return NULL;
+	}
+
+	opt->code = code;
+	opt->length = length;
+	opt->data = data;
+
+	return opt;
+}
+
+ednsopt_t *ednsopt_dup(const ednsopt_t *opt)
+{
+	ednsopt_t *dup = calloc(1, sizeof(*opt));
+	if (dup == NULL) {
+		return NULL;
+	}
+
+	dup->code = opt->code;
+	dup->length = opt->length;
+	dup->data = memdup(opt->data, opt->length);
+	if (dup->data == NULL) {
+		free(dup);
+		return NULL;
+	}
+
+	return dup;
+}
+
+void ednsopt_free(ednsopt_t *opt)
+{
+	if (opt == NULL) {
+		return;
+	}
+
+	free(opt->data);
+	free(opt);
+}
+
+void ednsopt_list_init(list_t *list)
+{
+	init_list(list);
+}
+
+void ednsopt_list_deinit(list_t *list)
+{
+	node_t *n, *next;
+	WALK_LIST_DELSAFE(n, next, *list) {
+		ednsopt_t *opt = (ednsopt_t *)n;
+		ednsopt_free(opt);
+	}
+
+	init_list(list);
+}
+
+int ednsopt_list_dup(list_t *dest, const list_t *src)
+{
+	list_t backup = *dest;
+	init_list(dest);
+
+	node_t *n = NULL;
+	WALK_LIST(n, *src) {
+		ednsopt_t *opt = (ednsopt_t *)n;
+		ednsopt_t *dup = ednsopt_dup(opt);
+		if (dup == NULL) {
+			ednsopt_list_deinit(dest);
+			*dest = backup;
+			return KNOT_ENOMEM;
+		}
+
+		add_tail(dest, &dup->n);
+	}
+
+	return KNOT_EOK;
+}
+
+bool ednsopt_list_empty(const list_t *list)
+{
+	return EMPTY_LIST(*list);
+}
+
 int kdig_init(kdig_params_t *params)
 {
 	if (params == NULL) {
@@ -1337,19 +1543,22 @@ static int parse_name(const char *value, list_t *queries, const query_t *conf)
 {
 	query_t	*query = NULL;
 	char	*ascii_name = (char *)value;
+	char	*fqd_name = NULL;
 
-	if (conf->idn) {
-		ascii_name = name_from_idn(value);
-		if (ascii_name == NULL) {
-			return KNOT_EINVAL;
+	if (value != NULL) {
+		if (conf->idn) {
+			ascii_name = name_from_idn(value);
+			if (ascii_name == NULL) {
+				return KNOT_EINVAL;
+			}
 		}
-	}
 
-	// If name is not FQDN, append trailing dot.
-	char *fqd_name = get_fqd_name(ascii_name);
+		// If name is not FQDN, append trailing dot.
+		fqd_name = get_fqd_name(ascii_name);
 
-	if (conf->idn) {
-		free(ascii_name);
+		if (conf->idn) {
+			free(ascii_name);
+		}
 	}
 
 	// Create new query.
@@ -1642,50 +1851,53 @@ static void print_help(void)
 	       "            [-y [algo:]keyname:key] [-E tapfile] [-G tapfile]\n"
 	       "            name [type] [class] [@server]\n"
 	       "\n"
-	       "       +[no]multiline        Wrap long records to more lines.\n"
-	       "       +[no]short            Show record data only.\n"
-	       "       +[no]generic          Use generic representation format.\n"
-	       "       +[no]aaflag           Set AA flag.\n"
-	       "       +[no]tcflag           Set TC flag.\n"
-	       "       +[no]rdflag           Set RD flag.\n"
-	       "       +[no]recurse          Same as +[no]rdflag\n"
-	       "       +[no]raflag           Set RA flag.\n"
-	       "       +[no]zflag            Set zero flag bit.\n"
-	       "       +[no]adflag           Set AD flag.\n"
-	       "       +[no]cdflag           Set CD flag.\n"
-	       "       +[no]dnssec           Set DO flag.\n"
-	       "       +[no]all              Show all packet sections.\n"
-	       "       +[no]qr               Show query packet.\n"
-	       "       +[no]header           Show packet header.\n"
-	       "       +[no]opt              Show EDNS pseudosection.\n"
-	       "       +[no]question         Show question section.\n"
-	       "       +[no]answer           Show answer section.\n"
-	       "       +[no]authority        Show authority section.\n"
-	       "       +[no]additional       Show additional section.\n"
-	       "       +[no]tsig             Show TSIG pseudosection.\n"
-	       "       +[no]stats            Show trailing packet statistics.\n"
-	       "       +[no]class            Show DNS class.\n"
-	       "       +[no]ttl              Show TTL value.\n"
-	       "       +[no]crypto           Show binary parts of RRSIGs and DNSKEYs.\n"
-	       "       +[no]tcp              Use TCP protocol.\n"
-	       "       +[no]fastopen         Use TCP Fast Open.\n"
-	       "       +[no]ignore           Don't use TCP automatically if truncated.\n"
-	       "       +[no]tls              Use TLS with Opportunistic privacy profile.\n"
-	       "       +[no]tls-ca[=FILE]    Use TLS with Out-Of-Band privacy profile.\n"
-	       "       +[no]tls-pin=BASE64   Use TLS with pinned certificate.\n"
-	       "       +[no]tls-hostname=STR Use TLS with remote server hostname.\n"
-	       "       +[no]nsid             Request NSID.\n"
-	       "       +[no]bufsize=B        Set EDNS buffer size.\n"
-	       "       +[no]padding[=N]      Pad with EDNS(0) (default or specify size).\n"
-	       "       +[no]alignment[=N]    Pad with EDNS(0) to blocksize (%u or specify size).\n"
-	       "       +[no]subnet=SUBN      Set EDNS(0) client subnet addr/prefix.\n"
-	       "       +[no]edns[=N]         Use EDNS(=version).\n"
-	       "       +[no]time=T           Set wait for reply interval in seconds.\n"
-	       "       +[no]retry=N          Set number of retries.\n"
-	       "       +noidn                Disable IDN transformation.\n"
+	       "       +[no]multiline            Wrap long records to more lines.\n"
+	       "       +[no]short                Show record data only.\n"
+	       "       +[no]generic              Use generic representation format.\n"
+	       "       +[no]aaflag               Set AA flag.\n"
+	       "       +[no]tcflag               Set TC flag.\n"
+	       "       +[no]rdflag               Set RD flag.\n"
+	       "       +[no]recurse              Same as +[no]rdflag\n"
+	       "       +[no]raflag               Set RA flag.\n"
+	       "       +[no]zflag                Set zero flag bit.\n"
+	       "       +[no]adflag               Set AD flag.\n"
+	       "       +[no]cdflag               Set CD flag.\n"
+	       "       +[no]dnssec               Set DO flag.\n"
+	       "       +[no]all                  Show all packet sections.\n"
+	       "       +[no]qr                   Show query packet.\n"
+	       "       +[no]header               Show packet header.\n"
+	       "       +[no]opt                  Show EDNS pseudosection.\n"
+	       "       +[no]question             Show question section.\n"
+	       "       +[no]answer               Show answer section.\n"
+	       "       +[no]authority            Show authority section.\n"
+	       "       +[no]additional           Show additional section.\n"
+	       "       +[no]tsig                 Show TSIG pseudosection.\n"
+	       "       +[no]stats                Show trailing packet statistics.\n"
+	       "       +[no]class                Show DNS class.\n"
+	       "       +[no]ttl                  Show TTL value.\n"
+	       "       +[no]crypto               Show binary parts of RRSIGs and DNSKEYs.\n"
+	       "       +[no]tcp                  Use TCP protocol.\n"
+	       "       +[no]fastopen             Use TCP Fast Open.\n"
+	       "       +[no]ignore               Don't use TCP automatically if truncated.\n"
+	       "       +[no]tls                  Use TLS with Opportunistic privacy profile.\n"
+	       "       +[no]tls-ca[=FILE]        Use TLS with Out-Of-Band privacy profile.\n"
+	       "       +[no]tls-pin=BASE64       Use TLS with pinned certificate.\n"
+	       "       +[no]tls-hostname=STR     Use TLS with remote server hostname.\n"
+	       "       +[no]nsid                 Request NSID.\n"
+	       "       +[no]bufsize=B            Set EDNS buffer size.\n"
+	       "       +[no]padding[=N]          Pad with EDNS(0) (default or specify size).\n"
+	       "       +[no]alignment[=N]        Pad with EDNS(0) to blocksize (%u or specify size).\n"
+	       "       +[no]subnet=SUBN          Set EDNS(0) client subnet addr/prefix.\n"
+	       "       +[no]edns[=N]             Use EDNS(=version).\n"
+	       "       +[no]time=T               Set wait for reply interval in seconds.\n"
+	       "       +[no]retry=N              Set number of retries.\n"
+	       "       +[no]cookie=HEX           Attach EDNS(0) cookie to the query.\n"
+	       "       +[no]badcookie            Repeat a query with the correct cookie.\n"
+	       "       +[no]ednsopt=CODE[:HEX]   Set custom EDNS option.\n"
+	       "       +noidn                    Disable IDN transformation.\n"
 	       "\n"
-	       "       -h, --help            Print the program help.\n"
-	       "       -V, --version         Print the program version.\n",
+	       "       -h, --help                Print the program help.\n"
+	       "       -V, --version             Print the program version.\n",
 	       PROGRAM_NAME, DEFAULT_ALIGNMENT_SIZE);
 }
 
@@ -1788,11 +2000,7 @@ static int parse_opt1(const char *opt, const char *value, kdig_params_t *params,
 		*index += add;
 		break;
 	case 'q':
-		if (val == NULL) {
-			ERR("missing name\n");
-			return KNOT_EINVAL;
-		}
-
+		// Allow empty QNAME.
 		if (parse_name(val, &params->queries, params->config)
 		    != KNOT_EOK) {
 			ERR("bad query name %s\n", val);
