@@ -1,4 +1,4 @@
-/*  Copyright (C) 2017 CZ.NIC, z.s.p.o. <knot-dns@labs.nic.cz>
+/*  Copyright (C) 2018 CZ.NIC, z.s.p.o. <knot-dns@labs.nic.cz>
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -26,11 +26,11 @@
 #include "contrib/ctype.h"
 #include "contrib/tolower.h"
 #include "contrib/wire_ctx.h"
-#include "dnssec/lib/dnssec/error.h"
-#include "dnssec/shared/shared.h"
+#include "libdnssec/error.h"
+#include "libdnssec/shared/shared.h"
 #include "knot/dnssec/kasp/policy.h"
 #include "knot/dnssec/zone-keys.h"
-#include "zscanner/scanner.h"
+#include "libzscanner/scanner.h"
 
 static bool is_timestamp(char *arg, knot_kasp_key_timing_t *timing)
 {
@@ -83,8 +83,17 @@ static bool str2bool(const char *s)
 	}
 }
 
+static void bitmap_set(kdnssec_generate_flags_t *bitmap, int flag, bool onoff)
+{
+        if (onoff) {
+                *bitmap |= flag;
+        } else {
+                *bitmap &= ~flag;
+        }
+}
+
 static bool genkeyargs(int argc, char *argv[], bool just_timing,
-                       bool *isksk, bool *iszsk, dnssec_key_algorithm_t *algorithm,
+                       kdnssec_generate_flags_t *flags, dnssec_key_algorithm_t *algorithm,
                        uint16_t *keysize, knot_kasp_key_timing_t *timing,
                        const char **addtopolicy)
 {
@@ -117,10 +126,13 @@ static bool genkeyargs(int argc, char *argv[], bool just_timing,
 				printf("Unknown algorithm: %s\n", argv[i] + 10);
 				return false;
 			}
-		} else if (!just_timing && strncasecmp(argv[i], "ksk=", 4) == 0) {
-			*isksk = str2bool(argv[i] + 4);
-		} else if (!just_timing && strncasecmp(argv[i], "zsk=", 4) == 0) {
-			*iszsk = str2bool(argv[i] + 4);
+		} else if (strncasecmp(argv[i], "ksk=", 4) == 0) {
+			bitmap_set(flags, DNSKEY_GENERATE_KSK, str2bool(argv[i] + 4));
+		} else if (strncasecmp(argv[i], "zsk=", 4) == 0) {
+			bitmap_set(flags, DNSKEY_GENERATE_ZSK, str2bool(argv[i] + 4));
+		} else if (!just_timing && strncasecmp(argv[i], "sep=", 4) == 0) {
+			bitmap_set(flags, DNSKEY_GENERATE_SEP_SPEC, true);
+			bitmap_set(flags, DNSKEY_GENERATE_SEP_ON, str2bool(argv[i] + 4));
 		} else if (!just_timing && strncasecmp(argv[i], "size=", 5) == 0) {
 			*keysize = atol(argv[i] + 5);
 		} else if (!just_timing && strncasecmp(argv[i], "addtopolicy=", 12) == 0) {
@@ -131,11 +143,6 @@ static bool genkeyargs(int argc, char *argv[], bool just_timing,
 		}
 	}
 
-	// set ZSK if none was specified
-	if (isksk && iszsk && !*isksk) {
-		*iszsk = true;
-	}
-
 	return true;
 }
 
@@ -144,15 +151,15 @@ int keymgr_generate_key(kdnssec_ctx_t *ctx, int argc, char *argv[])
 {
 	knot_time_t now = knot_time(), infty = 0;
 	knot_kasp_key_timing_t gen_timing = { now, infty, now, infty, now, infty, infty, infty, infty };
-	bool isksk = false, iszsk = false;
+	kdnssec_generate_flags_t flags = 0;
 	uint16_t keysize = 0;
 	const char *addtopolicy = NULL;
-	if (!genkeyargs(argc, argv, false, &isksk, &iszsk, &ctx->policy->algorithm,
+	if (!genkeyargs(argc, argv, false, &flags, &ctx->policy->algorithm,
 			&keysize, &gen_timing, &addtopolicy)) {
 		return KNOT_EINVAL;
 	}
 	if (keysize > 0) {
-		if (isksk) {
+		if ((flags & DNSKEY_GENERATE_KSK)) {
 			ctx->policy->ksk_size = keysize;
 		} else {
 			ctx->policy->zsk_size = keysize;
@@ -161,7 +168,7 @@ int keymgr_generate_key(kdnssec_ctx_t *ctx, int argc, char *argv[])
 
 	for (size_t i = 0; i < ctx->zone->num_keys; i++) {
 		knot_kasp_key_t *kasp_key = &ctx->zone->keys[i];
-		if (dnssec_key_get_flags(kasp_key->key) == dnskey_flags(isksk) &&
+		if ((kasp_key->is_ksk && (flags & DNSKEY_GENERATE_KSK)) &&
 		    dnssec_key_get_algorithm(kasp_key->key) != ctx->policy->algorithm) {
 			printf("warning: creating key with different algorithm than "
 			       "configured in the policy\n");
@@ -170,7 +177,7 @@ int keymgr_generate_key(kdnssec_ctx_t *ctx, int argc, char *argv[])
 	}
 
 	knot_kasp_key_t *key = NULL;
-	int ret = kdnssec_generate_key(ctx, isksk, iszsk, &key);
+	int ret = kdnssec_generate_key(ctx, flags, &key);
 	if (ret != KNOT_EOK) {
 		return ret;
 	}
@@ -180,10 +187,10 @@ int keymgr_generate_key(kdnssec_ctx_t *ctx, int argc, char *argv[])
 	if (addtopolicy != NULL) {
 		char *last_policy_last = NULL;
 
-		knot_dname_t *unused;
+		knot_dname_t *unused = NULL;
 		ret = kasp_db_get_policy_last(*ctx->kasp_db, addtopolicy, &unused,
 		                              &last_policy_last);
-		knot_dname_free(&unused, NULL);
+		knot_dname_free(unused, NULL);
 		if (ret != KNOT_EOK && ret != KNOT_ENOENT) {
 			return ret;
 		}
@@ -370,64 +377,73 @@ fail:
 	return knot_error_from_libdnssec(ret);
 }
 
-int keymgr_import_pem(kdnssec_ctx_t *ctx, const char *import_file, int argc, char *argv[])
+static int import_key(kdnssec_ctx_t *ctx, unsigned backend, const char *param,
+                      int argc, char *argv[])
 {
-	if (ctx == NULL || import_file == NULL) {
+	if (ctx == NULL || param == NULL) {
 		return KNOT_EINVAL;
 	}
 
 	// parse params
 	knot_time_t now = knot_time();
 	knot_kasp_key_timing_t timing = { .publish = now, .active = now };
-	bool isksk = false, iszsk = false;
+	kdnssec_generate_flags_t flags = 0;
 	uint16_t keysize = 0;
-	if (!genkeyargs(argc, argv, false, &isksk, &iszsk, &ctx->policy->algorithm,
-			&keysize, &timing, NULL)) {
+	if (!genkeyargs(argc, argv, false, &flags, &ctx->policy->algorithm,
+	                &keysize, &timing, NULL)) {
 		return KNOT_EINVAL;
 	}
 
-	// open file
-	int fd = open(import_file, O_RDONLY, 0);
-	if (fd == -1) {
-		return knot_map_errno();
-	}
-
-	// determine size
-	off_t fsize = lseek(fd, 0, SEEK_END);
-	if (fsize == -1) {
-		close(fd);
-		return knot_map_errno();
-	}
-	if (lseek(fd, 0, SEEK_SET) == -1) {
-		close(fd);
-		return knot_map_errno();
-	}
+	normalize_generate_flags(&flags);
 
 	dnssec_key_t *key = NULL;
 	char *keyid = NULL;
+	int ret = KNOT_EOK;
 
-	// alloc memory
-	dnssec_binary_t pem = { 0 };
-	int ret = dnssec_binary_alloc(&pem, fsize);
-	if (ret != DNSSEC_EOK) {
+	if (backend == KEYSTORE_BACKEND_PEM) {
+		// open file
+		int fd = open(param, O_RDONLY, 0);
+		if (fd == -1) {
+			return knot_map_errno();
+		}
+
+		// determine size
+		off_t fsize = lseek(fd, 0, SEEK_END);
+		if (fsize == -1) {
+			close(fd);
+			return knot_map_errno();
+		}
+		if (lseek(fd, 0, SEEK_SET) == -1) {
+			close(fd);
+			return knot_map_errno();
+		}
+
+		// alloc memory
+		dnssec_binary_t pem = { 0 };
+		ret = dnssec_binary_alloc(&pem, fsize);
+		if (ret != DNSSEC_EOK) {
+			close(fd);
+			goto fail;
+		}
+
+		// read pem
+		ssize_t read_count = read(fd, pem.data, pem.size);
 		close(fd);
-		goto fail;
-	}
+		if (read_count == -1) {
+			dnssec_binary_free(&pem);
+			ret = knot_map_errno();
+			goto fail;
+		}
 
-	// read pem
-	ssize_t read_count = read(fd, pem.data, pem.size);
-	close(fd);
-	if (read_count == -1) {
+		// put pem to kesytore
+		ret = dnssec_keystore_import(ctx->keystore, &pem, &keyid);
 		dnssec_binary_free(&pem);
-		ret = knot_map_errno();
-		goto fail;
-	}
-
-	// put pem to kesytore
-	ret = dnssec_keystore_import(ctx->keystore, &pem, &keyid);
-	dnssec_binary_free(&pem);
-	if (ret != DNSSEC_EOK) {
-		goto fail;
+		if (ret != DNSSEC_EOK) {
+			goto fail;
+		}
+	} else {
+		assert(backend == KEYSTORE_BACKEND_PKCS11);
+		keyid = strdup(param);
 	}
 
 	// create dnssec key
@@ -439,7 +455,7 @@ int keymgr_import_pem(kdnssec_ctx_t *ctx, const char *import_file, int argc, cha
 	if (ret != DNSSEC_EOK) {
 		goto fail;
 	}
-	dnssec_key_set_flags(key, dnskey_flags(isksk));
+	dnssec_key_set_flags(key, dnskey_flags(flags & DNSKEY_GENERATE_SEP_ON));
 	dnssec_key_set_algorithm(key, ctx->policy->algorithm);
 
 	// fill key structure from keystore (incl. pubkey from privkey computation)
@@ -457,8 +473,8 @@ int keymgr_import_pem(kdnssec_ctx_t *ctx, const char *import_file, int argc, cha
 	kkey->id = keyid;
 	kkey->key = key;
 	kkey->timing = timing;
-	kkey->is_ksk = isksk;
-	kkey->is_zsk = iszsk;
+	kkey->is_ksk = (flags & DNSKEY_GENERATE_KSK);
+	kkey->is_zsk = (flags & DNSKEY_GENERATE_ZSK);
 
 	// append to zone
 	ret = kasp_zone_append(ctx->zone, kkey);
@@ -477,6 +493,16 @@ fail:
 	return knot_error_from_libdnssec(ret);
 }
 
+int keymgr_import_pem(kdnssec_ctx_t *ctx, const char *import_file, int argc, char *argv[])
+{
+	return import_key(ctx, KEYSTORE_BACKEND_PEM, import_file, argc, argv);
+}
+
+int keymgr_import_pkcs11(kdnssec_ctx_t *ctx, const char *key_id, int argc, char *argv[])
+{
+	return import_key(ctx, KEYSTORE_BACKEND_PKCS11, key_id, argc, argv);
+}
+
 static void print_tsig(dnssec_tsig_algorithm_t mac, const char *name,
 		       const dnssec_binary_t *secret)
 {
@@ -601,7 +627,7 @@ int keymgr_foreign_key_id(char *argv[], knot_dname_t **key_zone, char **key_id)
 	if (*key_zone == NULL) {
 		return KNOT_ENOMEM;
 	}
-	(void)knot_dname_to_lower(*key_zone);
+	knot_dname_to_lower(*key_zone);
 
 	kdnssec_ctx_t kctx = { 0 };
 	int ret = kdnssec_ctx_init(conf(), &kctx, *key_zone, NULL);
@@ -626,9 +652,12 @@ int keymgr_foreign_key_id(char *argv[], knot_dname_t **key_zone, char **key_id)
 int keymgr_set_timing(knot_kasp_key_t *key, int argc, char *argv[])
 {
 	knot_kasp_key_timing_t temp = key->timing;
+	kdnssec_generate_flags_t flags = ((key->is_ksk ? DNSKEY_GENERATE_KSK : 0) | (key->is_zsk ? DNSKEY_GENERATE_ZSK : 0));
 
-	if (genkeyargs(argc, argv, true, NULL, NULL, NULL, NULL, &temp, NULL)) {
+	if (genkeyargs(argc, argv, true, &flags, NULL, NULL, &temp, NULL)) {
 		key->timing = temp;
+		key->is_ksk = (flags & DNSKEY_GENERATE_KSK);
+		key->is_zsk = (flags & DNSKEY_GENERATE_ZSK);
 		return KNOT_EOK;
 	}
 	return KNOT_EINVAL;
@@ -649,10 +678,10 @@ int keymgr_list_keys(kdnssec_ctx_t *ctx, knot_time_print_t format)
 {
 	for (size_t i = 0; i < ctx->zone->num_keys; i++) {
 		knot_kasp_key_t *key = &ctx->zone->keys[i];
-		printf("%s ksk=%s zsk=%s tag=%05d algorithm=%d public-only=%s ", key->id,
-		       (key->is_ksk ? "yes" : "no "), (key->is_zsk ? "yes" : "no"),
+		printf("%s ksk=%s zsk=%s tag=%05d algorithm=%-2d size=%-4u public-only=%s ", key->id,
+		       (key->is_ksk ? "yes" : "no "), (key->is_zsk ? "yes" : "no "),
 		       dnssec_key_get_keytag(key->key), (int)dnssec_key_get_algorithm(key->key),
-		       (key->is_pub_only ? "yes" : "no "));
+		       dnssec_key_get_size(key->key), (key->is_pub_only ? "yes" : "no "));
 		print_timer("created",       key->timing.created,        format, ' ');
 		print_timer("pre-active",    key->timing.pre_active,     format, ' ');
 		print_timer("publish",       key->timing.publish,        format, ' ');
