@@ -518,12 +518,12 @@ static int zone_exec(cmd_args_t *args, int (*fcn)(const knot_dname_t *, void *),
 		uint8_t id[KNOT_DNAME_MAXLEN];
 
 		for (int i = 0; i < args->argc; i++) {
-			if (knot_dname_from_str(id, args->argv[i], sizeof(id)) == NULL ||
-			    knot_dname_to_lower(id) != KNOT_EOK) {
+			if (knot_dname_from_str(id, args->argv[i], sizeof(id)) == NULL) {
 				log_zone_str_error(args->argv[i], "invalid name");
 				failed = true;
 				continue;
 			}
+			knot_dname_to_lower(id);
 
 			if (!conf_rawid_exists(conf(), C_ZONE, id, knot_dname_size(id))) {
 				log_zone_error(id, "%s", knot_strerror(KNOT_ENOZONE));
@@ -558,7 +558,7 @@ static int zone_check(const knot_dname_t *dname, void *data)
 	zone_contents_t *contents;
 	int ret = zone_load_contents(conf(), dname, &contents);
 	if (ret == KNOT_EOK) {
-		zone_contents_deep_free(&contents);
+		zone_contents_deep_free(contents);
 	}
 	return ret;
 }
@@ -707,6 +707,7 @@ const filter_desc_t zone_purge_filters[MAX_FILTERS] = {
 	{ "+zonefile", CTL_FILTER_PURGE_ZONEFILE },
 	{ "+journal",  CTL_FILTER_PURGE_JOURNAL },
 	{ "+kaspdb",   CTL_FILTER_PURGE_KASPDB },
+	{ "+orphan",   CTL_FILTER_PURGE_ORPHAN },
 };
 
 const filter_desc_t null_filter = { 0 };
@@ -1130,7 +1131,7 @@ static const cmd_help_t cmd_help_table[] = {
 	{ CMD_ZONE_GET,        "<zone> [<owner> [<type>]]",              "Get zone data within the transaction." },
 	{ CMD_ZONE_SET,        "<zone>  <owner> [<ttl>] <type> <rdata>", "Add zone record within the transaction." },
 	{ CMD_ZONE_UNSET,      "<zone>  <owner> [<type> [<rdata>]]",     "Remove zone data within the transaction." },
-	{ CMD_ZONE_PURGE,      "<zone>... [<filter>...]",                "Purge zone data, file, journal, and timers." },
+	{ CMD_ZONE_PURGE,      "<zone>... [<filter>...]",                "Purge zone data, zone file, journal, timers, and KASP data." },
 	{ CMD_ZONE_STATS,      "<zone> [<module>[.<counter>]]",          "Show zone statistics counter(s)."},
 	{ CMD_ZONE_STATUS,     "<zone> [<filter>...]",                   "Show the zone status." },
 	{ "",                  "",                                       "" },
